@@ -1,4 +1,4 @@
-/* Mixmaster version 2.9  --  (C) 1999 - 2003 Anonymizer Inc. and others.
+/* Mixmaster version 3.0  --  (C) 1999 - 2006 Anonymizer Inc. and others.
 
    Mixmaster may be redistributed and modified under certain conditions.
    This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF
@@ -6,7 +6,7 @@
    details.
 
    Dynamically allocated buffers
-   $Id: buffers.c 665 2003-11-09 01:47:32Z rabbi $ */
+   $Id: buffers.c 934 2006-06-24 13:40:39Z rabbi $ */
 
 
 #include "mix3.h"
@@ -725,7 +725,7 @@ int bufifind(BUFFER *b, char *k) {
 
 int bufiright(BUFFER *b, char *k) {
   int l = strlen(k);
-  if (l >= b->length)
+  if (l <= b->length)
     return (strieq(b->data + b->length - l, k));
   return(0);
 }
