@@ -1,4 +1,4 @@
-/* Mixmaster version 2.9  --  (C) 1999 - 2003 Anonymizer Inc. and others.
+/* Mixmaster version 3.0  --  (C) 1999 - 2006 Anonymizer Inc. and others.
 
    Mixmaster may be redistributed and modified under certain conditions.
    This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF
@@ -6,7 +6,7 @@
    details.
 
    Prepare messages for remailer chain
-   $Id: chain.c 665 2003-11-09 01:47:32Z rabbi $ */
+   $Id: chain.c 934 2006-06-24 13:40:39Z rabbi $ */
 
 
 #include "mix3.h"
@@ -24,6 +24,82 @@ void clienterr(BUFFER *msgbuf, char *err)
     errlog(ERRORMSG, "%s\n", err);
 }
 
+void parse_badchains(int badchains[MAXREM][MAXREM], char *file, char *startindicator, REMAILER *remailer, int maxrem) {
+  int i,j;
+  FILE *list;
+  char line[LINELEN];
+
+  if (!badchains)
+    return;
+
+  for (i = 0; i < maxrem; i++ )
+    for (j = 0; j < maxrem; j++ )
+      badchains[i][j] = 0;
+  list = mix_openfile(TYPE2REL, "r");
+  if (list != NULL) {
+    while (fgets(line, sizeof(line), list) != NULL &&
+      !strleft(line, startindicator)) ;
+    while (fgets(line, sizeof(line), list) != NULL &&
+      strleft(line, "(")) {
+      char *left, *right, *tmp;
+      int lefti, righti;
+
+      left = line + 1;
+      while (*left == ' ')
+	left ++;
+
+      tmp = left + 1;
+      while (*tmp != ' ' && *tmp != '\0' && *tmp != ')')
+	tmp ++;
+      if (*tmp == '\0' || *tmp == ')')
+	/* parsing this line failed */
+	continue;
+      *tmp = '\0';
+
+      right = tmp+1;
+      while (*right == ' ')
+	right ++;
+      tmp = right + 1;
+      while (*tmp != ' ' && *tmp != '\0' && *tmp != ')')
+	tmp ++;
+      if (*tmp == '\0')
+	/* parsing this line failed */
+	continue;
+      *tmp = '\0';
+
+      lefti = -1;
+      righti = -1;
+      for (i = 1; i < maxrem; i++) {
+	if (strcmp(remailer[i].name, left) == 0)
+	  lefti = i;
+	if (strcmp(remailer[i].name, right) == 0)
+	  righti = i;
+      }
+      if (strcmp(left, "*") == 0)
+	lefti = 0;
+      if (strcmp(right, "*") == 0)
+	righti = 0;
+
+      if (lefti == -1 || righti == -1)
+	/* we don't know about one or both remailers */
+	continue;
+      badchains[lefti][righti] = 1;
+    }
+    fclose(list);
+    /* If some broken chain includes all remailers (*) mark it broken for
+     * every single remailer - this simplifies handling in other places */
+    for (i=1; i < maxrem; i++ ) {
+      if (badchains[0][i])
+	for (j=1; j < maxrem; j++ )
+	  badchains[j][i] = 1;
+      if (badchains[i][0])
+	for (j=1; j < maxrem; j++ )
+	  badchains[i][j] = 1;
+    }
+  }
+}
+
+
 int chain_select(int hop[], char *chainstr, int maxrem, REMAILER *remailer,
 		 int type, BUFFER *feedback)
 {
@@ -42,7 +118,7 @@ int chain_select(int hop[], char *chainstr, int maxrem, REMAILER *remailer,
   /* put the chain backwards: final hop is in hop[0] */
 
   for (i = chain->length; i >= 0; i--)
-    if (i == 0 || chain->data[i - 1] <= ' ' || chain->data[i - 1] == ','
+    if (i == 0 || chain->data[i - 1] == ','
 	|| chain->data[i - 1] == ';' || chain->data[i - 1] == ':') {
       for (j = i; isspace(chain->data[j]);)	/* ignore whitespace */
 	j++;
@@ -102,52 +178,74 @@ end:
   return len;
 }
 
-int chain_randfinal(int type, REMAILER *remailer, int maxrem, int rtype)
+int chain_randfinal(int type, REMAILER *remailer, int badchains[MAXREM][MAXREM], int maxrem, int rtype, int chain[], int chainlen, int ignore_constraints_if_necessary)
 {
-  int num = 0;
+  int randavail;
   int i;
   int t;
+  int select[MAXREM];
+  int secondtolasthop = (chainlen <= 1 ? -1 : chain[1]);
+  int constraints_ignored = 0;
 
   t = rtype;
   if (rtype == 2)
     t = 1;
 
+start:
+  randavail = 0;
   /* select a random final hop */
   for (i = 1; i < maxrem; i++) {
-    if (((remailer[i].flags.mix && rtype == 0) ||
+    select[i] = 
+       ((remailer[i].flags.mix && rtype == 0) ||             /* remailer supports type */
 	 (remailer[i].flags.pgp && remailer[i].flags.ek && rtype == 1) ||
 	 (remailer[i].flags.newnym && rtype == 2)) &&
-	remailer[i].info[t].reliability >= 100 * RELFINAL &&
-	remailer[i].info[t].latency <= MAXLAT &&
-	(type == MSG_NULL || !remailer[i].flags.middle) &&
-	(remailer[i].flags.post || type != MSG_POST))
-      num++;
+	(remailer[i].info[t].reliability >= 100 * RELFINAL || constraints_ignored ) && /* remailer has sufficient reliability */
+	(remailer[i].info[t].latency <= MAXLAT || constraints_ignored ) &&             /* remailer has low enough latency */
+	(remailer[i].info[t].latency >= MINLAT || constraints_ignored ) &&             /* remailer has high enough latency */
+	(type == MSG_NULL || !remailer[i].flags.middle) &&   /* remailer is not middleman */
+	!remailer[i].flags.star_ex &&                        /* remailer is not excluded from random selection */
+	(remailer[i].flags.post || type != MSG_POST) &&      /* remailer supports post when this is a post */
+	((secondtolasthop == -1) || !badchains[secondtolasthop][i]);
+	                             /* we only have hop or the previous one can send to this (may be random) */
+    randavail += select[i];
   }
-  if (num == 0)
+
+  for (i = 1; i <= DISTANCE; i++)
+    if (i < chainlen && select[chain[i]] && chain[i]) {
+      select[chain[i]] = 0;
+      randavail--;
+    }
+
+  assert(randavail >= 0);
+  if (randavail == 0) {
+    if (ignore_constraints_if_necessary && !constraints_ignored) {
+      errlog(WARNING, "No reliable remailers. Ignoring for randhop\n");
+      constraints_ignored = 1;
+      goto start;
+    };
     i = -1;
-  else {
+  } else {
     do
-      i = rnd_number(maxrem) + 1;
-    while (!(((remailer[i].flags.mix && rtype == 0) ||
-	      (remailer[i].flags.pgp && remailer[i].flags.ek && rtype == 1) ||
-	      (remailer[i].flags.newnym && rtype == 2)) &&
-	     remailer[i].info[t].reliability >= 100 * RELFINAL &&
-	     remailer[i].info[t].latency <= MAXLAT &&
-	     (type == MSG_NULL || !remailer[i].flags.middle) &&
-	     (remailer[i].flags.post || type != MSG_POST)));
+      i = rnd_number(maxrem - 1) + 1;
+    while (!select[i]);
   }
   return (i);
 }
 
-int chain_rand(REMAILER *remailer, int maxrem,
-	       int thischain[], int chainlen, int t)
+int chain_rand(REMAILER *remailer, int badchains[MAXREM][MAXREM], int maxrem,
+	       int thischain[], int chainlen, int t, int ignore_constraints_if_necessary)
      /* set random chain. returns 0 if not random, 1 if random, -1 on error */
+/* t... 0 for mixmaster Type II
+ *      1 for cypherpunk Type I
+ */
 {
   int hop;
   int err = 0;
+  int constraints_ignored = 0;
 
   assert(t == 0 || t == 1);
 
+start:
   for (hop = 0; hop < chainlen; hop++)
     if (thischain[hop] == 0) {
       int select[MAXREM];
@@ -155,18 +253,35 @@ int chain_rand(REMAILER *remailer, int maxrem,
       int i;
 
       err = 1;
-      for (i = 1; i < maxrem; i++)
-	select[i] = ((remailer[i].flags.mix && t == 0) ||
-		     (remailer[i].flags.pgp && remailer[i].flags.ek && t == 1))
-	  && remailer[i].info[t].reliability >= 100 * MINREL &&
-	  remailer[i].info[t].latency <= MAXLAT,
-	  randavail += select[i];
+      if (hop > 0)
+	assert(thischain[hop-1]); /* we already should have chosen a remailer after this one */
+      for (i = 1; i < maxrem; i++) {
+	select[i] = ((remailer[i].flags.mix && t == 0) ||        /* remailer supports type */
+		     (remailer[i].flags.pgp && remailer[i].flags.ek && t == 1)) &&
+	  (remailer[i].info[t].reliability >= 100 * MINREL || constraints_ignored ) &&  /* remailer has sufficient reliability */
+	  (remailer[i].info[t].latency <= MAXLAT || constraints_ignored ) &&            /* remailer has low enough latency */
+	  (remailer[i].info[t].latency >= MINLAT || constraints_ignored ) &&            /* remailer has high enough latency */
+	  !remailer[i].flags.star_ex &&                          /* remailer is not excluded from random selection */
+	  !badchains[i][0] && !badchains[i][thischain[hop-1]] && /* remailer can send to the next one */
+	  (hop == chainlen-1 || !badchains[thischain[hop+1]][i]);
+	                           /* we are at the first hop or the previous one can send to this (may be random) */
+	randavail += select[i];
+      }
 
       for (i = hop - DISTANCE; i <= hop + DISTANCE; i++)
-	if (i >= 0 && i < chainlen && select[thischain[i]])
-	  select[thischain[i]] = 0, randavail--;
+	if (i >= 0 && i < chainlen && select[thischain[i]] && thischain[i]) {
+	  select[thischain[i]] = 0;
+	  randavail--;
+	}
 
+
+      assert(randavail >= 0);
       if (randavail < 1) {
+	if (ignore_constraints_if_necessary && !constraints_ignored) {
+	  errlog(WARNING, "No reliable remailers. Ignoring for randhop\n");
+	  constraints_ignored = 1;
+	  goto start;
+	};
 	err = -1;
 	goto end;
       }
@@ -181,7 +296,7 @@ end:
 int mix_encrypt(int type, BUFFER *message, char *chainstr, int numcopies,
 		BUFFER *chainlist)
 {
-  return (mix2_encrypt(type, message, chainstr, numcopies, chainlist));
+  return (mix2_encrypt(type, message, chainstr, numcopies, 0, chainlist));
 }
 
 /* float chain_reliablity(char *chain, int chaintype,
@@ -215,11 +330,13 @@ float chain_reliability(char *chain, int chaintype,
   int error = 0;
   int maxrem;
   int i;
+  int previous = -1;
   REMAILER remailer[MAXREM];
+  int badchains[MAXREM][MAXREM];
 
   /* chaintype 0=mix 1=ek 2=newnym */
   assert((chaintype == 0) || (chaintype == 1));
-  maxrem = (chaintype == 0 ? mix2_rlist(remailer) : t1_rlist(remailer));
+  maxrem = (chaintype == 0 ? mix2_rlist(remailer, badchains) : t1_rlist(remailer, badchains));
 
   /* Dissect chain */
   name_start = chain;
@@ -235,10 +352,15 @@ float chain_reliability(char *chain, int chaintype,
     for(i=0;
 	(i < maxrem) && (strcmp(remailer[i].name, remailer_name) != 0);
 	i++);
-    if(!strcmp(remailer[i].name, remailer_name)) /* Found it! */
+    if(!strcmp(remailer[i].name, remailer_name)) { /* Found it! */
       acc_reliability *=
 	((float) remailer[i].info[chaintype].reliability) / 10000;
-    else
+      if (previous != -1) {
+	if (badchains[previous][i] || badchains[0][i])
+	  acc_reliability = 0;
+      }
+      previous = i;
+    } else
       error = 1; /* Did not find this remailer. We can't calculate
 		    the reliablity for the whole chain. */
     name_start = name_end+sizeof(char);
