@@ -1,4 +1,4 @@
-/* Mixmaster version 2.9  --  (C) 1999 - 2003 Anonymizer Inc. and others.
+/* Mixmaster version 3.0  --  (C) 1999 - 2006 Anonymizer Inc. and others.
 
    Mixmaster may be redistributed and modified under certain conditions.
    This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF
@@ -6,28 +6,37 @@
    details.
 
    Encrypt message for Cypherpunk remailer chain
-   $Id: chain1.c 665 2003-11-09 01:47:32Z rabbi $ */
+   $Id: chain1.c 934 2006-06-24 13:40:39Z rabbi $ */
+
 
-#include "mix.h"
 #include "mix3.h"
-#ifdef USE_PGP
-#	include "pgp.h"
-#endif
+#include "pgp.h"
 #include <string.h>
 #include <ctype.h>
 
 #define N(X) (isdigit(X) ? (X)-'0' : 0)
 
-int t1_rlist(REMAILER remailer[])
+int t1_rlist(REMAILER remailer[], int badchains[MAXREM][MAXREM])
 {
-  FILE *list;
+  FILE *list, *excl;
   int i, listed = 0;
   int n = 0;
   char line[2 * LINELEN], l2[LINELEN], name[LINELEN], *flags;
+  BUFFER *starex;
+
+  starex = buf_new();
+  excl = mix_openfile(STAREX, "r");
+  if (excl != NULL) {
+    buf_read(starex, excl);
+    fclose(excl);
+  }
 
   list = mix_openfile(TYPE1LIST, "r");
-  if (list == NULL)
+  if (list == NULL) {
+    buf_free(starex);
     return (-1);
+  }
+
   while (fgets(line, sizeof(line), list) != NULL && n < MAXREM) {
     if (strleft(line, "$remailer") &&
 	strchr(line, '<') && strchr(line, '>') &&
@@ -73,7 +82,8 @@ int t1_rlist(REMAILER remailer[])
       remailer[i].info[1].reliability = 0;
       remailer[i].info[1].latency = 0;
       remailer[i].info[1].history[0] = '\0';
-    }
+      remailer[i].flags.star_ex = bufifind(starex, name);
+   }
     if (strleft(line,
 		"-----------------------------------------------------------------------"))
       break;
@@ -97,6 +107,7 @@ int t1_rlist(REMAILER remailer[])
 	}
   }
   fclose(list);
+  parse_badchains(badchains, TYPE1LIST, "Broken type-I remailer chains", remailer, n);
   if (listed < 4)		/* we have no valid reliability info */
     for (i = 1; i < n; i++)
       remailer[i].info[1].reliability = 10000;
@@ -104,6 +115,7 @@ int t1_rlist(REMAILER remailer[])
 #ifdef USE_PGP
   pgp_rlist(remailer, n);
 #endif /* USE_PGP */
+  buf_free(starex);
   return (n);
 }
 
@@ -126,6 +138,7 @@ int t1_encrypt(int type, BUFFER *message, char *chainstr, int latency,
 {
   BUFFER *b, *rem, *dest, *line, *field, *content;
   REMAILER remailer[MAXREM];
+  int badchains[MAXREM][MAXREM];
   int maxrem, chainlen = 0;
   int chain[20];
   int hop;
@@ -139,7 +152,7 @@ int t1_encrypt(int type, BUFFER *message, char *chainstr, int latency,
   field = buf_new();
   content = buf_new();
 
-  maxrem = t1_rlist(remailer);
+  maxrem = t1_rlist(remailer, badchains);
   if (maxrem < 1) {
     clienterr(feedback, "No remailer list!");
     err = -1;
@@ -155,14 +168,14 @@ int t1_encrypt(int type, BUFFER *message, char *chainstr, int latency,
     goto end;
   }
   if (chain[0] == 0)
-    chain[0] = chain_randfinal(type, remailer, maxrem, 1);
+    chain[0] = chain_randfinal(type, remailer, badchains, maxrem, 1, chain, chainlen, 0);
 
   if (chain[0] == -1) {
     clienterr(feedback, "Invalid remailer chain!");
     err = -1;
     goto end;
   }
-  if (chain_rand(remailer, maxrem, chain, chainlen, 1) == -1) {
+  if (chain_rand(remailer, badchains, maxrem, chain, chainlen, 1, 0) == -1) {
     clienterr(feedback, "No reliable remailers!");
     err = -1;
     goto end;
