@@ -1,4 +1,4 @@
-/* Mixmaster version 2.9  --  (C) 1999 - 2003 Anonymizer Inc. and others.
+/* Mixmaster version 3.0  --  (C) 1999 - 2006 Anonymizer Inc. and others.
 
    Mixmaster may be redistributed and modified under certain conditions.
    This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF
@@ -6,7 +6,7 @@
    details.
 
    Buffer compression (interface to zlib)
-   $Id: compress.c 665 2003-11-09 01:47:32Z rabbi $ */
+   $Id: compress.c 934 2006-06-24 13:40:39Z rabbi $ */
 
 
 #include "mix3.h"
@@ -115,7 +115,8 @@ int buf_zip(BUFFER *out, BUFFER *in, int bits)
     goto end;
 
   outstart = out->length;
-  buf_append(out, NULL, in->length);	/* try to fit all into one chunk */
+  /* 12 is overhead, 1.01 is maximum expansion, and 1 is there to force a round-up */
+  buf_append(out, NULL, (int)13+in->length*1.01); /* fit it in one chunk */
 
   s.next_in = in->data;
   s.avail_in = in->length;
@@ -127,6 +128,7 @@ int buf_zip(BUFFER *out, BUFFER *in, int bits)
     out->length -= s.avail_out;
     if (err != Z_OK)
       break;
+    errlog(ERRORMSG, "Compressed data did not fit in one chunk.\n");
     buf_append(out, NULL, BUFSIZE);
   }
   if (deflateEnd(&s) != Z_OK || err != Z_STREAM_END)
