@@ -1,4 +1,4 @@
-/* Mixmaster version 2.9  --  (C) 1999 - 2003 Anonymizer Inc. and others.
+/* Mixmaster version 3.0  --  (C) 1999 - 2008 Anonymizer Inc. and others.
 
    Mixmaster may be redistributed and modified under certain conditions.
    This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF
@@ -6,12 +6,15 @@
    details.
 
    Configuration
-   $Id: config.h 665 2003-11-09 01:47:32Z rabbi $ */
+   $Id: config.h 973 2008-03-03 16:55:38Z rabbi $ */
 
 
 #ifndef _CONFIG_H
 #define _CONFIG_H
 #include "version.h"
+#ifndef WIN32
+#include "sys/param.h"
+#endif /* WIN32 */
 
 /* Disclaimer to be inserted in all anonymous messages: */
 #define DISCLAIMER \
@@ -41,21 +44,7 @@
 #define DEFLTENTITY "text/plain; charset=" MIMECHARSET
 #endif
 
-/** Libraries and library functions **********************************/
-
-/* Use the OpenSSL crypto library (required) */
-#define USE_OPENSSL
-/* Use the RSA cryptosystem? */
-#define USE_RSA
-/* Use IDEA algorithm? (See file idea.txt) */
-/* #define USE_IDEA */
-/* Use AES algorithm? - should be handled by Install script setting compiler option -DUSE_AES */
-/* #define USE_AES */
-/* Support the OpenPGP message format? */
-#define USE_PGP
-
- /* the following are defined in the Makefile */
-#if 0
+#ifdef WIN32
 /* Use the PCRE regular expression library for destination blocking? */
 #define USE_PCRE
 /* Use zlib for compression? */
@@ -66,7 +55,9 @@
 #define USE_WINGUI
 /* Use sockets to deliver mail */
 #define USE_SOCK
-#endif
+/* Compile in Win32 service support */
+#define WIN32SERVICE
+#endif /* WIN32 */
 
 /** System dependencies **********************************************/
 /* Macros: UNIX  for Unix-style systems
@@ -91,29 +82,17 @@
 #endif
 
 #ifdef UNIX
-#define HAVE_UNAME
-#define HAVE_GECOS
 #define DEV_URANDOM "/dev/urandom"
 #ifdef __OpenBSD__
-#define DEV_RANDOM "/dev/srandom"
+#define DEV_RANDOM "/dev/arandom"
 #else
 #define DEV_RANDOM "/dev/random"
 #endif
 #endif
 
-#if defined(POSIX) || defined(USE_SOCK)
-#define HAVE_GETHOSTNAME
-#endif
-
 #ifdef POSIX
-#define HAVE_TERMIOS
-/* not a POSIX function, but avaiable on virtually all Unix systems */
-#define HAVE_GETTIMEOFDAY
-#endif
-
-#ifdef linux
-#define HAVE_GETDOMAINNAME
-#endif
+# define HAVE_TERMIOS
+#endif /* POSIX */
 
 #ifdef MSDOS
 #define SHORTNAMES
@@ -130,19 +109,72 @@
 #if defined(WIN32) && !defined(_USRDLL)
 #define DLLIMPORT __declspec(dllimport)
 #else
-#if defined(WIN32) && defined(MIXLIB_EXPORTS)
-#define DLLIMPORT __declspec(dllexport)
-#else
 #define DLLIMPORT
 #endif
+
+/* This block includes the config.h created by autoconf/configure.
+ * Eventually this old config.h stuff should be merged with the autoconf
+ * stuff perhaps. */
+#ifdef HAVE_CONFIG_H
+# include "../config.h"
+#else /* End of HAVE_CONFIG_H */
+
+/* Setup for stuff that happens when autoconf isn't run.  This should be 
+ * removed once we finally nuke that damn Install script. */
+
+/** Libraries and library functions **********************************/
+
+/* Use the OpenSSL crypto library (required) */
+#define USE_OPENSSL
+/* Use IDEA algorithm? (See file idea.txt) */
+/* #define USE_IDEA */
+/* Use AES algorithm? - should be handled by Install script setting compiler option -DUSE_AES */
+/* #define USE_AES */
+/* Support the OpenPGP message format? */
+#define USE_PGP
+
+#ifdef UNIX
+# define HAVE_UNAME
+# define HAVE_GECOS
 #endif
 
+#if defined(POSIX) || defined(USE_SOCK)
+# define HAVE_GETHOSTNAME
+#endif
+
+#ifdef POSIX
+/* not a POSIX function, but avaiable on virtually all Unix systems */
+# define HAVE_GETTIMEOFDAY
+#endif
+
+#ifdef linux
+# define HAVE_GETDOMAINNAME
+#endif
+
+#ifdef WIN32
+# ifdef _MSC_VER
+#pragma warning(disable: 4018) /* signed/unsigned mismatch */
+#pragma warning(disable: 4761) /* integral size mismatch */
+# endif
+#endif
+
+
+#endif /* End of not HAVE_CONFIG_H */
+
 /** Constants *********************************************************/
 
 /* Give up if a file is larger than BUFFER_MAX bytes: */
 /* #define BUFFER_MAX 64*1024*1024 */
 
+#ifdef MAXPATHLEN
+#define PATHMAX MAXPATHLEN
+#else /* MAXPATHLEN */
+#ifdef _MSC
+#define PATHMAX MAX_PATH
+#else /* _MSC */
 #define PATHMAX 512
+#endif /* not _MSC */
+#endif /* not MAXPATHLEN */
 #define LINELEN 128
 #define BUFSIZE 4096
 
@@ -156,12 +188,21 @@
 
 /** file names ********************************************************/
 
+#ifdef WIN32
+#define DEFAULT_MIXCONF "mix.cfg"  /* change to mix.ini eventually  */
+#else
 #define DEFAULT_MIXCONF "mix.cfg"      /* mixmaster configuration file */
+#endif
 #define DEFAULT_DISCLAIMFILE "disclaim.txt"
 #define DEFAULT_FROMDSCLFILE "fromdscl.txt"
 #define DEFAULT_MSGFOOTERFILE "footer.txt"
+#ifdef WIN32
+#define DEFAULT_POP3CONF "pop3.cfg"  /* change to pop3.ini eventually */
+#else
 #define DEFAULT_POP3CONF "pop3.cfg"
+#endif
 #define DEFAULT_HELPFILE "help.txt"
+#define DEFAULT_REQUESTDIR "requests"
 #define DEFAULT_ABUSEFILE "abuse.txt"
 #define DEFAULT_REPLYFILE "reply.txt"
 #define DEFAULT_USAGEFILE "usage.txt"
@@ -177,28 +218,30 @@
 #define DEFAULT_PUBRING "pubring.mix"
 #define DEFAULT_IDLOG "id.log"
 #define DEFAULT_STATS "stats.log"
+#define DEFAULT_PGPMAXCOUNT "pgpmaxcount.log"
 /* To enable multiple dest.blk files, edit the following line. */
 /* Filenames must be seperated by one space.                   */
 #define DEFAULT_DESTBLOCK "dest.blk rab.blk"
 #define DEFAULT_DESTALLOW "dest.alw"
+#define DEFAULT_DESTALLOW2 "dest.alw.nonpublished"
 #define DEFAULT_SOURCEBLOCK "source.blk"
 #define DEFAULT_HDRFILTER "header.blk"
 #define DEFAULT_REGULAR "time.log"
 #define DEFAULT_POOL "pool"            /* remailer pool subdirectory */
 #define DEFAULT_TYPE1LIST "rlist.txt"
 #define DEFAULT_TYPE2REL "mlist.txt"
-#ifdef SHORTNAMES
-#define DEFAULT_TYPE2LIST "type2.lis"
-#else
-#define DEFAULT_TYPE2LIST "type2.list"
-#endif
-#define DEFAULT_PIDFILE "mix.pid"
+#define DEFAULT_PIDFILE "mixmaster.pid"
+#define DEFAULT_STATSSRC "stats-src.txt"
 
 #define DEFAULT_PGPREMPUBRING "pubring.pgp"
 #define DEFAULT_PGPREMPUBASC "pubring.asc"
 #define DEFAULT_PGPREMSECRING "secring.pgp"
 #define DEFAULT_NYMSECRING "nymsec.pgp"
 #define DEFAULT_NYMDB "secrets.mix"
+#define DEFAULT_STAREX "starex.txt"
+#define DEFAULT_ALLPINGERSURL "http://www.noreply.org/allpingers/allpingers.txt"
+#define DEFAULT_ALLPINGERSFILE "allpingers.txt"
+#define DEFAULT_WGET "wget"
 
 DLLIMPORT extern char MIXCONF[];
 extern char DISCLAIMFILE[];
@@ -206,6 +249,7 @@ extern char FROMDSCLFILE[];
 extern char MSGFOOTERFILE[];
 extern char POP3CONF[];
 extern char HELPFILE[];
+extern char REQUESTDIR[];
 extern char ABUSEFILE[];
 extern char REPLYFILE[];
 extern char USAGEFILE[];
@@ -221,16 +265,18 @@ extern char SECRING[];
 extern char PUBRING[];
 extern char IDLOG[];
 extern char STATS[];
+extern char PGPMAXCOUNT[];
 extern char DESTBLOCK[];
 extern char DESTALLOW[];
+extern char DESTALLOW2[];
 extern char SOURCEBLOCK[];
 extern char HDRFILTER[];
 extern char REGULAR[];
 extern char POOL[];
 extern char TYPE1LIST[];
 extern char TYPE2REL[];
-extern char TYPE2LIST[];
 extern char PIDFILE[];
+extern char STAREX[];
 
 extern char PGPREMPUBRING[];
 extern char PGPREMPUBASC[];
@@ -273,8 +319,10 @@ int REMIX;
 int REPGP;
 extern char MIXDIR[];
 extern char POOLDIR[];
+extern char EXTFLAGS[];
 extern char SENDMAIL[];
 extern char SENDANONMAIL[];
+extern char PRECEDENCE[];
 extern char SMTPRELAY[];
 extern char SMTPUSERNAME[];
 extern char SMTPPASSWORD[];
@@ -296,9 +344,12 @@ extern char HELONAME[];
 extern char ENVFROM[];
 extern char SHORTNAME[];
 extern int POOLSIZE;
-extern int RATE;
+DLLIMPORT extern int RATE;
+extern int INDUMMYP;
+extern int OUTDUMMYP;
 extern int MIDDLEMAN;
 extern int AUTOBLOCK;
+extern int STATSDETAILS;
 extern char FORWARDTO[];
 extern int SIZELIMIT;
 extern int INFLATEMAX;
@@ -308,14 +359,18 @@ extern int LISTSUPPORTED;
 extern long PACKETEXP;
 extern long IDEXP;
 DLLIMPORT extern int VERBOSE;
-extern long SENDPOOLTIME;
+DLLIMPORT extern long SENDPOOLTIME;
 extern long MAILINTIME;
+extern long KEYLIFETIME;
+extern long KEYOVERLAPPERIOD;
+extern long KEYGRACEPERIOD;
 extern int NUMCOPIES;
 extern char CHAIN[];
 extern int DISTANCE;
 extern int MINREL;
 extern int RELFINAL;
 extern long MAXLAT;
+extern long MINLAT;
 DLLIMPORT extern char PGPPUBRING[];
 DLLIMPORT extern char PGPSECRING[];
 DLLIMPORT extern char PASSPHRASE[];
@@ -330,5 +385,19 @@ extern char MAILUSAGE[];
 extern char MAILANON[];
 extern char MAILERROR[];
 extern char MAILBOUNCE[];
+DLLIMPORT extern int CLIENTAUTOFLUSH;
+extern int MAXRECIPIENTS;
+extern long TIMESKEW_FORWARD;
+extern long TIMESKEW_BACK;
+extern int TEMP_FAIL;
+extern char ALLPINGERSURL[];
+extern char ALLPINGERSFILE[];
+
+extern char WGET[];
+extern char STATSSRC[];
+extern int STATSAUTOUPDATE;
+extern long STATSINTERVAL;
+
+DLLIMPORT extern char ENTEREDPASSPHRASE[LINELEN];
 
 #endif
