@@ -1,4 +1,4 @@
-/* Mixmaster version 2.9  --  (C) 1999 - 2003 Anonymizer Inc. and others.
+/* Mixmaster version 3.0  --  (C) 1999 - 2006 Anonymizer Inc. and others.
 
    Mixmaster may be redistributed and modified under certain conditions.
    This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF
@@ -6,13 +6,14 @@
    details.
 
    Interface to cryptographic library
-   $Id: crypto.c 665 2003-11-09 01:47:32Z rabbi $ */
+   $Id: crypto.c 934 2006-06-24 13:40:39Z rabbi $ */
 
 
 #include "mix3.h"
 #include "crypto.h"
 #include <assert.h>
 #include <string.h>
+#include <time.h>
 
 #ifdef USE_OPENSSL
 int digestmem_md5(byte *b, int n, BUFFER *md)
@@ -72,7 +73,6 @@ int digest_rmd160(BUFFER *b, BUFFER *md)
   return (digestmem_rmd160(b->data, b->length, md));
 }
 
-#ifdef USE_RSA
 #define MAX_RSA_MODULUS_LEN 128
 
 static int read_seckey(BUFFER *buf, SECKEY *key, const byte id[])
@@ -298,13 +298,24 @@ int v2createkey(void)
   ek = buf_new();
   iv = buf_new();
 
-  errlog(NOTICE, "Generating Mix RSA key.\n");
+  errlog(NOTICE, "Generating RSA key.\n");
   k = RSA_generate_key(1024, 65537, NULL, NULL);
   err = write_seckey(b, k, keyid);
   RSA_free(k);
   if (err == 0) {
     f = mix_openfile(SECRING, "a");
     if (f != NULL) {
+      time_t now = time(NULL);
+      struct tm *gt;
+      gt = gmtime(&now);
+      strftime(line, LINELEN, "%Y-%m-%d", gt);
+      fprintf(f, "%s\nCreated: %s\n", begin_key, line);
+      if (KEYLIFETIME) {
+	now += KEYLIFETIME;
+	gt = gmtime(&now);
+	strftime(line, LINELEN, "%Y-%m-%d", gt);
+	fprintf(f, "Expires: %s\n", line);
+      }
       id_encode(keyid, line);
       buf_appends(ek, PASSPHRASE);
       digest_md5(ek, ek);
@@ -312,7 +323,7 @@ int v2createkey(void)
       buf_crypt(b, ek, iv, ENCRYPT);
       encode(b, 40);
       encode(iv, 0);
-      fprintf(f, "%s\n%s\n0\n%s\n", begin_key, line, iv->data);
+      fprintf(f, "%s\n0\n%s\n", line, iv->data);
       buf_write(b, f);
       fprintf(f, "%s\n\n", end_key);
       fclose(f);
@@ -370,8 +381,6 @@ int pk_encrypt(BUFFER *in, BUFFER *keybuf)
   RSA_free(key);
   return (err);
 }
-#endif /* USE_RSA */
-
 int buf_crypt(BUFFER *buf, BUFFER *key, BUFFER *iv, int enc)
 {
   des_key_schedule ks1;
