@@ -1,4 +1,4 @@
-/* Mixmaster version 2.9  --  (C) 1999 - 2003 Anonymizer Inc. and others.
+/* Mixmaster version 3.0  --  (C) 1999 - 2006 Anonymizer Inc. and others.
 
    Mixmaster may be redistributed and modified under certain conditions.
    This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF
@@ -6,7 +6,7 @@
    details.
 
    Interface to cryptographic library
-   $Id: crypto.h 665 2003-11-09 01:47:32Z rabbi $ */
+   $Id: crypto.h 934 2006-06-24 13:40:39Z rabbi $ */
 
 
 #ifndef _CRYPTO_H
@@ -26,9 +26,7 @@
 #include <openssl/bn.h>
 #include <openssl/dh.h>
 #include <openssl/dsa.h>
-#ifdef USE_RSA
 #include <openssl/rsa.h>
-#endif /* USE_RSA */
 #ifdef USE_IDEA
 #include <openssl/idea.h>
 #endif /* USE_IDEA */
@@ -38,10 +36,8 @@
 #include <openssl/cast.h>
 #include <openssl/rand.h>
 
-#ifdef USE_RSA
 typedef RSA PUBKEY;
 typedef RSA SECKEY;
-#endif /* USE_RSA */
 
 #else /* end of USE_OPENSSL */
 /* #error "No crypto library." */
