@@ -1,4 +1,4 @@
-/* Mixmaster version 2.9  --  (C) 1999 - 2003 Anonymizer Inc. and others.
+/* Mixmaster version 3.0  --  (C) 1999 - 2006 Anonymizer Inc. and others.
 
    Mixmaster may be redistributed and modified under certain conditions.
    This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF
@@ -6,7 +6,7 @@
    details.
 
    Socket-based mail transport services
-   $Id: mail.c 665 2003-11-09 01:47:32Z rabbi $ */
+   $Id: mail.c 934 2006-06-24 13:40:39Z rabbi $ */
 
 
 #include "mix3.h"
@@ -97,15 +97,65 @@ int sendmail_loop(BUFFER *message, char *from, BUFFER *address)
   return(err);
 }
 
+/* Returns true if more than one of the recipients in the
+ * rcpt buffer is a remailer
+ */
+int has_more_than_one_remailer(BUFFER *rcpts)
+{
+  BUFFER *newlinelist;
+  BUFFER *line;
+  int remailers = 0;
+  REMAILER type1[MAXREM];
+  REMAILER type2[MAXREM];
+  int num1;
+  int num2;
+  int i;
+
+  newlinelist = buf_new();
+  line = buf_new();
+
+  num1 = t1_rlist(type1, NULL);
+  num2 = mix2_rlist(type2, NULL);
+
+  rfc822_addr(rcpts, newlinelist);
+
+  while (buf_getline(newlinelist, line) != -1) {
+    int found = 0;
+    printf("%s\n", line->data);
+
+    for (i = 0; i < num2; i++)
+      if (strcmp(type2[i].addr, line->data) == 0) {
+	found = 1;
+	break;
+      }
+    if (!found)
+      for (i = 0; i < num1; i++)
+	if (strcmp(type1[i].addr, line->data) == 0) {
+	  found = 1;
+	  break;
+	}
+    if (found)
+      remailers++;
+  }
+  printf("found %d\n", remailers);
+
+  buf_free(newlinelist);
+  buf_free(line);
+
+  return(remailers > 1);
+}
+
 int sendmail(BUFFER *message, char *from, BUFFER *address)
 {
   /* returns: 0: ok  1: problem, try again  -1: failed */
 
-  BUFFER *head, *block;
+  BUFFER *head, *block, *rcpt;
   FILE *f;
   int err = -1;
+  int rcpt_cnt;
 
   head = buf_new();
+  rcpt = buf_new();
 
   block = readdestblk( );
   if ( !block ) block = buf_new( );
@@ -122,9 +172,59 @@ int sendmail(BUFFER *message, char *from, BUFFER *address)
   if (address != NULL)
     buf_appendf(head, "To: %b\n", address);
 
+  if (PRECEDENCE[0])
+	buf_appendf(head, "Precedence: %s\n", PRECEDENCE);
+
   buf_rewind(message);
 
-  if (SMTPRELAY[0])
+  /* search recipient addresses */
+  if (address == NULL) {
+    BUFFER *field, *content;
+    field = buf_new();
+    content = buf_new();
+
+    rcpt_cnt = 0;
+    while (buf_getheader(message, field, content) == 0) {
+      if (bufieq(field, "to") || bufieq(field, "cc") || bufieq(field, "bcc")) {
+	int thislinercpts = 1;
+	char *tmp = content->data;
+	while ((tmp = strchr(tmp+1, ',')))
+	  thislinercpts ++;
+	rcpt_cnt += thislinercpts;
+
+	if ( rcpt->data[0] )
+	  buf_appends(rcpt, ", ");
+	buf_cat(rcpt, content);
+      }
+    }
+    buf_free(field);
+    buf_free(content);
+  } else if (address->data[0]) {
+    char *tmp = address->data;
+    rcpt_cnt = 1;
+    while ((tmp = strchr(tmp+1, ',')))
+      rcpt_cnt ++;
+
+    buf_set(rcpt, address);
+  } else
+    rcpt_cnt = 0;
+
+  buf_rewind(message);
+
+  if ( ! rcpt_cnt ) {
+    errlog(NOTICE, "No recipients found.\n");
+    err = 0;
+  } else if ( rcpt_cnt > MAXRECIPIENTS ) { 
+    errlog(NOTICE, "Too many recipients.  Dropping message.\n");
+    err = 0;
+  } else if ( rcpt_cnt > 1 && has_more_than_one_remailer(rcpt) ) {
+    errlog(NOTICE, "Message is destined to more than one remailer.  Dropping.\n");
+    err = 0;
+  } else if ( REMAIL && strcmp(REMAILERADDR, rcpt->data) == 0) {
+    buf_cat(head, message);
+    errlog(LOG, "Message loops back to us; storing in pool rather than sending it.\n");
+    err = pool_add(head, "inf");
+  } else if (SMTPRELAY[0])
     err = smtpsend(head, message, from);
   else if (strieq(SENDMAIL, "outfile")) {
     char path[PATHMAX];
@@ -153,17 +253,7 @@ int sendmail(BUFFER *message, char *from, BUFFER *address)
     /* Step 2:  Stat the file.  Wait for ENOENT as a response. */
     for (count = 0;; count++) {
       snprintf(path, PATHMAX, "%s%cout.%lu.%u_%lu.%s,S=%lu.txt",
-	POOLDIR, DIRSEP, time(NULL),
-#ifdef POSIX
-	getpid(),
-#else /* end of POSIX */
-#ifdef WIN32
-    _getpid(),
-#else /* end of defined(WIN32) */
-	0,
-#endif
-#endif
-	namecounter++, hostname, head->length + message->length);
+	POOLDIR, DIRSEP, time(NULL), getpid(), namecounter++, hostname, head->length + message->length);
 
       if (stat(path, &statbuf) == 0)
 	errno = EEXIST;
@@ -174,11 +264,7 @@ int sendmail(BUFFER *message, char *from, BUFFER *address)
       }
       if (count > 5)
 	break; /* Too many retries - give up */
-#ifdef WIN32
-      Sleep(2000); /* sleep and retry */
-#else /* end of WIN32 */
       sleep(2); /* sleep and retry */
-#endif /* else not WIN32 */
     }
 #endif /* else not SHORTNAMES */
     if (f != NULL) {
@@ -198,11 +284,15 @@ int sendmail(BUFFER *message, char *from, BUFFER *address)
       closepipe(f);
     }
   }
-  if (err != 0)
+  if (err != 0) {
+    errlog(ERRORMSG, "Unable to execute sendmail. Check path!\n");
     err = 1;			/* error while sending, retry later */
+  }
+
 end:
   buf_free(block);
   buf_free(head);
+  buf_free(rcpt);
   return (err);
 }
 
