@@ -1,4 +1,4 @@
-/* Mixmaster version 2.9  --  (C) 1999 - 2003 Anonymizer Inc. and others.
+/* Mixmaster version 3.0  --  (C) 1999 - 2006 Anonymizer Inc. and others.
 
    Mixmaster may be redistributed and modified under certain conditions.
    This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF
@@ -6,10 +6,11 @@
    details.
 
    Command-line based frontend
-   $Id: main.c 665 2003-11-09 01:47:32Z rabbi $ */
+   $Id: main.c 937 2006-06-24 15:52:20Z colin $ */
 
 
 #include "mix3.h"
+#include "pgp.h"
 #include <stdio.h>
 #include <string.h>
 #include <ctype.h>
@@ -19,9 +20,11 @@
 #else /* end of POSIX */
 #include <io.h>
 #endif /* else if not POSIX */
+#include <assert.h>
 
 static char *largopt(char *p, char *opt, char *name, int *error);
 static void noarg(char *name, char p);
+static int check_get_pass(int force, int never_ask_for_passphrase);
 
 /** main *****************************************************************/
 
@@ -37,8 +40,11 @@ int main(int argc, char *argv[])
 #endif /* WIN32SERVICE */
 {
   int error = 0, deflt = 1, help = 0, readmail = 0, send = -1, sendpool = 0,
-  header = 1, maint = 0, keygen = 0, verbose = 2, sign = 0, encrypt = 0;
+  header = 1, maint = 0, keygen = 0, verbose = 0, sign = 0, encrypt = 0,
+  redirect_mail = 0, about=0, version=0;
   int daemon = 0, type_list = 0, nodetach = 0;
+  int update_stats = 0, update_pingerlist = 0;
+  int never_ask_for_passphrase = 0;
 
 #ifdef USE_SOCK
   int pop3 = 0;
@@ -50,10 +56,14 @@ int main(int argc, char *argv[])
   char *p, *q;
   char chain[1024] = "";
   char nym[LINELEN] = "";
-  BUFFER *nymopt, *pseudonym, *attachments;
+  BUFFER *nymopt, *pseudonym, *attachments, *statssrc;
   int numcopies = 0;		/* default value set in mix.cfg */
   BUFFER *msg, *chainlist, *field, *content;
   FILE *f;
+  char pingpath[PATHMAX];
+
+  /* Check if parse_yearmonthday works */
+  assert(parse_yearmonthday("2003-04-01") == 1049155200);
 
   mix_init(NULL);
 
@@ -64,6 +74,7 @@ int main(int argc, char *argv[])
   attachments = buf_new();
   field = buf_new();
   content = buf_new();
+  statssrc = buf_new();
 
 #ifdef USE_NCURSES
   if (argc == 1) {
@@ -71,13 +82,12 @@ int main(int argc, char *argv[])
       menu_main();
     else
       menu_folder(0, NULL);
-    goto end;
+    goto clientpool;
   }
 #endif /* USE_NCURSES */
   if (argc > 1 && strleft(argv[1], "-f")) {
-    menu_folder(strlen(argv[1]) > 2 ? argv[1][2] : 0,
-		argc < 3 ? NULL : argv[2]);
-    goto end;
+    menu_folder(strlen(argv[1]) > 2 ? argv[1][2] : 0, argc < 3 ? NULL : argv[2]);
+    goto clientpool;
   }
   for (i = 1; i < argc; i++) {
     p = argv[i];
@@ -86,6 +96,10 @@ int main(int argc, char *argv[])
 	p += 2;
 	if (strieq(p, "help"))
 	  help = 1, deflt = 0;
+	else if (streq(p, "version"))
+	  version = 1, deflt = 0;
+	else if (streq(p, "about"))
+	  about = 1, deflt = 0;
 	else if (streq(p, "verbose"))
 	  verbose = 1;
 	else if (streq(p, "type-list"))
@@ -102,6 +116,8 @@ int main(int argc, char *argv[])
 	  sendpool = 1, deflt = 0;
 	else if (streq(p, "read-mail"))
 	  readmail = 1, deflt = 0;
+	else if (streq(p, "redirect"))
+	  redirect_mail = 1, deflt = 0;
 	else if (streq(p, "store-mail"))
 	  readmail = 2, deflt = 0;
 #ifdef USE_SOCK
@@ -120,7 +136,32 @@ int main(int argc, char *argv[])
 	  sign = 1;
 	else if (streq(p, "encrypt"))
 	  encrypt = 1;
-	else if ((q = largopt(p, "to", argv[0], &error)) != NULL) {
+	else if (streq(p, "no-ask-passphrase"))
+	  never_ask_for_passphrase = 1;
+	else if (streq(p, "update-pinger-list"))
+	  update_pingerlist = 1;
+	else if (streq(p, "update-stats")) {
+	  buf_clear(statssrc);
+	  f = mix_openfile(STATSSRC, "r");
+	  if (f != NULL) {
+	    buf_read(statssrc, f);
+	    fclose(f);
+	  }
+	  if (statssrc->length > 0) {
+	    update_stats = 1;
+	  } else {
+	    deflt = 0;
+	    fprintf(stderr, "%s: No current stats source --%s\n", argv[0], p);
+	  }
+	} else if (strleft(p, "update-stats") && p[strlen("update-stats")] == '=') {
+	  buf_clear(statssrc);
+	  buf_appendf(statssrc, "%s", (p + strlen("update-stats") + 1));
+	  if (statssrc->length > 0) {
+	    update_stats = 1;
+	  } else {
+	    fprintf(stderr, "%s: No stats source specified --%s\n", argv[0], p);
+	  }
+	} else if ((q = largopt(p, "to", argv[0], &error)) != NULL) {
 	  header = 0;
 	  buf_appendf(msg, "To: %s\n", q);
 	} else if ((q = largopt(p, "post-to", argv[0], &error)) != NULL) {
@@ -140,6 +181,7 @@ int main(int argc, char *argv[])
 	  buf_appendf(msg, "Latency: %s\n", q);
 	} else if ((q = largopt(p, "attachment", argv[0], &error)) != NULL) {
 	  buf_appendf(attachments, "%s\n", q);
+#ifdef NYMSUPPORT
 	} else if ((q = largopt(p, "nym-config", argv[0], &error)) != NULL) {
 	  deflt = 0;
 	  strncpy(nym, q, sizeof(nym));
@@ -149,6 +191,7 @@ int main(int argc, char *argv[])
 	    buf_appends(nymopt, argv[++i] + 5);
 	} else if ((q = largopt(p, "nym", argv[0], &error)) != NULL) {
 	  buf_appendf(msg, "Nym: %s\n", q);
+#endif /* NYMSUPPORT */
 	}
 #endif /* USE_PGP */
 	else if ((q = largopt(p, "copies", argv[0], &error)) != NULL) {
@@ -204,6 +247,9 @@ int main(int argc, char *argv[])
 	  case 'T':
 	    type_list = 1;
 	    break;
+	  case 'V':
+	    version = 1, deflt = 0;
+	    break;
 	  case 't':
 	    if (*(p + 1) == 'o')
 	      p++;
@@ -211,8 +257,7 @@ int main(int argc, char *argv[])
 	    if (i < argc - 1)
 	      buf_appendf(msg, "To: %s\n", argv[++i]);
 	    else {
-	      fprintf(stderr, "%s: Missing argument for option -to\n",
-		      argv[0]);
+	      fprintf(stderr, "%s: Missing argument for option -to\n", argv[0]);
 	      error = 1;
 	    }
 	    break;
@@ -307,6 +352,7 @@ int main(int argc, char *argv[])
     ret = 1;
     goto end;
   }
+
   if (type_list) {
     BUFFER *type2list;
     type2list = buf_new();
@@ -319,14 +365,73 @@ int main(int argc, char *argv[])
     buf_free(type2list);
     goto end;
   }
-  if (help || (isatty(fileno(stdin)) && isatty(fileno(stdout))))
-    fprintf(stderr, "Mixmaster %s (%s) - %s\n", VERSION, SOURCE_REVISION, COPYRIGHT);
 
+  if (version) {
+    printf("Mixmaster %s\n", VERSION);
+    ret = 0;
+    goto end;
+  }
+
+  if (update_pingerlist) {
+    mixfile(pingpath, ALLPINGERSFILE);
+    if (verbose) printf ("downloading %s...\n", ALLPINGERSURL);
+    if (url_download(ALLPINGERSURL, pingpath) < 0) {
+      printf("    Download failed... Try again later.\n");
+      errlog(ERRORMSG, "All Pingers File Download failed.\n");
+    } else {
+      if (verbose) printf("    Done.\n");
+      errlog(LOG, "All Pingers File Downloaded OK.\n");
+    }
+    ret = 0;
+    goto end;
+  }
+  
+  if (update_stats) {
+    ret = download_stats(statssrc->data);
+    if (ret == -3) {
+      fprintf(stderr, "Stats source does not include all required files.\n");
+    } else if (ret == -2) {
+      fprintf(stderr, "Could not open stats source file for writing\n");
+    } else if (ret == -1) {
+      fprintf(stderr, "Stats source download failed.\n");
+    }
+    ret = 0;
+    goto end;
+  }
+
+#ifdef USE_NCURSES
+/* If we get here then it's possible we still want to use the NCURSES interface */
+    if (deflt && (send == -1) && isatty(fileno(stdin))) {
+      menu_main();
+      goto clientpool;
+    }
+#endif /* USE_NCURSES */
+  
+  if (help ||about ||(isatty(fileno(stdin)) && isatty(fileno(stdout))))
+    fprintf(stderr, "Mixmaster %s\n", VERSION);
+  if (help ||about)
+    printf("\n\n");
+  if (about) {
+    printf("Many people have contributed to the source code for Mixmaster.\n");
+    printf("These contributors include:\n\n");
+    printf("Lance Cottrell\n");
+    printf("Janis Jagars\n");
+    printf("Ulf Moeller\n");
+    printf("Peter Palfrader\n");
+    printf("Len Sassaman\n");
+    printf("\nand others. For full information on copyright and license issues,\n");
+    printf("read the bundled file COPYRIGHT.\n\n");
+    ret = 0;
+    goto end;
+  }
+  
   if (help) {
     printf("Usage: %s [options] [user@host] [filename]\n\n", argv[0]);
     printf("Options:\n\
 \n\
 -h, --help                        summary of command line options\n\
+-V, --version                     print version information\n\
+    --about                       print authorship information\n\
 -T, --type-list                   list available remailers\n\
 -t, --to=user@host                the recipient's address(es)\n\
 -g, --post-to=newsgroup           newsgroup(s) to post to\n\
@@ -336,8 +441,10 @@ int main(int argc, char *argv[])
     --header='header line'        arbitrary message headers\n\
 -a, --attachment=file             attach a file\n"
 #ifdef USE_PGP
-	   "-n, --nym=yournym                 use pseudonym to send the message\n\
-    --encrypt                     encrypt the message using the PGP format\n\
+#ifdef NYMSUPPORT
+	   "-n, --nym=yournym                 use pseudonym to send the message\n"
+#endif /* NYMSUPPORT */
+"    --encrypt                     encrypt the message using the PGP format\n\
     --sign                        sign the message using the PGP format\n"
 #endif /* USE_PGP */
 	   "-l, --chain=mix1,mix2,mix3,...    specify a remailer chain\n\
@@ -345,12 +452,17 @@ int main(int argc, char *argv[])
 -d, --dummy                       generate a dummy message\n\
 -S, --send                        send the message(s) in the pool\n"
 #ifdef USE_PGP
+#ifdef NYMSUPPORT
 	   "    --nym-config=yournym          generate a new pseudonym\n\
     --latency=hours               reply chain latency\n\
     --reply-chain=rem1,rem2,...   reply chain for the pseudonym\n"
+#endif /* NYMSUPPORT */
 #endif /* USE_PGP */
 	   "-v, --verbose                     output informational messages\n\
--f [file]                         read a mail folder\n"
+-f [file]                         read a mail folder\n\
+    --update-pinger-list          Download an updated all pingers list file\n\
+    --update-stats[=source]       Download updated stats\n"
+
 #ifndef USE_NCURSES
 	   "\n-fr, -ff, -fg [file]              send reply/followup/group reply to a message\n"
 #endif /* USE_NCURSES */
@@ -384,6 +496,7 @@ WinNT service:\n\
     ret = 0;
     goto end;
   }
+
   if (deflt && send == -1)
     send = MSG_MAIL;
   if (nym[0] != 0)
@@ -402,7 +515,11 @@ WinNT service:\n\
   if (header == 0)
     buf_nl(msg);
 
-  if (readmail || send == MSG_MAIL || send == MSG_POST) {
+  /* timeskew check */
+  if (REMAIL == 1)
+    mix_check_timeskew();
+
+  if (readmail || redirect_mail || send == MSG_MAIL || send == MSG_POST) {
     if (filename == NULL || streq(filename, "-"))
       f = stdin;
     else {
@@ -412,11 +529,12 @@ WinNT service:\n\
     }
 
     if (f && buf_read(msg, f) != -1) {
-      if (readmail == 1)
+      if (readmail == 1) {
+	check_get_pass(1, never_ask_for_passphrase);
 	mix_decrypt(msg);
-	else if (readmail == 2)
+      } else if (readmail == 2)
 	pool_add(msg, "inf");
-      if (send == MSG_MAIL || send == MSG_POST) {
+      if (send == MSG_MAIL || send == MSG_POST || redirect_mail) {
 	BUFFER *sendmsg;
 	int numdest = 0;
 
@@ -436,10 +554,10 @@ WinNT service:\n\
 	    } else
 	      strncpy(chain, content->data, sizeof(chain));
 	  else {		/* line goes into message */
-	    if ((send == MSG_MAIL && bufieq(field, "to"))
+	    if (((redirect_mail || send == MSG_MAIL) && bufieq(field, "to"))
 		|| (send == MSG_POST && bufieq(field, "newsgroups")))
 	      numdest++;
-	    if (bufieq(field, "from"))
+	    if (bufieq(field, "from") && !redirect_mail)
 	      fprintf(stderr, "Warning: The message has a From: line.\n");
 	    buf_appendheader(sendmsg, field, content);
 	  }
@@ -473,9 +591,11 @@ WinNT service:\n\
 	  buf_free(pass);
 	}
 	if (nym[0] != 0) {
+#ifdef NYMSUPPORT
 	  if (nym_encrypt(sendmsg, nym, send) == 0)
 	    send = MSG_MAIL;
 	  else
+#endif /* NYMSUPPORT */
 	    fprintf(stderr, "Nym error, sending message anonymously.\n");
 	}
 #endif /* USE_PGP */
@@ -486,8 +606,10 @@ WinNT service:\n\
 	  fprintf(stderr, "Invalid number of copies!\n");
 	  ret = 2;
 	} else {
-	  if (mix_encrypt(send, sendmsg, chain, numcopies,
-			  chainlist) == -1) {
+	  if ( ( redirect_mail ?
+		redirect_message(sendmsg, chain, numcopies, chainlist) :
+		mix_encrypt(send, sendmsg, chain, numcopies, chainlist)
+		) == -1) {
 	    ret = 2;
 	    if (chainlist->length)
 	      fprintf(stderr, "%s\n", chainlist->data);
@@ -516,15 +638,16 @@ WinNT service:\n\
     if (mix_encrypt(MSG_NULL, NULL, chain, numcopies, chainlist) == -1) {
       ret = 2;
       if (chainlist->length)
-	fprintf(stderr, "%s\n", chainlist->data);
+	printf("%s\n", chainlist->data);
       else
 	fprintf(stderr, "Failed!\n");
     } else if (verbose) {
-      fprintf(stderr, "Chain: ");
-      buf_write(chainlist, stderr);
+      printf("Chain: ");
+      buf_write(chainlist, stdout);
     }
   }
 #ifdef USE_PGP
+#ifdef NYMSUPPORT
   if (nym[0] != 0) {
     char nymserver[LINELEN] = "*";
     BUFFER *chains;
@@ -554,20 +677,30 @@ WinNT service:\n\
     user_delpass();
     buf_free(chains);
   }
+#endif /* NYMSUPPORT */
 #endif /* USE_PGP */
 
-  if (keygen)
+  if (keygen) {
+    check_get_pass(0, never_ask_for_passphrase);
     keymgt(keygen);
+  }
   if (sendpool)
     mix_send();
 #ifdef USE_SOCK
   if (pop3)
     pop3get();
 #endif /* USE_SOCK */
-  if (maint)
+  if (maint) {
+    check_get_pass(1, never_ask_for_passphrase);
     mix_regular(0);
-  if (REMAIL == 0)
-    mix_regular(0); /* check client pool */
+  }
+
+clientpool:
+  if ((REMAIL == 0) && (CLIENTAUTOFLUSH == 1)) {
+    SENDPOOLTIME = 0;
+    RATE = 100;
+    mix_send();
+  };
 
 end:
   buf_free(field);
@@ -577,8 +710,10 @@ end:
   buf_free(nymopt);
   buf_free(pseudonym);
   buf_free(attachments);
+  buf_free(statssrc);
 
   if (daemon) {
+    check_get_pass(1, never_ask_for_passphrase);
 #ifdef UNIX
     if (! nodetach) {
       int pid;
@@ -615,7 +750,7 @@ end:
 #endif /* UNIX */
     mix_daemon();
 #ifdef UNIX
-/* ifdef this one to, so that we do not need to export it from windows dll */
+/* ifdef this one too, so that we do not need to export it from windows dll */
     clear_pidfile(PIDFILE);
 #endif /* UNIX */
   }
@@ -638,3 +773,48 @@ static void noarg(char *name, char p)
 {
   fprintf(stderr, "%s: Missing argument for option -%c\n", name, p);
 }
+
+static int check_get_pass(int force, int never_ask_for_passphrase)
+/* get a passphrase and check against keys
+ * if force != 0 passphrase must match with some key */
+{
+    BUFFER *pass, *pass2, *key;
+    int n = 0;
+
+    if (PASSPHRASE[0] == '\0' && isatty(fileno(stdin)) && ! never_ask_for_passphrase) {
+      pass = buf_new();
+      pass2 = buf_new();
+      key = buf_new();
+      buf_sets(pass, PASSPHRASE);
+      while (
+#ifdef USE_PGP
+	     pgpdb_getkey(PK_DECRYPT, PGP_ES_RSA, NULL, NULL, NULL, NULL, NULL,
+			   NULL, NULL, NULL, pass) < 0 &&
+	     pgpdb_getkey(PK_DECRYPT, PGP_E_ELG,  NULL, NULL, NULL, NULL, NULL,
+			   NULL, NULL, NULL, pass) < 0 &&
+#endif /* USE_PGP */
+	     getv2seckey(NULL, key) < 0)
+      {
+	user_delpass();
+	if (n)
+	  fprintf(stderr, "re-");
+	user_pass(pass);
+	strncpy(PASSPHRASE, pass->data, LINELEN);
+	PASSPHRASE[LINELEN-1] = 0;
+	if (!force) {
+	  if (n && buf_eq(pass, pass2))
+	    break;
+	  buf_set(pass2, pass);
+	}
+	n=1;
+      }
+      user_delpass();
+      buf_free(pass);
+      buf_free(pass2);
+      buf_free(key);
+
+      strncpy(ENTEREDPASSPHRASE, PASSPHRASE, LINELEN);
+      ENTEREDPASSPHRASE[LINELEN-1] = 0;
+    }
+    return 1;
+}
