@@ -1,4 +1,4 @@
-/* Mixmaster version 2.9  --  (C) 1999 - 2003 Anonymizer Inc. and others.
+/* Mixmaster version 3.0  --  (C) 1999 - 2006 Anonymizer Inc. and others.
 
    Mixmaster may be redistributed and modified under certain conditions.
    This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF
@@ -6,7 +6,7 @@
    details.
 
    Menu-based user interface
-   $Id: menu.c 678 2003-11-09 08:58:07Z weasel $ */
+   $Id: menu.c 934 2006-06-24 13:40:39Z rabbi $ */
 
 
 #include "menu.h"
@@ -108,6 +108,7 @@ void read_folder(char command, char *foldername, char *nym)
 	mix_status("Reading message %d", num);
 #ifdef USE_PGP
       if (ispgp)
+#ifdef NYMSUPPORT
 	switch (nym_decrypt(mail, NULL, log)) {
 	case 2:
 	  from = -1, subject = -1;
@@ -126,6 +127,13 @@ void read_folder(char command, char *foldername, char *nym)
 	default:
 	  ;
 	}
+#else
+	if (!eof) {
+	  buf_clear(mail);
+	  from = -1, subject = -1;
+	  continue;
+	}
+#endif /* NYMSUPPORT */
 #endif /* USE_PGP */
       buf_cat(folder, mail);
       buf_clear(mail);
@@ -467,6 +475,7 @@ void menu_main(void)
   int y, x;
   int pool, n;
   int c;
+  int space;
   BUFFER *chainlist, *line;
   char nym[LINELEN] = ANON;
 
@@ -481,17 +490,26 @@ menu_redraw:
   for (;;) {
     standout();
     mvprintw(0, 0, "Mixmaster %s", VERSION);
+#if 0
     mvprintw(0, COLS - sizeof(COPYRIGHT), COPYRIGHT);
+#endif
+    for (space = 0; space < (COLS - 10 - sizeof(VERSION)); space++) {
+        printw(" ");
+    };
     standend();
     mix_status(NULL);
 
+#ifdef NYMSUPPORT
     mvprintw(8, 4, "n)ym: %s", nym);
+#endif /* NYMSUPPORT */
     y = 12, x = 25;
     mvprintw(y++, x, "m)ail");
     mvprintw(y++, x, "p)ost to Usenet");
     mvprintw(y++, x, "r)ead mail (or news article)");
     mvprintw(y++, x, "d)ummy message");
     mvprintw(y++, x, "s)end messages from pool");
+    mvprintw(y++, x, "e)dit configuration file");
+    mvprintw(y++, x, "u)pdate stats");
     mvprintw(y++, x, "q)uit");
 
     pool = pool_read(NULL);
@@ -509,9 +527,11 @@ menu_redraw:
       case '\014':
 	mix_status("");
 	goto menu_redraw;
+#ifdef NYMSUPPORT
       case 'n':
 	menu_nym(nym);
 	goto menu_redraw;
+#endif /* NYMSUPPORT */
       case 'm':
       case 'p':
 	send_message(c, nym, NULL);
@@ -557,6 +577,17 @@ menu_redraw:
 	    read_folder(0, name, nym);
 	}
 	break;
+      case 'e':
+	do {
+	  char path[PATHMAX];
+	  mixfile(path, MIXCONF);
+	  menu_spawn_editor(path, 0);
+	  mix_config();
+	} while (0);
+	break;
+      case 'u':
+	update_stats();
+	break;
       case 'q':
       case 'Q':
 	goto quit;
@@ -820,6 +851,7 @@ void menu_chain(char *chainstr, int chaintype, int post)
      /* chaintype 0=mix 1=ek 2=newnym */
 {
   REMAILER remailer[MAXREM];
+  int badchains[MAXREM][MAXREM];
   int rlist[2 * MAXREM];
   char newchain[CHAINMAX];
   char info[LINELEN];
@@ -842,9 +874,9 @@ void menu_chain(char *chainstr, int chaintype, int post)
   standend();
 
   if (chaintype == 1)
-    num = t1_rlist(remailer);
+    num = t1_rlist(remailer, badchains);
   else
-    num = mix2_rlist(remailer);
+    num = mix2_rlist(remailer, badchains);
 
   if (num < 1) {
     mix_status("Can't read remailer list.");
@@ -909,7 +941,7 @@ void menu_chain(char *chainstr, int chaintype, int post)
     if (chainlen > 0) {
       ok = 1;
       middlemanlast = remailer[chain[chainlen - 1]].flags.middle;
-      if (post && !remailer[chain[chainlen - 1]].flags.post)
+      if (post && !remailer[chain[chainlen - 1]].flags.post && !(chain[chainlen - 1] == 0 /*randhop*/))
 	ok = 0;
     } else
       ok = 1;
