@@ -1,4 +1,4 @@
-/* Mixmaster version 2.9  --  (C) 1999 - 2003 Anonymizer Inc. and others.
+/* Mixmaster version 3.0  --  (C) 1999 - 2006 Anonymizer Inc. and others.
 
    Mixmaster may be redistributed and modified under certain conditions.
    This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF
@@ -6,7 +6,7 @@
    details.
 
    Menu-based user interface
-   $Id: menu.h 665 2003-11-09 01:47:32Z rabbi $ */
+   $Id: menu.h 934 2006-06-24 13:40:39Z rabbi $ */
 
 
 #ifndef _MENU_H
@@ -28,6 +28,8 @@ void read_folder(char command, char *foldername, char *nym);
 void menu_init(void);
 void menu_exit(void);
 
+void menu_spawn_editor(char *path, int lineno);
+
 #ifdef USE_NCURSES
 void read_message(BUFFER *message, char *nym);
 void menu_nym(char *);
@@ -37,6 +39,7 @@ void askfilename(char *fn);
 void savemsg(BUFFER *message);
 int menu_replychain(int *d, int *l, char *mdest, char *pdest, char *psub,
 		    char *r);
+void update_stats(void);
 
 #endif /* USE_NCURSES */
 
