@@ -1,4 +1,4 @@
-/* Mixmaster version 2.9  --  (C) 1999 - 2003 Anonymizer Inc. and others.
+/* Mixmaster version 3.0  --  (C) 1999 - 2006 Anonymizer Inc. and others.
 
    Mixmaster may be redistributed and modified under certain conditions.
    This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF
@@ -6,11 +6,11 @@
    details.
 
    Menu-based user interface - nym management
-   $Id: menunym.c 665 2003-11-09 01:47:32Z rabbi $ */
+   $Id: menunym.c 934 2006-06-24 13:40:39Z rabbi $ */
 
+#ifdef NYMSUPPORT
 
 #include "menu.h"
-#include "mix.h"
 #include "mix3.h"
 #include <string.h>
 #include <stdlib.h>
@@ -469,3 +469,4 @@ select:
 }
 
 #endif /* USE_NCURSES */
+#endif /* NYMSUPPORT */
