@@ -1,4 +1,4 @@
-/* Mixmaster version 2.9  --  (C) 1999 - 2003 Anonymizer Inc. and others.
+/* Mixmaster version 3.0  --  (C) 1999 - 2006 Anonymizer Inc. and others.
 
    Mixmaster may be redistributed and modified under certain conditions.
    This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF
@@ -6,28 +6,20 @@
    details.
 
    Menu-based user interface -- send message
-   $Id: menusend.c 665 2003-11-09 01:47:32Z rabbi $ */
+   $Id: menusend.c 934 2006-06-24 13:40:39Z rabbi $ */
+
 
-#include "mix.h"
 #include "menu.h"
 #include "mix3.h"
 #include <string.h>
 #include <ctype.h>
 #include <stdlib.h>
 #ifdef POSIX
-#	include <unistd.h>
+#include <unistd.h>
 #else /* end of POSIX */
-#	include <io.h>
+#include <io.h>
 #endif /* else if not POSIX */
 
-#ifdef USE_PGP
-#	include "pgp.h"
-#endif
-
-/* Command line option +nn to position the cursor? */
-#define cursorpos (strfind(editor, "emacs") || streq(editor, "vi") || \
-		   streq(editor, "joe"))
-
 void send_message(int type, char *nym, BUFFER *in)
 {
   char dest[LINELEN] = "", subject[LINELEN] = "";
@@ -180,10 +172,12 @@ redraw:
     standend();
     mix_status(NULL);
     cl(2, 0);
+#ifdef NYMSUPPORT
     printw("n)ym: %s", thisnym);
+#endif /* NYMSUPPORT */
     if (!strleft(thisnym, NONANON)) {
-      cl(4, 0);
       chain_reliability(chain, 0, reliability);   /* chaintype 0=mix */
+      cl(4, 0);
       printw("c)hain: %-35s (reliability: %s)", chain, reliability);
       cl(5, 0);
       printw("r)edundancy: %3d copies ", numcopies);
@@ -216,10 +210,10 @@ redraw:
 
     if (txt->length == 0)
       mvprintw(LINES - 3, 18,
-	       "e)dit message           f)ile                  q)uit");
+	       "e)dit message           f)ile      q)uit w/o sending");
     else
       mvprintw(LINES - 3, 0,
-	       "m)ail message           e)dit message          f)ile                q)uit");
+	       "m)ail message      e)dit message         f)ile          q)uit w/o sending");
     move(LINES - 1, COLS - 1);
     refresh();
     c = getch();
@@ -232,9 +226,11 @@ redraw:
 	  c = 'm';
       }
       switch (c) {
+#ifdef NYMSUPPORT
       case 'n':
 	menu_nym(thisnym);
 	goto redraw;
+#endif /* NYMSUPPORT */
       case '\014':
 	goto redraw;
       case 'd':
@@ -293,16 +289,9 @@ redraw:
       case 'e':
 #endif /* USE_NCURSES */
 	{
-	  char s[PATHMAX];
-	  char *editor;
-	  int showhdr;              /* show header in the editor? */
 	  int linecount;
 
 	edit:
-	  editor = getenv("EDITOR");
-	  if (editor == NULL)
-	    editor = "vi";
-	  showhdr = 1;
 	  linecount = 1;
 	  sprintf(path, "%s%cx%02x%02x%02x%02x.txt", POOLDIR, DIRSEP,
 		  rnd_byte(), rnd_byte(), rnd_byte(), rnd_byte());
@@ -312,43 +301,25 @@ redraw:
 	    beep();
 #endif /* USE_NCURSES */
 	  } else {
+	    if (type == 'f' || type == 'p')
+	      fprintf(f, "Newsgroups: %s\n", dest);
+	    if (type == 'r' || type == 'g' || type == 'm')
+	      fprintf(f, "To: %s\n", dest);
+	    fprintf(f, "Subject: %s\n", subject);
+	    linecount += 2;
+	    if (hdr)
+	      while (buf_getline(txt, NULL) == 0) linecount++;
+	    else
+	      fprintf(f, "\n");
+	    linecount++;
+	    if (txt->length == 0)
+	      fprintf(f, "\n");
 
-	    if (!cursorpos && txt->length == 0 && (type == 'm' || type == 'p'))
-	      showhdr = 0;
-
-	    if (showhdr)
-	    {
-	      if (type == 'f' || type == 'p')
-		fprintf(f, "Newsgroups: %s\n", dest);
-	      if (type == 'r' || type == 'g' || type == 'm')
-		fprintf(f, "To: %s\n", dest);
-	      fprintf(f, "Subject: %s\n", subject);
-	      linecount += 2;
-	      if (hdr)
-		while (buf_getline(txt, NULL) == 0) linecount++;
-	      else
-		fprintf(f, "\n");
-	      linecount++;
-	      if (txt->length == 0)
-		fprintf(f, "\n");
-	    }
 	    buf_write(txt, f);
 	    fclose(f);
 	  }
-	  if (linecount > 1 && cursorpos)
-	    snprintf(s, PATHMAX, "%s +%d %s", editor, linecount, path);
-	  else
-	    snprintf(s, PATHMAX, "%s %s", editor, path);
 
-#ifdef USE_NCURSES
-	  clear();
-	  refresh();
-	  endwin();
-#endif /* USE_NCURSES */
-	  system(s);
-#ifdef USE_NCURSES
-	  refresh();
-#endif /* USE_NCURSES */
+	  menu_spawn_editor(path, linecount);
 
 	  f = fopen(path, "r");
 	  if (f == NULL) {
@@ -362,28 +333,27 @@ redraw:
 	  }
 	  buf_reset(txt);
 	  hdr = 0;
-	  if (showhdr) {
-	    buf_reset(tmp);
-	    buf_read(tmp, f);
-	    while (buf_getheader(tmp, field, content) == 0) {
-	      if (bufieq(field, "subject"))
-		strncpy(subject, content->data,
-			sizeof(subject));
-	      else if ((type == 'p' || type == 'f') &&
-		       bufieq(field, "newsgroups"))
-		strncpy(dest, content->data, sizeof(dest));
-	      else if (bufieq(field, "to"))
-		strncpy(dest, content->data, sizeof(dest));
-	      else {
-		buf_appendheader(txt, field, content);
-		hdr = 1;
-	      }
+
+	  buf_reset(tmp);
+	  buf_read(tmp, f);
+	  while (buf_getheader(tmp, field, content) == 0) {
+	    if (bufieq(field, "subject"))
+	      strncpy(subject, content->data,
+		      sizeof(subject));
+	    else if ((type == 'p' || type == 'f') &&
+		     bufieq(field, "newsgroups"))
+	      strncpy(dest, content->data, sizeof(dest));
+	    else if (bufieq(field, "to"))
+	      strncpy(dest, content->data, sizeof(dest));
+	    else {
+	      buf_appendheader(txt, field, content);
+	      hdr = 1;
 	    }
-	    if (hdr)
-	      buf_nl(txt);
-	    buf_rest(txt, tmp);
-	  } else
-	    buf_read(txt, f);
+	  }
+	  if (hdr)
+	    buf_nl(txt);
+	  buf_rest(txt, tmp);
+
 	  fclose(f);
 	  unlink(path);
 	  strcatn(path, "~", PATHMAX);
@@ -492,11 +462,13 @@ redraw:
 	    goto quit;
 	  } else {
 #ifdef USE_PGP
+#ifdef NYMSUPPORT
 	    if (!streq(thisnym, ANON)) {
 	      if (nym_encrypt(msg, thisnym, (type == 'p' || type == 'f') ?
 			      MSG_POST : MSG_MAIL) == 0)
 		type = 'm';
 	    }
+#endif /* NYMSUPPORT */
 #endif /* USE_PGP */
 	    err = mix_encrypt((type == 'p' || type == 'f') ?
 			      MSG_POST : MSG_MAIL,
