@@ -1,4 +1,4 @@
-/* Mixmaster version 2.9  --  (C) 1999 - 2003 Anonymizer Inc. and others.
+/* Mixmaster version 3.0  --  (C) 1999 - 2006 Anonymizer Inc. and others.
 
    Mixmaster may be redistributed and modified under certain conditions.
    This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF
@@ -6,15 +6,12 @@
    details.
 
    MIME functions
-   $Id: mime.c 665 2003-11-09 01:47:32Z rabbi $ */
+   $Id: mime.c 934 2006-06-24 13:40:39Z rabbi $ */
+
 
-#include "mix.h"
 #include "mix3.h"
 #include <ctype.h>
 #include <string.h>
-#ifdef USE_PGP
-#	include "pgp.h"
-#endif
 
 #define hex(i) (isdigit(i) ? (i) - '0' : tolower(i) - 'a' + 10)
 
@@ -521,10 +518,14 @@ int entity_decode(BUFFER *msg, int message, int mptype, BUFFER *data)
   } else if (mptype == pgpsig && bufeq(type, "application") &&
 	     bufieq(subtype, "pgp-signature")) {
     buf_rest(part, msg);
+#ifdef USE_PGP
     if (pgp_decrypt(part, NULL, data, PGPPUBRING, NULL) == PGP_SIGOK)
       buf_appendf(out, "[-- OpenPGP signature from:\n    %b --]\n", data);
     else
       buf_appends(out, "[-- Invalid OpenPGP signature --]\n");
+#else /* USE_PGP */
+    buf_appends(out, "[-- No OpenPGP support --]\n");
+#endif /* !USE_PGP */
   } else if (type->length == 0 || bufieq(type, "text")) {
     while (buf_getline(msg, line) != -1) {
       int softbreak;
@@ -764,6 +765,9 @@ static int entity_encode(BUFFER *message, BUFFER *out, BUFFER *messagehdr,
 
 int pgpmime_sign(BUFFER *message, BUFFER *uid, BUFFER *pass, char *secring)
 {
+#ifndef USE_PGP
+  return (-1)
+#else /* end of not USE_PGP */
   BUFFER *out, *body, *mboundary, *algo;
   int err;
 
@@ -806,4 +810,5 @@ int pgpmime_sign(BUFFER *message, BUFFER *uid, BUFFER *pass, char *secring)
   buf_free(mboundary);
   buf_free(algo);
   return (err);
+#endif /* else if USE_PGP */
 }
