@@ -1,4 +1,4 @@
-/* Mixmaster version 2.9  --  (C) 1999 - 2003 Anonymizer Inc. and others.
+/* Mixmaster version 3.0  --  (C) 1999 - 2006 Anonymizer Inc. and others.
 
    Mixmaster may be redistributed and modified under certain conditions.
    This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF
@@ -6,7 +6,7 @@
    details.
 
    Mixmaster initialization, configuration
-   $Id: mix.c 665 2003-11-09 01:47:32Z rabbi $ */
+   $Id: mix.c 962 2007-11-19 13:42:41Z zax $ */
 
 
 #include "mix3.h"
@@ -29,6 +29,8 @@
 #endif /* else if not POSIX */
 #ifdef WIN32
 #include <windows.h>
+#include <shlobj.h>
+#include <shlobj.h>
 #endif /* WIN32 */
 #include <assert.h>
 #include "menu.h"
@@ -36,46 +38,46 @@
 int buf_vappendf(BUFFER *b, char *fmt, va_list args);
 
 /** filenames ************************************************************/
-char MIXCONF[PATHMAX] = DEFAULT_MIXCONF;       /* mixmaster configuration file */
-char DISCLAIMFILE[PATHMAX] = DEFAULT_DISCLAIMFILE;
-char FROMDSCLFILE[PATHMAX] = DEFAULT_FROMDSCLFILE;
-char MSGFOOTERFILE[PATHMAX] = DEFAULT_MSGFOOTERFILE;
-char POP3CONF[PATHMAX] = DEFAULT_POP3CONF;
-char HELPFILE[PATHMAX] = DEFAULT_HELPFILE;
-char ABUSEFILE[PATHMAX] = DEFAULT_ABUSEFILE;
-char REPLYFILE[PATHMAX] = DEFAULT_REPLYFILE;
-char USAGEFILE[PATHMAX] = DEFAULT_USAGEFILE;
-char USAGELOG[PATHMAX] = DEFAULT_USAGELOG;
-char BLOCKFILE[PATHMAX] = DEFAULT_BLOCKFILE;
-char ADMKEYFILE[PATHMAX] = DEFAULT_ADMKEYFILE;
-char KEYFILE[PATHMAX] = DEFAULT_KEYFILE;
-char PGPKEY[PATHMAX] = DEFAULT_PGPKEY;
-char DSAPARAMS[PATHMAX] = DEFAULT_DSAPARAMS;
-char DHPARAMS[PATHMAX] = DEFAULT_DHPARAMS;
-char MIXRAND[PATHMAX] = DEFAULT_MIXRAND;
-char SECRING[PATHMAX] = DEFAULT_SECRING;
-char PUBRING[PATHMAX] = DEFAULT_PUBRING;
-char IDLOG[PATHMAX] = DEFAULT_IDLOG;
-char STATS[PATHMAX] = DEFAULT_STATS;
-/* To enable multiple dest.blk files, edit the following line. */
-/* Filenames must be seperated by one space.                   */
-char DESTBLOCK[PATHMAX] = DEFAULT_DESTBLOCK;
-char DESTALLOW[PATHMAX] = DEFAULT_DESTALLOW;
-char SOURCEBLOCK[PATHMAX] = DEFAULT_SOURCEBLOCK;
-char HDRFILTER[PATHMAX] = DEFAULT_HDRFILTER;
-char REGULAR[PATHMAX] = DEFAULT_REGULAR;
-char POOL[PATHMAX] = DEFAULT_POOL;             /* remailer pool subdirectory */
-char TYPE1LIST[PATHMAX] = DEFAULT_TYPE1LIST;
-char TYPE2REL[PATHMAX] = DEFAULT_TYPE2REL;
-char TYPE2LIST[PATHMAX] = DEFAULT_TYPE2LIST;
-char PIDFILE[PATHMAX] = DEFAULT_PIDFILE;
-
-char PGPREMPUBRING[PATHMAX] = DEFAULT_PGPREMPUBRING;
-char PGPREMPUBASC[PATHMAX] = DEFAULT_PGPREMPUBASC;
-char PGPREMSECRING[PATHMAX] = DEFAULT_PGPREMSECRING;
-char NYMSECRING[PATHMAX] = DEFAULT_NYMSECRING;
-char NYMDB[PATHMAX] = DEFAULT_NYMDB;
+char MIXCONF[PATHMAX] = DEFAULT_MIXCONF;
+char DISCLAIMFILE[PATHMAX];
+char FROMDSCLFILE[PATHMAX];
+char MSGFOOTERFILE[PATHMAX];
+char POP3CONF[PATHMAX];
+char HELPFILE[PATHMAX];
+char REQUESTDIR[PATHMAX];
+char ABUSEFILE[PATHMAX];
+char REPLYFILE[PATHMAX];
+char USAGEFILE[PATHMAX];
+char USAGELOG[PATHMAX];
+char BLOCKFILE[PATHMAX];
+char ADMKEYFILE[PATHMAX];
+char KEYFILE[PATHMAX];
+char PGPKEY[PATHMAX];
+char DSAPARAMS[PATHMAX];
+char DHPARAMS[PATHMAX];
+char MIXRAND[PATHMAX];
+char SECRING[PATHMAX];
+char PUBRING[PATHMAX];
+char IDLOG[PATHMAX];
+char STATS[PATHMAX];
+char PGPMAXCOUNT[PATHMAX];
+char DESTBLOCK[PATHMAX];
+char DESTALLOW[PATHMAX];
+char DESTALLOW2[PATHMAX];
+char SOURCEBLOCK[PATHMAX];
+char HDRFILTER[PATHMAX];
+char REGULAR[PATHMAX];
+char POOL[PATHMAX];
+char TYPE1LIST[PATHMAX];
+char TYPE2REL[PATHMAX];
+char PIDFILE[PATHMAX];
 
+char PGPREMPUBRING[PATHMAX];
+char PGPREMPUBASC[PATHMAX];
+char PGPREMSECRING[PATHMAX];
+char NYMSECRING[PATHMAX];
+char NYMDB[PATHMAX];
+char STAREX[PATHMAX];
 
 /** config ***************************************************************/
 
@@ -83,19 +85,15 @@ char MIXDIR[PATHMAX];
 char POOLDIR[PATHMAX];
 
 /* programs */
-#ifdef WIN32
-char SENDMAIL[LINELEN] = "outfile";
-#else /* end of WIN32 */
-char SENDMAIL[LINELEN] = "/usr/lib/sendmail -t";
-#endif /* else if not WIN32 */
+char SENDMAIL[LINELEN];
 char SENDANONMAIL[LINELEN];
 char NEWS[LINELEN];
 char TYPE1[LINELEN];
 
 /* addresses */
-char MAILtoNEWS[LINELEN] = "mail2news@anon.lcs.mit.edu";
-char REMAILERNAME[LINELEN] = "Anonymous Remailer";
-char ANONNAME[LINELEN] = "Anonymous";
+char MAILtoNEWS[LINELEN];
+char REMAILERNAME[LINELEN];
+char ANONNAME[LINELEN];
 char REMAILERADDR[LINELEN];
 char ANONADDR[LINELEN];
 char COMPLAINTS[LINELEN];
@@ -107,74 +105,93 @@ char SMTPPASSWORD[LINELEN];
 #ifdef USE_SOCK
 char HELONAME[LINELEN];
 char ENVFROM[LINELEN];
-int POP3DEL = 0;
-int POP3SIZELIMIT = 0;
-long POP3TIME = 60 * 60;
+int POP3DEL;
+int POP3SIZELIMIT;
+long POP3TIME;
 
 #endif /* USE_SOCK */
 
 char SHORTNAME[LINELEN];
+char ALLPINGERSURL[BUFSIZE];
+char ALLPINGERSFILE[PATHMAX];
+char WGET[PATHMAX];
+char STATSSRC[PATHMAX];
+int  STATSAUTOUPDATE;
+long STATSINTERVAL;
+
 
 /* remailer configuration */
-int REMAIL = 0;
-int MIX = 1;
-int PGP = 1;
-int UNENCRYPTED = 0;
-int REMIX = 1;
-int REPGP = 1;
+int REMAIL;
+int MIX;
+int PGP;
+int UNENCRYPTED;
+int REMIX;
+int REPGP;
+char EXTFLAGS[LINELEN];	/* user-defined capstring flags */
 
-int POOLSIZE = 0;
-int RATE = 100;
-int MIDDLEMAN = 0;
-int AUTOBLOCK = 1;
-char FORWARDTO[LINELEN] = "*";
-int SIZELIMIT = 0;		/* maximal size of remailed messages */
-int INFLATEMAX = 50;		/* maximal size of Inflate: padding */
-int MAXRANDHOPS = 20;
-int BINFILTER = 0;		/* filter binary attachments? */
-int LISTSUPPORTED = 1;                /* list supported remailers in remailer-conf reply? */
-long PACKETEXP = 7 * SECONDSPERDAY;	/* Expiration time for old packets */
-long IDEXP = 7 * SECONDSPERDAY;	/* 0 = no ID log !! */
-long SENDPOOLTIME = 0;          /* frequency for sending pool messages */
-long MAILINTIME = 5 * 60;	/* frequency for processing MAILIN mail */
+char PRECEDENCE[LINELEN];	/* default Precedence: header for outgoing mail */
+int POOLSIZE;
+int RATE;
+int INDUMMYP;
+int OUTDUMMYP;
+int INDUMMYMAXP;
+int OUTDUMMYMAXP;
+int MIDDLEMAN;
+int AUTOBLOCK;
+int STATSDETAILS;
+char FORWARDTO[LINELEN];
+int SIZELIMIT;		/* maximal size of remailed messages */
+int INFLATEMAX;		/* maximal size of Inflate: padding */
+int MAXRANDHOPS;
+int BINFILTER;		/* filter binary attachments? */
+int LISTSUPPORTED;	/* list supported remailers in remailer-conf reply? */
+long PACKETEXP;		/* Expiration time for old packets */
+long IDEXP;		/* 0 = no ID log !! */
+long SENDPOOLTIME;	/* frequency for sending pool messages */
+long MAILINTIME;	/* frequency for processing MAILIN mail */
+
+long KEYLIFETIME;
+long KEYOVERLAPPERIOD;
+long KEYGRACEPERIOD;
 
 char ERRLOG[LINELEN];
 char ADDRESS[LINELEN];
 char NAME[LINELEN];
 
-char ORGANIZATION[LINELEN] = "Anonymous Posting Service";
-char MID[LINELEN] = "y";
+char ORGANIZATION[LINELEN];
+char MID[LINELEN];
 
 /* client config */
-int NUMCOPIES = 1;
-char CHAIN[LINELEN] = "*,*,*,*";
-int VERBOSE = 2;
-int DISTANCE = 2;
-int MINREL = 98;
-int RELFINAL = 99;
-long MAXLAT = 36 * 60 * 60;
+int NUMCOPIES;
+char CHAIN[LINELEN];
+int VERBOSE;
+int DISTANCE;
+int MINREL;
+int RELFINAL;
+long MAXLAT;
+long MINLAT;
 char PGPPUBRING[PATHMAX];
 char PGPSECRING[PATHMAX];
-#ifdef COMPILEDPASS
-char PASSPHRASE[LINELEN] = COMPILEDPASS;
-#else /* end of COMPILEDPASS */
-char PASSPHRASE[LINELEN] = "";
-#endif /* else if not COMPILEDPASS */
-char MAILIN[PATHMAX] = "";
-char MAILBOX[PATHMAX] = "mbox";
+char PASSPHRASE[LINELEN];
+char MAILIN[PATHMAX];
+char MAILBOX[PATHMAX];
 char MAILABUSE[PATHMAX];
 char MAILBLOCK[PATHMAX];
-#ifdef WIN32
-char MAILUSAGE[PATHMAX] = "nul:";
-char MAILANON[PATHMAX] = "nul:";
-char MAILERROR[PATHMAX] = "nul:";
-#else /* end of WIN32 */
-char MAILUSAGE[PATHMAX] = "/dev/null";
-char MAILANON[PATHMAX] = "/dev/null";
-char MAILERROR[PATHMAX] = "/dev/null";
-#endif /* else if not WIN32 */
+char MAILUSAGE[PATHMAX];
+char MAILANON[PATHMAX];
+char MAILERROR[PATHMAX];
 char MAILBOUNCE[PATHMAX];
 
+int CLIENTAUTOFLUSH;
+int MAXRECIPIENTS;
+
+long TIMESKEW_FORWARD;
+long TIMESKEW_BACK;
+int TEMP_FAIL;
+
+char ENTEREDPASSPHRASE[LINELEN] = "";
+
+static int rereadconfig = 0;
 static int terminatedaemon = 0;
 
 #if defined(S_IFDIR) && !defined(S_ISDIR)
@@ -326,13 +343,25 @@ static int readtconfline(char *line, char *name, int namelen, long *var)
       while (isspace(*line))
 	line++;
       switch (tolower(*line)) {
-      case 'd':
+      case 'y': /* years */
+	l += 365 * 24 * 60 * 60 * n;
+	break;
+      case 'b': /* months */
+	l += 30 * 24 * 60 * 60 * n;
+	break;
+      case 'w': /* weeks */
+	l += 7 * 24 * 60 * 60 * n;
+	break;
+      case 'd': /* days */
 	l += 24 * 60 * 60 * n;
 	break;
-      case 'm':
+      case 's': /* seconds */
+	l += n;
+	break;
+      case 'm': /* minutes */
 	l += 60 * n;
 	break;
-      case 'h':
+      case 'h': /* hours - default */
       default:
 	l += 60 * 60 * n;
 	break;
@@ -345,6 +374,174 @@ static int readtconfline(char *line, char *name, int namelen, long *var)
     return (0);
 }
 
+static void mix_setdefaults()
+{
+#define strnncpy(a,b) strncpy(a, b, sizeof(a)); a[sizeof(a)-1] = '\0'
+
+	strnncpy(DISCLAIMFILE , DEFAULT_DISCLAIMFILE);
+	strnncpy(FROMDSCLFILE , DEFAULT_FROMDSCLFILE);
+	strnncpy(MSGFOOTERFILE, DEFAULT_MSGFOOTERFILE);
+	strnncpy(POP3CONF     , DEFAULT_POP3CONF);
+	strnncpy(HELPFILE     , DEFAULT_HELPFILE);
+	strnncpy(REQUESTDIR   , DEFAULT_REQUESTDIR);
+	strnncpy(ABUSEFILE    , DEFAULT_ABUSEFILE);
+	strnncpy(REPLYFILE    , DEFAULT_REPLYFILE);
+	strnncpy(USAGEFILE    , DEFAULT_USAGEFILE);
+	strnncpy(USAGELOG     , DEFAULT_USAGELOG);
+	strnncpy(BLOCKFILE    , DEFAULT_BLOCKFILE);
+	strnncpy(ADMKEYFILE   , DEFAULT_ADMKEYFILE);
+	strnncpy(KEYFILE      , DEFAULT_KEYFILE);
+	strnncpy(PGPKEY       , DEFAULT_PGPKEY);
+	strnncpy(DSAPARAMS    , DEFAULT_DSAPARAMS);
+	strnncpy(DHPARAMS     , DEFAULT_DHPARAMS);
+	strnncpy(MIXRAND      , DEFAULT_MIXRAND);
+	strnncpy(SECRING      , DEFAULT_SECRING);
+	strnncpy(PUBRING      , DEFAULT_PUBRING);
+	strnncpy(IDLOG        , DEFAULT_IDLOG);
+	strnncpy(STATS        , DEFAULT_STATS);
+	strnncpy(PGPMAXCOUNT  , DEFAULT_PGPMAXCOUNT);
+	strnncpy(DESTBLOCK    , DEFAULT_DESTBLOCK);
+	strnncpy(DESTALLOW    , DEFAULT_DESTALLOW);
+	strnncpy(DESTALLOW2   , DEFAULT_DESTALLOW2);
+	strnncpy(SOURCEBLOCK  , DEFAULT_SOURCEBLOCK);
+	strnncpy(HDRFILTER    , DEFAULT_HDRFILTER);
+	strnncpy(REGULAR      , DEFAULT_REGULAR);
+	strnncpy(POOL         , DEFAULT_POOL);
+	strnncpy(TYPE1LIST    , DEFAULT_TYPE1LIST);
+	strnncpy(TYPE2REL     , DEFAULT_TYPE2REL);
+	strnncpy(PIDFILE      , DEFAULT_PIDFILE);
+
+	strnncpy(PGPREMPUBRING, DEFAULT_PGPREMPUBRING);
+	strnncpy(PGPREMPUBASC , DEFAULT_PGPREMPUBASC);
+	strnncpy(PGPREMSECRING, DEFAULT_PGPREMSECRING);
+	strnncpy(NYMSECRING   , DEFAULT_NYMSECRING);
+	strnncpy(NYMDB        , DEFAULT_NYMDB);
+	strnncpy(STAREX       , DEFAULT_STAREX);
+	strnncpy(ALLPINGERSURL, DEFAULT_ALLPINGERSURL);
+	strnncpy(ALLPINGERSFILE, DEFAULT_ALLPINGERSFILE);
+	strnncpy(WGET         , DEFAULT_WGET);
+	strnncpy(STATSSRC     , DEFAULT_STATSSRC);
+
+	strnncpy(MIXDIR       , "");
+	strnncpy(POOLDIR      , "");
+
+/* programs */
+#ifdef WIN32
+	strnncpy(SENDMAIL     , "outfile");
+#else /* end of WIN32 */
+	strnncpy(SENDMAIL     , "/usr/lib/sendmail -t");
+#endif /* else if not WIN32 */
+	strnncpy(SENDANONMAIL , "");
+	strnncpy(NEWS         , "");
+	strnncpy(TYPE1        , "");
+
+/* addresses */
+	strnncpy(MAILtoNEWS   , "mail2news@dizum.com,mail2news@m2n.mixmin.net");
+	strnncpy(REMAILERNAME , "Anonymous Remailer");
+	strnncpy(ANONNAME     , "Anonymous");
+	strnncpy(REMAILERADDR , "");
+	strnncpy(ANONADDR     , "");
+	strnncpy(COMPLAINTS   , "");
+	strnncpy(SMTPRELAY    , "");
+	AUTOREPLY             = 0;
+
+#ifdef USE_SOCK
+  	strnncpy(HELONAME     , "");
+	strnncpy(ENVFROM      , "");
+	POP3DEL               = 0;
+	POP3SIZELIMIT         = 0;
+	POP3TIME              = 60 * 60;
+
+#endif /* USE_SOCK */
+
+	strnncpy(SHORTNAME    , "");
+
+/* 	configuration */
+	REMAIL        = 0;
+	MIX           = 1;
+	PGP           = 1;
+	UNENCRYPTED   = 0;
+	REMIX         = 1;
+	REPGP         = 1;
+	STATSAUTOUPDATE = 0;
+	STATSINTERVAL = 8 * 60 * 60;
+	strnncpy(EXTFLAGS, "");
+
+    strnncpy(PRECEDENCE, "");
+	POOLSIZE      = 0;
+	RATE          = 100;
+	INDUMMYP      = 3;	/* add dummy messages with probability p for each message added to the pool */
+	OUTDUMMYP     = 10;	/* add dummy messages with probability p each time we send from the pool */
+	INDUMMYMAXP   = 84;	/* for both of the above:  while (rnd < p) { senddummy(); }  */
+	OUTDUMMYMAXP  = 96;     /* set max INDUMMYP and OUTDUMMYP such that 24 and 5.25 dummy messages will */
+	MIDDLEMAN     = 0;      /* be generated on average. More than this is insane. */
+	AUTOBLOCK     = 1;
+	STATSDETAILS  = 1;
+	strnncpy(FORWARDTO, "*");
+	SIZELIMIT     = 0;		/* maximal size of remailed messages */
+	INFLATEMAX    = 50;		/* maximal size of Inflate: padding */
+	MAXRANDHOPS   = 5;
+	BINFILTER     = 0;		/* filter binary attachments? */
+	LISTSUPPORTED = 1;		/* list supported remailers in remailer-conf reply? */
+	PACKETEXP     = 7 * SECONDSPERDAY;	/* Expiration time for old packets */
+	IDEXP         = 7 * SECONDSPERDAY;	/* 0 = no ID log !! */
+	SENDPOOLTIME  = 0;              /* frequency for sending pool messages */
+	MAILINTIME    = 5 * 60;		/* frequency for processing MAILIN mail */
+
+	KEYLIFETIME      = 13 * 30 * 24 * 60 * 60;	/* validity period for keys. */
+	KEYOVERLAPPERIOD =  1 * 30 * 24 * 60 * 60;	/* when keys have this amount of time */
+	                                                /* left before expiration, create  */
+	                                        	/* new ones when ./mix -K is run.*/
+	KEYGRACEPERIOD   =       7 * 24 * 60 * 60;	/* accept mail to the old key for this */
+	                                        	/* amount of time after it has expired. */
+
+
+	strnncpy(ERRLOG      , "");
+	strnncpy(ADDRESS     , "");
+	strnncpy(NAME        , "");
+
+	strnncpy(ORGANIZATION, "Anonymous Posting Service");
+	strnncpy(MID         , "y");
+
+/* client config */
+	NUMCOPIES = 1;
+	strnncpy(CHAIN, "*,*,*,*");
+	VERBOSE = 2;
+	DISTANCE = 2;
+	MINREL = 98;
+	RELFINAL = 99;
+	MAXLAT = 36 * 60 * 60;
+	MINLAT = 5 * 60;
+	strnncpy(PGPPUBRING, "");
+	strnncpy(PGPSECRING, "");
+#ifdef COMPILEDPASS
+	strnncpy(PASSPHRASE, COMPILEDPASS);
+#else /* end of COMPILEDPASS */
+	strnncpy(PASSPHRASE, "");
+#endif /* else if not COMPILEDPASS */
+	strnncpy(MAILIN    , "");
+	strnncpy(MAILBOX   , "mbox");
+	strnncpy(MAILABUSE , "");
+	strnncpy(MAILBLOCK , "");
+#ifdef WIN32
+	strnncpy(MAILUSAGE , "nul:");
+	strnncpy(MAILANON  , "nul:");
+	strnncpy(MAILERROR , "nul:");
+#else /* end of WIN32 */
+	strnncpy(MAILUSAGE , "/dev/null");
+	strnncpy(MAILANON  , "/dev/null");
+	strnncpy(MAILERROR , "/dev/null");
+#endif /* else if not WIN32 */
+	strnncpy(MAILBOUNCE, "");
+
+	CLIENTAUTOFLUSH = 1;
+	MAXRECIPIENTS   = 5;
+
+	TIMESKEW_FORWARD = 2*7*24*60*60;
+	TIMESKEW_BACK = 12*60*60;
+	TEMP_FAIL = 75;
+}
+
 int mix_configline(char *line)
 {
   return (read_conf(ADDRESS) || read_conf(NAME) ||
@@ -357,26 +554,32 @@ int mix_configline(char *line)
 	  read_conf(HELONAME) || read_conf(ENVFROM) ||
 #endif /* USE_SOCK */
 	  read_conf(SENDMAIL) || read_conf(SENDANONMAIL) ||
+	  read_conf(PRECEDENCE) ||
 	  read_conf_i(REMAIL) || read_conf_i(MIX) ||
 	  read_conf_i(PGP) || read_conf_i(UNENCRYPTED) ||
 	  read_conf_i(REMIX) || read_conf(NEWS) ||
-	  read_conf_i(REPGP) ||
+	  read_conf_i(REPGP) || read_conf(EXTFLAGS) ||
 	  read_conf(MAILtoNEWS) || read_conf(ERRLOG) ||
 	  read_conf(ORGANIZATION) || read_conf(MID) ||
 	  read_conf(TYPE1) || read_conf_i(POOLSIZE) ||
 	  read_conf_i(RATE) || read_conf_i(MIDDLEMAN) ||
+	  read_conf_i(INDUMMYP) ||
+	  read_conf_i(OUTDUMMYP) ||
 	  read_conf_i(AUTOBLOCK) || read_conf(FORWARDTO) ||
+	  read_conf_i(STATSDETAILS) ||
 	  read_conf_i(SIZELIMIT) || read_conf_i(INFLATEMAX) ||
 	  read_conf_i(MAXRANDHOPS) || read_conf_i(BINFILTER) ||
+	  read_conf_i(LISTSUPPORTED) ||
 	  read_conf_t(PACKETEXP) || read_conf_t(IDEXP) ||
-	read_conf_i(LISTSUPPORTED) ||
 	  read_conf_t(SENDPOOLTIME) || read_conf_i(NUMCOPIES) ||
 	  read_conf_t(MAILINTIME) ||
 	  read_conf(CHAIN) || read_conf_i(VERBOSE) ||
 	  read_conf_i(DISTANCE) || read_conf_i(MINREL) ||
 	  read_conf_i(RELFINAL) || read_conf_t(MAXLAT) ||
+	  read_conf_t(MINLAT) ||
 	  read_conf(PGPPUBRING) || read_conf(PGPSECRING) ||
-	  read_conf(PASSPHRASE) ||
+	  read_conf(PASSPHRASE) || read_conf_t(KEYLIFETIME) ||
+	  read_conf_t(KEYGRACEPERIOD) || read_conf_t(KEYOVERLAPPERIOD) ||
 #ifdef USE_SOCK
 	  read_conf_i(POP3DEL) || read_conf_i(POP3SIZELIMIT) ||
 	  read_conf_t(POP3TIME) ||
@@ -387,8 +590,9 @@ int mix_configline(char *line)
 	  read_conf(MAILBOUNCE) || read_conf(MAILIN) ||
 
 	  read_conf(DISCLAIMFILE) || read_conf(FROMDSCLFILE) ||
+	  read_conf(MSGFOOTERFILE) ||
 	  read_conf(POP3CONF) || read_conf(HELPFILE) ||
-	read_conf(MSGFOOTERFILE) ||
+	  read_conf(REQUESTDIR)  ||
 	  read_conf(ABUSEFILE) || read_conf(REPLYFILE) ||
 	  read_conf(USAGEFILE) || read_conf(USAGELOG) ||
 	  read_conf(BLOCKFILE) || read_conf(ADMKEYFILE) ||
@@ -397,14 +601,26 @@ int mix_configline(char *line)
 	  read_conf(MIXRAND) || read_conf(SECRING) ||
 	  read_conf(PUBRING) || read_conf(IDLOG) ||
 	  read_conf(STATS) || read_conf(DESTBLOCK) ||
-	  read_conf(DESTALLOW) || read_conf(SOURCEBLOCK) ||
+	  read_conf(PGPMAXCOUNT) ||
+	  read_conf(DESTALLOW) || read_conf(DESTALLOW2) ||
+	  read_conf(SOURCEBLOCK) ||
+	  read_conf(STAREX) || read_conf(ALLPINGERSURL) ||
+	  read_conf(ALLPINGERSFILE) ||
 	  read_conf(HDRFILTER) || read_conf(REGULAR) ||
 	  read_conf(POOL) || read_conf(TYPE1LIST) ||
-	  read_conf(TYPE2REL) || read_conf(TYPE2LIST) ||
+	  read_conf(TYPE2REL) ||
 	  read_conf(PGPREMPUBRING) || read_conf(PGPREMPUBASC) ||
 	  read_conf(PGPREMSECRING) || read_conf(NYMSECRING) ||
-	  read_conf(NYMDB) || read_conf(PIDFILE) );
+	  read_conf(NYMDB) || read_conf(PIDFILE) ||
+	  read_conf(WGET) || read_conf(STATSSRC) ||
+	  read_conf_i(STATSAUTOUPDATE) || read_conf_t(STATSINTERVAL) ||
 
+	  read_conf_i(CLIENTAUTOFLUSH) ||
+	  read_conf_i(MAXRECIPIENTS) ||
+	  
+	  read_conf_t(TIMESKEW_FORWARD) ||
+	  read_conf_t(TIMESKEW_BACK) ||
+	  read_conf_i(TEMP_FAIL) );
 }
 
 int mix_config(void)
@@ -426,12 +642,26 @@ int mix_config(void)
   int rkey = 0;
 #endif /* WIN32 */
 
+  mix_setdefaults();
+
 #ifdef POSIX
   pw = getpwuid(getuid());
 #endif /* POSIX */
 
-  if (MIXDIR[0])		/* if set by main() */
-    err = mixdir(MIXDIR, 1);
+ /* find our base directory
+  *
+  * first match wins.
+  *
+  *  - what the MIXPATH environment variable points to, if it is set.
+  *  - On WIN32, HKEY_CURRENT_USER\Software\Mixmaster\MixDir, if it exists
+  *  - whatever is compiled in with -DSPOOL
+  *  - On Win32 %APPDATA%\Mixmaster
+  *  - on POSIX, ~/Mix  (or ~/<HOMEMIXDIR>)
+  *  - the current working directory
+  */
+
+  if (err == -1 && (d = getenv("MIXPATH")) != NULL)
+    err = mixdir(d, 1);
 
 #ifdef WIN32
   RegOpenKeyEx(HKEY_CURRENT_USER, "Software", 0, KEY_ALL_ACCESS, &regsw);
@@ -444,14 +674,29 @@ int mix_config(void)
   }
 #endif /* WIN32 */
 
-  if (err == -1 && (d = getenv("MIXPATH")) != NULL)
-    err = mixdir(d, 1);
-
 #ifdef SPOOL
   if (err == -1 && strlen(SPOOL) > 0)
     err = mixdir(SPOOL, 0);
 #endif /* SPOOL */
 
+#ifdef WIN32
+    if (err == -1) {
+      LPMALLOC lpmalloc;
+      ITEMIDLIST *itemidlist;
+      if (SUCCEEDED(SHGetMalloc(&lpmalloc)))
+      {
+	SHGetSpecialFolderLocation(0,CSIDL_APPDATA,&itemidlist);
+	SHGetPathFromIDList(itemidlist,line);
+	lpmalloc->lpVtbl->Free(lpmalloc,&itemidlist);
+	lpmalloc->lpVtbl->Release(lpmalloc);
+
+	strcatn(line, "\\Mixmaster", PATHMAX);
+	err = mixdir(line, 1);
+
+      }
+    }
+#endif /* WIN32 */
+
 #ifdef POSIX
   if (err == -1 && pw != NULL) {
     strncpy(line, pw->pw_dir, PATHMAX);
@@ -501,6 +746,10 @@ int mix_config(void)
     IDEXP = 5 * SECONDSPERDAY;
   if (MAXRANDHOPS > 20)
     MAXRANDHOPS = 20;
+  if (INDUMMYP > INDUMMYMAXP)
+    INDUMMYP = INDUMMYMAXP;
+  if (OUTDUMMYP > OUTDUMMYMAXP)
+    OUTDUMMYP = OUTDUMMYMAXP;
 
   if (strchr(SHORTNAME, '.'))
     *strchr(SHORTNAME, '.') = '\0';
@@ -549,9 +798,6 @@ int mix_config(void)
   if (TYPE1[0] == '\0')
     PGP = 0;
 #endif /* not USE_PGP */
-#ifndef USE_RSA
-  MIX = 0;
-#endif /* not USE_RSA */
 
 #ifdef WIN32
   if (RegOpenKeyEx(regsw, "PGP", 0, KEY_ALL_ACCESS, &regpgp) == 0)
@@ -619,6 +865,11 @@ int mix_config(void)
     REMAIL = 0;
   }
 
+  if (ENTEREDPASSPHRASE[0] != '\0') {
+    strncpy(PASSPHRASE, ENTEREDPASSPHRASE, LINELEN);
+    PASSPHRASE[LINELEN-1] = 0;
+  };
+
   return (0);
 }
 
@@ -626,6 +877,33 @@ int mix_config(void)
 
 static int initialized = 0;
 
+void mix_check_timeskew() {
+  FILE *f;
+  long now, tpool = 0, tpop3 = 0, tdaily = 0, tmailin = 0, latest = 0;
+
+  f = mix_openfile(REGULAR, "r+");
+  if (f != NULL) {
+    lock(f);
+    fscanf(f, "%ld %ld %ld %ld", &tpool, &tpop3, &tdaily, &tmailin);
+    latest = tpool;
+    latest = latest > tpop3 ? latest : tpop3;
+    latest = latest > tdaily ? latest : tdaily;
+    latest = latest > tmailin ? latest : tmailin;
+    now = time(NULL);
+
+
+    if (( (TIMESKEW_BACK    != 0) && (now < latest - TIMESKEW_BACK   )) ||
+        ( (TIMESKEW_FORWARD != 0) && (now > latest + TIMESKEW_FORWARD)) ) {
+      /* Possible timeskew */
+      errlog(ERRORMSG, "Possible timeskew detected.  Check clock and rm %s\n", REGULAR);
+      exit(TEMP_FAIL);
+    }
+    fclose(f);
+  } else {
+    /* shrug */
+  }
+}
+
 int mix_init(char *mixdir)
 {
   if (!initialized) {
@@ -655,10 +933,26 @@ void mix_exit(void)
   initialized=0;
 }
 
+void mix_upd_stats(void)
+{
+  FILE *f;
+  BUFFER *statssrc;
+  statssrc = buf_new();
+  buf_clear(statssrc);
+  f = mix_openfile(STATSSRC, "r");
+  if (f != NULL) {
+    buf_read(statssrc, f);
+    fclose(f);
+  }
+  if (statssrc->length > 0)
+    download_stats(statssrc->data);
+  buf_free(statssrc);
+}
+
 int mix_regular(int force)
 {
   FILE *f;
-  long now, tpool = 0, tpop3 = 0, tdaily = 0, tmailin = 0;
+  long now, tpool = 0, tpop3 = 0, tdaily = 0, tmailin = 0, tstats = 0;
   int ret = 0;
 
   mix_init(NULL);
@@ -667,7 +961,7 @@ int mix_regular(int force)
   f = mix_openfile(REGULAR, "r+");
   if (f != NULL) {
     lock(f);
-    fscanf(f, "%ld %ld %ld %ld", &tpool, &tpop3, &tdaily, &tmailin);
+    fscanf(f, "%ld %ld %ld %ld %ld", &tpool, &tpop3, &tdaily, &tmailin, &tstats);
     if (now - tpool >= SENDPOOLTIME)
       force |= FORCE_POOL | FORCE_MAILIN;
 #ifdef USE_SOCK
@@ -678,6 +972,8 @@ int mix_regular(int force)
       force |= FORCE_DAILY;
     if (now - tmailin >= MAILINTIME)
       force |= FORCE_MAILIN;
+    if (now - tstats >= STATSINTERVAL)
+      force |= FORCE_STATS;
     if (force & FORCE_POOL)
       tpool = now;
     if (force & FORCE_POP3)
@@ -686,16 +982,18 @@ int mix_regular(int force)
       tdaily = now;
     if (force & FORCE_MAILIN)
       tmailin = now;
+    if (force & FORCE_STATS)
+      tstats = now;
     rewind(f);
-    fprintf(f, "%ld %ld %ld %ld\n", tpool, tpop3, tdaily, tmailin);
+    fprintf(f, "%ld %ld %ld %ld %ld\n", tpool, tpop3, tdaily, tmailin, tstats);
     unlock(f);
     fclose(f);
   } else {
-    force = FORCE_POOL | FORCE_POP3 | FORCE_DAILY | FORCE_MAILIN;
+    force = FORCE_POOL | FORCE_POP3 | FORCE_DAILY | FORCE_MAILIN | FORCE_STATS;
     f = mix_openfile(REGULAR, "w+");
     if (f != NULL) {
       lock(f);
-      fprintf(f, "%ld %ld %ld %ld\n", now, now, now, now);
+      fprintf(f, "%ld %ld %ld %ld %ld\n", now, now, now, now, now);
       unlock(f);
       fclose(f);
     } else
@@ -712,6 +1010,8 @@ int mix_regular(int force)
     ret = process_mailin();
   if (force & FORCE_POOL)
     ret = pool_send();
+  if ((force & FORCE_STATS) && (STATSAUTOUPDATE != 0))
+    mix_upd_stats();
 
   return (ret);
 }
@@ -719,6 +1019,7 @@ int mix_regular(int force)
 int mix_daily(void)
 {
   idexp();
+  pgpmaxexp();
   pool_packetexp();
   stats(NULL);
   keymgt(0);
@@ -736,7 +1037,9 @@ int mix_daily(void)
  */
 #ifdef POSIX
 void sighandler(int signal) {
-  if (signal == SIGINT || signal == SIGTERM)
+  if (signal == SIGHUP)
+    rereadconfig = 1;
+  else if (signal == SIGINT || signal == SIGTERM)
     terminatedaemon = 1;
 };
 #endif /* POSIX */
@@ -788,6 +1091,7 @@ int is_nt_service(void)
 #ifdef WIN32SERVICE
     STARTUPINFO StartupInfo;
     OSVERSIONINFO VersionInfo;
+    DWORD dwsize;
 
     if (issvc != -1)    /* do it only once */
 	return issvc;
@@ -831,6 +1135,19 @@ int mix_daemon(void)
   for(;;) {
     if (terminatedaemon)
       break;
+    if (rereadconfig) {
+      rereadconfig = 0;
+      mix_config();
+      t = SENDPOOLTIME;
+      if (MAILINTIME < t && (MAILIN != NULL && MAILIN[0] != '\0'))
+	t = MAILINTIME;
+#ifdef USE_SOCK
+      if (POP3TIME < t)
+	t = POP3TIME;
+      if (t < 5)
+	t = 5; /* Some kind of safety for broken systems */
+#endif /* USE_SOCK */
+    }
     if (slept >= t) {
       mix_regular(0);
       slept = 0;
@@ -845,10 +1162,10 @@ int mix_daemon(void)
     }
 #endif /* WIN32SERVICE */
 
-    if (!terminatedaemon) {
+    if (!terminatedaemon && !rereadconfig) {
       setsignalhandler(0); /* set signal handlers;  don't restart system calls */
 #ifdef WIN32
-      Sleep(t * 1000); /* how to get the real number of seconds slept? */
+      sleep(t); /* how to get the real number of seconds slept? */
       slept = t;
 #else /* end of WIN32 */
       slept += (t - slept) - sleep(t - slept);
@@ -916,7 +1233,11 @@ void mix_status(char *fmt,...)
 
   if (fmt != NULL) {
     va_start(args, fmt);
+#ifdef _MSC
+    _vsnprintf(statusline, sizeof(statusline) - 1, fmt, args);
+#else /* end of _MSC */
     vsnprintf(statusline, sizeof(statusline) - 1, fmt, args);
+#endif /* else if not _MSC */
     va_end(args);
   }
 #ifdef USE_NCURSES
