@@ -1,6 +1,4 @@
-/* Mixmaster version 2.9  --  (C) 1999 - 2003 Anonymizer Inc. and others.
-
-   $Id: mix.h 665 2003-11-09 01:47:32Z rabbi $
+/* Mixmaster version 3.0  --  (C) 1999 - 2006 Anonymizer Inc. and others.
 
    Mixmaster may be redistributed and modified under certain conditions.
    This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF
@@ -166,7 +164,7 @@ int mix_send(void);
   Return value: The size of the pool after the messages have been sent.
 
 
-void mix_regular(int force);
+int mix_regular(int force);
 
   This function is responsible for regular actions of the remailer such
   as sending messages from the pool, getting mail from POP3 servers and
@@ -805,17 +803,11 @@ int buf_unzip(BUFFER *buf, int type);
 #define _MIXLIB_H
 
 #include <stdio.h>
-#include "config.h"
+#include <time.h>
 #ifdef WIN32
-#	include <windows.h>
+#include <windows.h>
 #endif /* WIN32 */
 
-#ifdef MIXLIB_EXPORTS
-#define DllExport	__declspec( dllexport )
-#else
-#define DllExport
-#endif
-
 typedef unsigned char byte;
 
 typedef struct {
@@ -826,19 +818,19 @@ typedef struct {
   byte sensitive;
 } BUFFER;
 
-DllExport int mix_init(char *);
-DllExport void mix_exit(void);
-DllExport void rnd_update(byte *b, int n);
-DllExport void rnd_initialized(void);
+int mix_init(char *);
+void mix_exit(void);
+void rnd_update(byte *b, int n);
+void rnd_initialized(void);
 #ifdef WIN32
-DllExport int rnd_mouse(UINT i, WPARAM w, LPARAM l);
+int rnd_mouse(UINT i, WPARAM w, LPARAM l);
 #endif /* WIN32 */
 
-DllExport BUFFER *buf_new(void);
-DllExport int buf_free(BUFFER *buf);
-DllExport int buf_read(BUFFER *message, FILE *infile);
-DllExport int buf_write(BUFFER *message, FILE *outfile);
-DllExport int buf_write_sync(BUFFER *message, FILE *outfile);
+BUFFER *buf_new(void);
+int buf_free(BUFFER *buf);
+int buf_read(BUFFER *message, FILE *infile);
+int buf_write(BUFFER *message, FILE *outfile);
+int buf_write_sync(BUFFER *message, FILE *outfile);
 
 #define MSG_MAIL 1
 #define MSG_POST 2
@@ -846,18 +838,20 @@ DllExport int buf_write_sync(BUFFER *message, FILE *outfile);
 
 extern char MIXDIR[];
 
-DllExport int mix_encrypt(int type, BUFFER *message, char *chain, int numcopies,
+int mix_encrypt(int type, BUFFER *message, char *chain, int numcopies,
 		BUFFER *feedback);
-DllExport int mix_decrypt(BUFFER *message);
-DllExport int mix_send(void);
+int mix_decrypt(BUFFER *message);
+int mix_send(void);
 
 #define FORCE_POOL 1
 #define FORCE_POP3 2
 #define FORCE_DAILY 4
 #define FORCE_MAILIN 8
-DllExport int mix_regular(int force);
-DllExport int mix_daemon(void);
-DllExport int process_mailin(void);
+#define FORCE_STATS 16
+void mix_check_timeskew(void);
+int mix_regular(int force);
+int mix_daemon(void);
+int process_mailin(void);
 
 #ifdef USE_PGP
 
@@ -865,11 +859,11 @@ DllExport int process_mailin(void);
 #define NYM_MODIFY 1
 #define NYM_DELETE 2
 
-DllExport int nym_config(int mode, char *nym, char *nymserver, BUFFER *pseudonym,
+int nym_config(int mode, char *nym, char *nymserver, BUFFER *pseudonym,
 	       char *sendchain, int sendnumcopies, BUFFER *chains,
 	       BUFFER *options);
-DllExport int nym_encrypt(BUFFER *msg, char *nym, int type);
-DllExport int nym_decrypt(BUFFER *msg, char *nym, BUFFER *log);
+int nym_encrypt(BUFFER *msg, char *nym, int type);
+int nym_decrypt(BUFFER *msg, char *nym, BUFFER *log);
 
 #define PGP_SIGN 1
 #define PGP_ENCRYPT 2
@@ -899,11 +893,25 @@ DllExport int nym_decrypt(BUFFER *msg, char *nym, BUFFER *log);
 #define PGP_E_ELG 16
 #define PGP_S_DSA 17
 
-DllExport int pgp_encrypt(int mode, BUFFER *message, BUFFER *encr, BUFFER *sigid,
+int pgp_encrypt(int mode, BUFFER *message, BUFFER *encr, BUFFER *sigid,
 		BUFFER *pass, char *pubring, char *secring);
-DllExport int pgp_decrypt(BUFFER *message, BUFFER *pass, BUFFER *sig, char *pubring,
+int pgp_decrypt(BUFFER *message, BUFFER *pass, BUFFER *sig, char *pubring,
 		char *secring);
-DllExport int pgp_keygen(int algo, int bits, BUFFER *userid, BUFFER *pass,
+int pgp_keygen(int algo, int bits, BUFFER *userid, BUFFER *pass,
 		  char *pubring, char *secring, int remail);
 #endif /* USE_PGP */
+
+
+/* parsedate */
+time_t parsedate(char *p);
+
+
+
+#ifdef WIN32
+
+#define sleep(x) Sleep((x)*1000)
+#define strcasecmp stricmp
+
+#endif /* WIN32 */
+
 #endif /* not _MIXLIB_H */
