@@ -1,4 +1,4 @@
-/* Mixmaster version 2.9  --  (C) 1999 - 2003 Anonymizer Inc. and others.
+/* Mixmaster version 3.0  --  (C) 1999 - 2006 Anonymizer Inc. and others.
 
    Mixmaster may be redistributed and modified under certain conditions.
    This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF
@@ -6,101 +6,102 @@
    details.
 
    Function prototypes
-   $Id: mix3.h 665 2003-11-09 01:47:32Z rabbi $ */
+   $Id: mix3.h 934 2006-06-24 13:40:39Z rabbi $ */
 
 
 #ifndef _MIX3_H
 #define _MIX3_H
-#define COPYRIGHT "Copyright Anonymizer Inc."
+#define COPYRIGHT "Copyright Anonymizer Inc. et al."
 
-#include <stdio.h>
+#include "config.h"
 #include "mix.h"
 
 #ifdef WIN32
-#	ifndef USE_SOCK
-#		define _WINSOCKAPI_		/* don't include winsock */
-#	endif /* not USE_SOCK */
-#	include <windows.h>
-#	ifdef _MSC_VER
-#		define snprintf _snprintf
-#		define vsnprintf _vsnprintf
-#	endif /* _MSC_VER */
-#	define DIRSEP '\\'
-#	define DIRSEPSTR "\\"
+#ifndef USE_SOCK
+#define _WINSOCKAPI_		/* don't include winsock */
+#endif /* not USE_SOCK */
+#include <windows.h>
+#ifdef _MSC
+#define snprintf _snprintf
+#endif /* _MSC */
+#define DIRSEP '\\'
+#define DIRSEPSTR "\\"
 #else /* end of WIN32 */
-#	define DIRSEP '/'
-#	define DIRSEPSTR "/"
+#define DIRSEP '/'
+#define DIRSEPSTR "/"
 #endif /* else if not WIN32 */
 
 #define NOT_IMPLEMENTED {printf("Function not implemented.\n");return -1;}
 #define SECONDSPERDAY 86400
 
+#include <time.h>
+
 /* Dynamically allocated buffers */
 
-DllExport int buf_reset(BUFFER *buffer);
-DllExport int buf_clear(BUFFER *buffer);
-DllExport int buf_append(BUFFER *buffer, byte *mess, int len);
-DllExport int buf_cat(BUFFER *to, BUFFER *from);
-DllExport int buf_set(BUFFER *to, BUFFER *from);
-DllExport int buf_rest(BUFFER *to, BUFFER *from);
-DllExport int buf_appendrnd(BUFFER *to, int n);
-DllExport int buf_appendzero(BUFFER *to, int n);
-DllExport int buf_setc(BUFFER *buf, byte c);
-DllExport int buf_appendc(BUFFER *to, byte b);
-DllExport int buf_setrnd(BUFFER *b, int n);
-DllExport int buf_setf(BUFFER *buffer, char *fmt, ...);
-DllExport int buf_appendf(BUFFER *buffer, char *fmt, ...);
-DllExport int buf_sets(BUFFER *buf, char *s);
-DllExport int buf_appends(BUFFER *buffer, char *s);
-DllExport int buf_nl(BUFFER *buffer);
-DllExport int buf_pad(BUFFER *buffer, int size);
-DllExport int buf_prepare(BUFFER *buffer, int size);
-DllExport int buf_rewind(BUFFER *buffer);
-DllExport int buf_getc(BUFFER *buffer);
-DllExport void buf_ungetc(BUFFER *buffer);
-DllExport int buf_get(BUFFER *buffer, BUFFER *to, int n);
-DllExport int buf_getline(BUFFER *buffer, BUFFER *line);
-DllExport int buf_chop(BUFFER *b);
-DllExport void buf_move(BUFFER *dest, BUFFER *src);
-DllExport byte *buf_data(BUFFER *buffer);
-DllExport int buf_isheader(BUFFER *buffer);
-DllExport int buf_getheader(BUFFER *buffer, BUFFER *field, BUFFER *content);
-DllExport int buf_appendheader(BUFFER *buffer, BUFFER *field, BUFFER *contents);
-DllExport int buf_lookahead(BUFFER *buffer, BUFFER *line);
-DllExport int buf_eq(BUFFER *b1, BUFFER *b2);
-DllExport int buf_ieq(BUFFER *b1, BUFFER *b2);
-DllExport void buf_cut_out(BUFFER *buffer, BUFFER *cut_out, BUFFER *rest,
+int buf_reset(BUFFER *buffer);
+int buf_clear(BUFFER *buffer);
+int buf_append(BUFFER *buffer, byte *mess, int len);
+int buf_cat(BUFFER *to, BUFFER *from);
+int buf_set(BUFFER *to, BUFFER *from);
+int buf_rest(BUFFER *to, BUFFER *from);
+int buf_appendrnd(BUFFER *to, int n);
+int buf_appendzero(BUFFER *to, int n);
+int buf_setc(BUFFER *buf, byte c);
+int buf_appendc(BUFFER *to, byte b);
+int buf_setrnd(BUFFER *b, int n);
+int buf_setf(BUFFER *buffer, char *fmt, ...);
+int buf_appendf(BUFFER *buffer, char *fmt, ...);
+int buf_sets(BUFFER *buf, char *s);
+int buf_appends(BUFFER *buffer, char *s);
+int buf_nl(BUFFER *buffer);
+int buf_pad(BUFFER *buffer, int size);
+int buf_prepare(BUFFER *buffer, int size);
+int buf_rewind(BUFFER *buffer);
+int buf_getc(BUFFER *buffer);
+void buf_ungetc(BUFFER *buffer);
+int buf_get(BUFFER *buffer, BUFFER *to, int n);
+int buf_getline(BUFFER *buffer, BUFFER *line);
+int buf_chop(BUFFER *b);
+void buf_move(BUFFER *dest, BUFFER *src);
+byte *buf_data(BUFFER *buffer);
+int buf_isheader(BUFFER *buffer);
+int buf_getheader(BUFFER *buffer, BUFFER *field, BUFFER *content);
+int buf_appendheader(BUFFER *buffer, BUFFER *field, BUFFER *contents);
+int buf_lookahead(BUFFER *buffer, BUFFER *line);
+int buf_eq(BUFFER *b1, BUFFER *b2);
+int buf_ieq(BUFFER *b1, BUFFER *b2);
+void buf_cut_out(BUFFER *buffer, BUFFER *cut_out, BUFFER *rest,
 		 int from, int len);
 
-DllExport int buf_appendl(BUFFER *b, long l);
-DllExport int buf_appendl_lo(BUFFER *b, long l);
-DllExport long buf_getl(BUFFER *b);
-DllExport long buf_getl_lo(BUFFER *b);
-DllExport int buf_appendi(BUFFER *b, int i);
-DllExport int buf_appendi_lo(BUFFER *b, int i);
-DllExport int buf_geti(BUFFER *b);
-DllExport int buf_geti_lo(BUFFER *b);
+int buf_appendl(BUFFER *b, long l);
+int buf_appendl_lo(BUFFER *b, long l);
+long buf_getl(BUFFER *b);
+long buf_getl_lo(BUFFER *b);
+int buf_appendi(BUFFER *b, int i);
+int buf_appendi_lo(BUFFER *b, int i);
+int buf_geti(BUFFER *b);
+int buf_geti_lo(BUFFER *b);
 
 /* String comparison */
-DllExport int strieq(const char *s1, const char *s2);
-DllExport int strileft(const char *string, const char *keyword);
-DllExport int striright(const char *string, const char *keyword);
-DllExport int strifind(const char *string, const char *keyword);
+int strieq(const char *s1, const char *s2);
+int strileft(const char *string, const char *keyword);
+int striright(const char *string, const char *keyword);
+int strifind(const char *string, const char *keyword);
 
-DllExport int streq(const char *s1, const char *s2);
-DllExport int strfind(const char *string, const char *keyword);
-DllExport int strleft(const char *string, const char *keyword);
+int streq(const char *s1, const char *s2);
+int strfind(const char *string, const char *keyword);
+int strleft(const char *string, const char *keyword);
 
-DllExport void strcatn(char *dest, const char *src, int n);
+void strcatn(char *dest, const char *src, int n);
 
-DllExport int bufleft(BUFFER *b, char *k);
-DllExport int buffind(BUFFER *b, char *k);
-DllExport int bufeq(BUFFER *b, char *k);
+int bufleft(BUFFER *b, char *k);
+int buffind(BUFFER *b, char *k);
+int bufeq(BUFFER *b, char *k);
 
-DllExport int bufileft(BUFFER *b, char *k);
-DllExport int bufifind(BUFFER *b, char *k);
-DllExport int bufiright(BUFFER *b, char *k);
-DllExport int bufieq(BUFFER *b, char *k);
+int bufileft(BUFFER *b, char *k);
+int bufifind(BUFFER *b, char *k);
+int bufiright(BUFFER *b, char *k);
+int bufieq(BUFFER *b, char *k);
 
 /* Utility functions */
 void whoami(char *addr, char *defaultname);
@@ -120,8 +121,13 @@ int file_to_out(const char *name);
 FILE *mix_openfile(const char *name, const char *a);
 FILE *openpipe(const char *prog);
 int closepipe(FILE *fp);
+int maildirWrite(char *maildir, BUFFER *message, int create);
 int write_pidfile(char *pidfile);
 int clear_pidfile(char *pidfile);
+time_t parse_yearmonthday(char* str);
+
+int url_download(char* url, char* dest);
+int download_stats(char *sourcename);
 
 typedef struct {
   char *name;
@@ -139,12 +145,12 @@ int doblock(BUFFER *line, BUFFER *filter, int logandreset);
 int doallow(BUFFER *line, BUFFER *filter);
 int allowmessage(BUFFER *in);
 
-DllExport void errlog(int type, char *format,...);
-DllExport void clienterr(BUFFER *msgbuf, char *err);
-DllExport void logmail(char *mailbox, BUFFER *message);
+void errlog(int type, char *format,...);
+void clienterr(BUFFER *msgbuf, char *err);
+void logmail(char *mailbox, BUFFER *message);
 
-DllExport void mix_status(char *fmt,...);
-DllExport void mix_genericerror(void);
+void mix_status(char *fmt,...);
+void mix_genericerror(void);
 
 #define ERRORMSG 1
 #define WARNING 2
@@ -152,18 +158,18 @@ DllExport void mix_genericerror(void);
 #define LOG 4
 #define DEBUGINFO 5
 
-DllExport int decode(BUFFER *in, BUFFER *out);
-DllExport int encode(BUFFER *b, int linelen);
-DllExport void id_encode(byte id[], byte *s);
-DllExport void id_decode(byte *s, byte id[]);
+int decode(BUFFER *in, BUFFER *out);
+int encode(BUFFER *b, int linelen);
+void id_encode(byte id[], byte *s);
+void id_decode(byte *s, byte id[]);
 
 int decode_header(BUFFER *content);
 int boundary(BUFFER *line, BUFFER *mboundary);
 void get_parameter(BUFFER *content, char *attribute, BUFFER *value);
 int get_type(BUFFER *content, BUFFER *type, BUFFER *subtype);
 int mail_encode(BUFFER *in, int encoding);
-DllExport int hdr_encode(BUFFER *in, int n);
-DllExport int attachfile(BUFFER *message, BUFFER *filename);
+int hdr_encode(BUFFER *in, int n);
+int attachfile(BUFFER *message, BUFFER *filename);
 int pgpmime_sign(BUFFER *message, BUFFER *uid, BUFFER *pass, char *secring);
 int mime_attach(BUFFER *message, BUFFER *attachment, BUFFER *type);
 void mimedecode(BUFFER *msg);
@@ -173,16 +179,16 @@ int qp_decode_message(BUFFER *msg);
 #define MIME_7BIT 2   /* transport is 7bit */
 
 /* randomness */
-DllExport int rnd_bytes(byte *b, int n);
-DllExport byte rnd_byte(void);
-DllExport int rnd_number(int n);
-DllExport int rnd_add(byte *b, int l);
-DllExport int rnd_seed(void);
-DllExport void rnd_time(void);
+int rnd_bytes(byte *b, int n);
+byte rnd_byte(void);
+int rnd_number(int n);
+int rnd_add(byte *b, int l);
+int rnd_seed(void);
+void rnd_time(void);
 
-DllExport int rnd_init(void);
-DllExport int rnd_final(void);
-DllExport void rnd_error(void);
+int rnd_init(void);
+int rnd_final(void);
+void rnd_error(void);
 
 #define RND_QUERY 0
 #define RND_NOTSEEDED -1
@@ -203,7 +209,8 @@ int digestmem_md5(byte *b, int n, BUFFER *md);
 int digest_sha1(BUFFER *b, BUFFER *md);
 int digest_rmd160(BUFFER *b, BUFFER *md);
 
-DllExport int keymgt(int force);
+#define KEY_ID_LEN 32
+int keymgt(int force);
 int key(BUFFER *b);
 int adminkey(BUFFER *b);
 
@@ -228,11 +235,12 @@ int pk_encrypt(BUFFER *plaintext, BUFFER *privkey);
 int check_seckey(BUFFER *buf, const byte id[]);
 int check_pubkey(BUFFER *buf, const byte id[]);
 int v2createkey(void);
+int getv2seckey(byte keyid[], BUFFER *key);
 int seckeytopub(BUFFER *pub, BUFFER *sec, byte keyid[]);
 
 /* configuration, general remailer functions */
-DllExport int mix_configline(char *line);
-DllExport int mix_config(void);
+int mix_configline(char *line);
+int mix_config(void);
 int mix_initialized(void);
 int mix_daily(void);
 
@@ -240,13 +248,14 @@ int mix_daily(void);
 #define INTERMEDIATE 0
 int pool_send(void);
 int pool_read(BUFFER *pool);
-DllExport int pool_add(BUFFER *msg, char *type);
+int pool_add(BUFFER *msg, char *type);
 FILE *pool_new(char *type, char *tmpname, char *path);
 int mix_pool(BUFFER *msg, int type, long latent);
 int pool_packetfile(char *fname, BUFFER *mid, int packetnum);
 void pool_packetexp(void);
 int idexp(void);
-DllExport void pop3get(void);
+int pgpmaxexp(void);
+void pop3get(void);
 
 typedef struct {  /* added for binary id.log change */
   char id[16];
@@ -265,7 +274,12 @@ int stats_out(int);
 #define PGP_ARMOR_NYMSIG        4
 #define PGP_ARMOR_SECKEY        5
 
+#define PGP_TYPE_UNDEFINED	0
+#define PGP_TYPE_PRIVATE	1
+#define PGP_TYPE_PUBLIC		2
+
 int pgp_keymgt(int force);
+int pgp_latestkeys(BUFFER* outtxt, int algo);
 int pgp_armor(BUFFER *buf, int mode);
 int pgp_dearmor(BUFFER *buf, BUFFER *out);
 int pgp_pubkeycert(BUFFER *userid, char *keyring, BUFFER *pass,
@@ -273,20 +287,20 @@ int pgp_pubkeycert(BUFFER *userid, char *keyring, BUFFER *pass,
 int pgp_signtxt(BUFFER *msg, BUFFER *uid, BUFFER *pass,
 		char *secring, int remail);
 int pgp_isconventional(BUFFER *buf);
-DllExport int pgp_mailenc(int mode, BUFFER *msg, char *sigid,
+int pgp_mailenc(int mode, BUFFER *msg, char *sigid,
 		BUFFER *pass, char *pubring, char *secring);
 int pgp_signhashalgo(BUFFER *algo, BUFFER *userid, char *secring,
 		     BUFFER *pass);
 
 /* menu */
-DllExport int menu_initialized;
-DllExport void menu_main(void);
-DllExport void menu_folder(char command, char *name);
-DllExport int menu_getuserpass(BUFFER *p, int mode);
+int menu_initialized;
+void menu_main(void);
+void menu_folder(char command, char *name);
+int menu_getuserpass(BUFFER *p, int mode);
 
-DllExport int user_pass(BUFFER *b);
-DllExport int user_confirmpass(BUFFER *b);
-DllExport void user_delpass(void);
+int user_pass(BUFFER *b);
+int user_confirmpass(BUFFER *b);
+void user_delpass(void);
 
 /* remailer */
 typedef struct {
@@ -311,6 +325,8 @@ typedef struct {
 
     unsigned int post:1;
     unsigned int middle:1;
+
+    unsigned int star_ex:1;
   } flags;
   struct rinfo {
     int reliability;
@@ -321,21 +337,24 @@ typedef struct {
 
 #define CHAINMAX 421
 #define MAXREM 100
-DllExport int prepare_type2list(BUFFER *out);
-int mix2_rlist(REMAILER remailer[]);
-int t1_rlist(REMAILER remailer[]);
+int prepare_type2list(BUFFER *out);
+int mix2_rlist(REMAILER remailer[], int badchains[MAXREM][MAXREM]);
+int t1_rlist(REMAILER remailer[], int badchains[MAXREM][MAXREM]);
 int pgp_rlist(REMAILER remailer[], int n);
 int pgp_rkeylist(REMAILER remailer[], int keyid[], int n);
+void parse_badchains(int badchains[MAXREM][MAXREM], char *file, char *startindicator, REMAILER *remailer, int maxrem);
 int chain_select(int hop[], char *chainstr, int maxrem, REMAILER *remailer,
 		 int type, BUFFER *feedback);
-int chain_rand(REMAILER *remailer, int maxrem,
-	       int thischain[], int chainlen, int t);
-int chain_randfinal(int type, REMAILER *remailer, int maxrem, int t);
+int chain_rand(REMAILER *remailer, int badchains[MAXREM][MAXREM], int maxrem,
+	       int thischain[], int chainlen, int t, int ignore_constraints_if_necessary);
+int chain_randfinal(int type, REMAILER *remailer, int badchains[MAXREM][MAXREM],
+	       int maxrem, int rtype, int chain[], int chainlen, int ignore_constraints_if_necessary);
 
 float chain_reliability(char *chain, int chaintype,
 			char *reliability_string);
+int redirect_message(BUFFER *sendmsg, char *chain, int numcopies, BUFFER *chainlist);
 int mix2_encrypt(int type, BUFFER *message, char *chainstr, int numcopies,
-		 BUFFER *feedback);
+		int ignore_constraints_if_necessary, BUFFER *feedback);
 int t1_encrypt(int type, BUFFER *message, char *chainstr, int latency,
 	       BUFFER *ek, BUFFER *feedback);
 
@@ -345,13 +364,6 @@ int t1_decrypt(BUFFER *in);
 int t2_decrypt(BUFFER *in);
 
 int mix2_decrypt(BUFFER *m);
-int mix3_decrypt(BUFFER *m);
-
-/* type 2 */
-#define MAGIC0 0
-#define MAGIC1 3
-
-int v3_magic(byte *b);
 int v2body(BUFFER *body);
 int v2body_setlen(BUFFER *body);
 int v2partial(BUFFER *body, BUFFER *mid, int packet, int numpackets);
@@ -384,17 +396,17 @@ int nymlist_getnym(char *nym, BUFFER *config, BUFFER *ek, BUFFER *opt,
 int nymlist_getstatus(char *nym);
 
 /* Visual C lacks dirent */
-#ifdef _MSC_VER
+#ifdef _MSC
 typedef HANDLE DIR;
 
 struct dirent {
-  char d_name[MAX_PATH];
+  char d_name[PATHMAX];
 };
 
 DIR *opendir(const char *name);
 struct dirent *readdir(DIR *dir);
 int closedir(DIR *dir);
-#endif /* _MSC_VER */
+#endif /* _MSC */
 
 /* sockets */
 #if defined(WIN32) && defined(USE_SOCK)
@@ -412,8 +424,8 @@ int closesocket(SOCKET s);
 #endif /* else if not defined(WIN32) && defined(USE_SOCK) */
 
 #ifdef WIN32
-DllExport int is_nt_service(void);
-DllExport void set_nt_exit_event();
+int is_nt_service(void);
+void set_nt_exit_event();
 #endif /* WIN32 */
 
 /* check for memory leaks */
