@@ -1,4 +1,4 @@
-/* Mixmaster version 2.9  --  (C) 1999 - 2002 Anonymizer Inc. and others.
+/* Mixmaster version 3.0  --  (C) 1999 - 2006 Anonymizer Inc. and others.
 
    Mixmaster may be redistributed and modified under certain conditions.
    This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF
@@ -6,9 +6,11 @@
    details.
 
    Create nym server messages
-   $Id: nym.c 651 2003-11-07 06:52:03Z weasel $ */
+   $Id: nym.c 934 2006-06-24 13:40:39Z rabbi $ */
+
+
+#ifdef NYMSUPPORT
 
-#include "mix.h"
 #include "mix3.h"
 #include "pgp.h"
 #include <string.h>
@@ -23,6 +25,7 @@ int nym_config(int mode, char *nym, char *nymserver, BUFFER *pseudonym,
   return (-1);
 #else /* end of not USE_PGP */
   REMAILER remailer[MAXREM];
+  int badchains[MAXREM][MAXREM];
   KEYRING *r;
   int maxrem;
   int chain[20];
@@ -60,13 +63,13 @@ int nym_config(int mode, char *nym, char *nymserver, BUFFER *pseudonym,
   }
 
   if (nymserver) {
-    maxrem = t1_rlist(remailer);
+    maxrem = t1_rlist(remailer, badchains);
     if (maxrem < 1)
       return (-1);
     if (chain_select(chain, nymserver, maxrem, remailer, 2, NULL) != 1)
       return (-1);
     if (chain[0] == 0)
-      chain[0] = chain_randfinal(MSG_MAIL, remailer, maxrem, 2);
+      chain[0] = chain_randfinal(MSG_MAIL, remailer, badchains, maxrem, 2, NULL, -1);
     if (chain[0] == -1)
       return (-1);
     assert(strchr(nym, '@') == NULL && strchr(remailer[chain[0]].addr, '@'));
@@ -108,7 +111,7 @@ int nym_config(int mode, char *nym, char *nymserver, BUFFER *pseudonym,
     buf_appends(req, "Public-Key:\n");
 
   getkey:
-    r = pgpdb_open(NYMSECRING, userpass, 0);
+    r = pgpdb_open(NYMSECRING, userpass, 0, PGP_TYPE_PRIVATE);
     if (r == NULL) {
       err = -3;
       goto end;
@@ -662,3 +665,5 @@ int nymlist_getstatus(char *nym)
   else
     return (-1);
 }
+
+#endif /* NYMSUPPORT */
