@@ -1,4 +1,4 @@
-/* Mixmaster version 2.9  --  (C) 1999 - 2003 Anonymizer Inc. and others.
+/* Mixmaster version 3.0  --  (C) 1999 - 2006 Anonymizer Inc. and others.
 
    Mixmaster may be redistributed and modified under certain conditions.
    This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF
@@ -6,10 +6,9 @@
    details.
 
    OpenPGP messages
-   $Id: pgp.c 665 2003-11-09 01:47:32Z rabbi $ */
+   $Id: pgp.c 934 2006-06-24 13:40:39Z rabbi $ */
 
 
-#include "mix.h"
 #include "mix3.h"
 #ifdef USE_PGP
 #include "pgp.h"
@@ -177,7 +176,7 @@ int pgp_encrypt(int mode, BUFFER *in, BUFFER *to, BUFFER *sigid,
 		BUFFER *pass, char *pubring, char *secring)
 {
   BUFFER *dek, *out, *sig, *dest, *tmp;
-  int err = 0, sym = 0, mdc = 0;
+  int err = 0, sym = PGP_K_ANY, mdc = 0;
   int text;
 
   out = buf_new();
@@ -208,7 +207,7 @@ int pgp_encrypt(int mode, BUFFER *in, BUFFER *to, BUFFER *sigid,
     if (err == -1)
       goto end;
     if (to->ptr == to->length) {
-      if ((err = pgpdb_getkey(PK_ENCRYPT, PGP_ANY, &sym, &mdc, NULL, dest, NULL,
+      if ((err = pgpdb_getkey(PK_ENCRYPT, PGP_ANY, &sym, &mdc, NULL, NULL, dest, NULL,
 			      NULL, pubring, NULL)) < 0)
 	goto end;
       pgp_setkey(dek, sym);
@@ -290,10 +289,10 @@ unsigned long crc24(BUFFER * in)
     for (i = 0; i < 8; i++) {
       crc <<= 1;
       if (crc & 0x1000000)
-       crc ^= POLY;
+	crc ^= POLY;
     }
   }
-#else /* end of 0 */
+#else
   /* pre-computed CRC table -- much faster */
   unsigned long table[256];
   unsigned long t;
@@ -312,7 +311,7 @@ unsigned long crc24(BUFFER * in)
   }
   for (p = 0; p < in->length; p++)
     crc = crc << 8 ^ table[(in->data[p] ^ crc >> 16) & 255];
-#endif /* end of not 0 */
+#endif
   return crc & ((1<<24)-1);
 }
 
@@ -442,13 +441,11 @@ int pgp_keygen(int algo, int bits, BUFFER *userid, BUFFER *pass, char *pubring,
 	       char *secring, int remail)
 {
   switch (algo) {
-#ifdef USE_RSA
   case PGP_ES_RSA:
 #ifndef USE_IDEA
     errlog(WARNING, "IDEA disabled: OpenPGP RSA key cannot be used for decryption!\n");
 #endif
     return (pgp_rsakeygen(bits, userid, pass, pubring, secring, remail));
-#endif /* USE_RSA */
   case PGP_E_ELG:
     return (pgp_dhkeygen(bits, userid, pass, pubring, secring, remail));
   default:
@@ -468,7 +465,7 @@ int pgp_signtxt(BUFFER *msg, BUFFER *uid, BUFFER *pass,
 
   buf_appends(out, begin_pgpsigned);
   buf_nl(out);
-  if (pgpdb_getkey(PK_SIGN, PGP_ANY, NULL, NULL, NULL, uid, NULL, NULL, secring, pass) == PGP_S_DSA)
+  if (pgpdb_getkey(PK_SIGN, PGP_ANY, NULL, NULL, NULL, NULL, uid, NULL, NULL, secring, pass) == PGP_S_DSA)
     buf_appends(out, "Hash: SHA1\n");
   buf_nl(out);
   while (buf_getline(msg, line) != -1) {
