@@ -1,4 +1,4 @@
-/* Mixmaster version 2.9  --  (C) 1999 - 2003 Anonymizer Inc. and others.
+/* Mixmaster version 3.0  --  (C) 1999 - 2006 Anonymizer Inc. and others.
 
    Mixmaster may be redistributed and modified under certain conditions.
    This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF
@@ -6,7 +6,7 @@
    details.
 
    OpenPGP messages
-   $Id: pgp.h 665 2003-11-09 01:47:32Z rabbi $ */
+   $Id: pgp.h 934 2006-06-24 13:40:39Z rabbi $ */
 
 
 #ifdef USE_PGP
@@ -43,6 +43,7 @@
 #define PGP_MDC 19
 
 /* symmetric algorithms */
+#define PGP_K_ANY 0
 #define PGP_K_IDEA 1
 #define PGP_K_3DES 2
 #define PGP_K_CAST5 3
@@ -60,12 +61,22 @@
 #define PGP_SIG_BINARY 0
 #define PGP_SIG_CANONIC 1
 #define PGP_SIG_CERT 0x10
+#define PGP_SIG_CERT1 0x11
+#define PGP_SIG_CERT2 0x12
+#define PGP_SIG_CERT3 0x13
+#define isPGP_SIG_CERT(x) (x >= PGP_SIG_CERT && x <= PGP_SIG_CERT3)
 #define PGP_SIG_BINDSUBKEY 0x18
+#define PGP_SIG_KEYREVOKE 0x20
+#define PGP_SIG_SUBKEYREVOKE 0x28
+#define PGP_SIG_CERTREVOKE 0x30
 
 /* signature subpacket types */
 #define PGP_SUB_CREATIME 2
+#define PGP_SUB_CERTEXPIRETIME 3
+#define PGP_SUB_KEYEXPIRETIME 9
 #define PGP_SUB_PSYMMETRIC 11
 #define PGP_SUB_ISSUER 16
+#define PGP_SUB_PRIMARY 25
 #define PGP_SUB_FEATURES 30
 
 #define ARMORED 1
@@ -104,7 +115,9 @@ int pgp_rsa(BUFFER *buf, BUFFER *key, int mode);
 void pgp_sigcanonic(BUFFER *msg);
 int pgp_makepubkey(BUFFER *seckey, BUFFER *outtxt, BUFFER *pubkey,
 		   BUFFER *pass, int keyalgo);
-int pgp_getkey(int mode, int algo, int *sym, int *mdc, BUFFER *keypacket, BUFFER *key,
+int pgp_makekeyheader(int type, BUFFER *keypacket, BUFFER *outtxt,
+                   BUFFER *pass, int keyalgo);
+int pgp_getkey(int mode, int algo, int *sym, int *mdc, long *expires, BUFFER *keypacket, BUFFER *key,
 	       BUFFER *keyid, BUFFER *userid, BUFFER *pass);
 int pgp_rsakeygen(int bits, BUFFER *userid, BUFFER *pass, char *pubring,
 		  char *secring, int remail);
@@ -150,7 +163,7 @@ int pgp_digest(int hashalgo, BUFFER *in, BUFFER *d);
 
 /* pgpdb.c */
 
-int pgpdb_getkey(int mode, int algo, int *sym, int *mdc, BUFFER *key, BUFFER *user,
+int pgpdb_getkey(int mode, int algo, int *sym, int *mdc, long *expires, BUFFER *key, BUFFER *user,
 		 BUFFER *founduid, BUFFER *keyid, char *keyring, BUFFER *pass);
 
 typedef struct {
@@ -158,12 +171,16 @@ typedef struct {
   BUFFER *db;
   LOCK *lock;
   int modified;
+  int type; /* undefined, public, private */
   char filename[LINELEN];
   BUFFER *encryptkey;
+#ifndef NDEBUG
+  int writer;
+#endif
 } KEYRING;
 
-KEYRING *pgpdb_new(char *keyring, int filetype, BUFFER *encryptkey);
-KEYRING *pgpdb_open(char *keyring, BUFFER *encryptkey, int writer);
+KEYRING *pgpdb_new(char *keyring, int filetype, BUFFER *encryptkey, int type);
+KEYRING *pgpdb_open(char *keyring, BUFFER *encryptkey, int writer, int type);
 int pgpdb_append(KEYRING *keydb, BUFFER *p);
 int pgpdb_getnext(KEYRING *keydb, BUFFER *p, BUFFER *keyid, BUFFER *userid);
 int pgpdb_close(KEYRING *keydb);
