@@ -1,4 +1,4 @@
-/* Mixmaster version 2.9  --  (C) 1999 - 2003 Anonymizer Inc. and others.
+/* Mixmaster version 3.0  --  (C) 1999 - 2006 Anonymizer Inc. and others.
 
    Mixmaster may be redistributed and modified under certain conditions.
    This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF
@@ -6,10 +6,9 @@
    details.
 
    Create OpenPGP packets
-   $Id: pgpcreat.c 665 2003-11-09 01:47:32Z rabbi $ */
+   $Id: pgpcreat.c 934 2006-06-24 13:40:39Z rabbi $ */
 
 
-#include "mix.h"
 #include "mix3.h"
 #ifdef USE_PGP
 #include "pgp.h"
@@ -455,7 +454,7 @@ int pgp_sessionkey(BUFFER *out, BUFFER *user, BUFFER *keyid, BUFFER *seskey,
     tempbuf = 1;
   }
   sym = seskey->data[0];
-  if ((algo = pgpdb_getkey(PK_ENCRYPT, PGP_ANY, &sym, NULL, key, user, NULL, keyid,
+  if ((algo = pgpdb_getkey(PK_ENCRYPT, PGP_ANY, &sym, NULL, NULL, key, user, NULL, keyid,
 			   pubring, NULL)) == -1)
     goto end;
 
@@ -470,12 +469,10 @@ int pgp_sessionkey(BUFFER *out, BUFFER *user, BUFFER *keyid, BUFFER *seskey,
   buf_appendi(encrypt, csum);
 
   switch (algo) {
-#ifdef USE_RSA
   case PGP_ES_RSA:
     err = pgp_rsa(encrypt, key, PK_ENCRYPT);
     mpi_put(out, encrypt);
     break;
-#endif /* USE_RSA */
    case PGP_E_ELG:
     err = pgp_elgencrypt(encrypt, key);
     buf_cat(out, encrypt);
@@ -629,7 +626,7 @@ int pgp_signhashalgo(BUFFER *algo, BUFFER *userid, char *secring, BUFFER *pass)
 {
   int pkalgo;
 
-  pkalgo = pgpdb_getkey(PK_SIGN, PGP_ANY, NULL, NULL, NULL, userid, NULL, NULL,
+  pkalgo = pgpdb_getkey(PK_SIGN, PGP_ANY, NULL, NULL, NULL, NULL, userid, NULL, NULL,
 			secring, pass);
   if (pkalgo == PGP_S_DSA)
     buf_sets(algo, "sha1");
@@ -671,26 +668,29 @@ int pgp_sign(BUFFER *msg, BUFFER *msg2, BUFFER *sig, BUFFER *userid,
   }
   if (keypacket) {
     buf_rewind(keypacket);
-    algo = pgp_getkey(PK_SIGN, PGP_ANY, NULL, NULL, keypacket, key, id, NULL, pass);
+    algo = pgp_getkey(PK_SIGN, PGP_ANY, NULL, NULL, NULL, keypacket, key, id, NULL, pass);
   } else
-    algo = pgpdb_getkey(PK_SIGN, PGP_ANY, NULL, NULL, key, userid, NULL, id, secring,
+    algo = pgpdb_getkey(PK_SIGN, PGP_ANY, NULL, NULL, NULL, key, userid, NULL, id, secring,
 			pass);
   if (algo <= -1) {
     err = algo;
     goto end;
   }
-  if (algo == PGP_S_DSA)
+  if (algo == PGP_S_DSA || algo == PGP_E_ELG)
     version = 4;
   if (version == 3)
     hashalgo = PGP_H_MD5;
   else
     hashalgo = PGP_H_SHA1;
 
-  if (!self)
+  if (!self && type != PGP_SIG_BINDSUBKEY)
     version = 3;
 
   switch (type) {
    case PGP_SIG_CERT:
+   case PGP_SIG_CERT1:
+   case PGP_SIG_CERT2:
+   case PGP_SIG_CERT3:
      type1 = pgp_getpacket(msg, d) == PGP_PUBKEY;
      assert (type1);
      buf_setc(msg, 0x99);
@@ -764,6 +764,16 @@ int pgp_sign(BUFFER *msg, BUFFER *msg2, BUFFER *sig, BUFFER *userid,
     pgp_subpacket(d, PGP_SUB_CREATIME);
     buf_cat(sub, d);
 
+    if (self || type == PGP_SIG_BINDSUBKEY) {
+      /* until we can handle the case where our pgp keys expire, don't create keys that expire */
+      if (0 && KEYLIFETIME) { /* add key expirtaion time */
+	buf_clear(d);
+	buf_appendl(d, KEYLIFETIME);
+	pgp_subpacket(d, PGP_SUB_KEYEXPIRETIME);
+	buf_cat(sub, d);
+      }
+    }
+
     if (self) {
       buf_setc(d, PGP_K_CAST5);
 #ifdef USE_AES
@@ -821,7 +831,7 @@ int pgp_pubkeycert(BUFFER *userid, char *keyring, BUFFER *pass,
   int err = -1;
 
   key = buf_new();
-  r = pgpdb_open(keyring, pass, 0);
+  r = pgpdb_open(keyring, pass, 0, PGP_TYPE_UNDEFINED);
   if (r != NULL)
     while (pgpdb_getnext(r, key, NULL, userid) != -1) {
       if (pgp_makepubkey(key, NULL, out, pass, 0) != -1)
