@@ -1,4 +1,4 @@
-/* Mixmaster version 2.9  --  (C) 1999 - 2003 Anonymizer Inc. and others.
+/* Mixmaster version 3.0  --  (C) 1999 - 2006 Anonymizer Inc. and others.
 
    Mixmaster may be redistributed and modified under certain conditions.
    This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF
@@ -6,9 +6,9 @@
    details.
 
    OpenPGP key database
-   $Id: pgpdb.c 665 2003-11-09 01:47:32Z rabbi $ */
+   $Id: pgpdb.c 934 2006-06-24 13:40:39Z rabbi $ */
+
 
-#include "mix.h"
 #include "mix3.h"
 #ifdef USE_PGP
 #include "pgp.h"
@@ -61,16 +61,21 @@ static int pgp_readkeyring(BUFFER *keys, char *filename)
 end_greedy_dearmor:
     buf_free(line);
     buf_free(tmp);
+
   }
   buf_free(armored);
   return (err);
 }
 
-KEYRING *pgpdb_open(char *keyring, BUFFER *encryptkey, int writer)
+KEYRING *pgpdb_open(char *keyring, BUFFER *encryptkey, int writer, int type)
 {
   KEYRING *keydb;
 
-  keydb = pgpdb_new(keyring, -1, encryptkey);
+  assert(! ((writer) && (type == PGP_TYPE_UNDEFINED)));
+  keydb = pgpdb_new(keyring, -1, encryptkey, type);
+#ifndef NDEBUG
+  keydb->writer = writer;
+#endif
   if (writer)
     keydb->lock = lockfile(keyring);
   keydb->filetype = pgp_readkeyring(keydb->db, keyring);
@@ -88,7 +93,7 @@ KEYRING *pgpdb_open(char *keyring, BUFFER *encryptkey, int writer)
   return (keydb);
 }
 
-KEYRING *pgpdb_new(char *keyring, int filetype, BUFFER *encryptkey)
+KEYRING *pgpdb_new(char *keyring, int filetype, BUFFER *encryptkey, int type)
 {
   KEYRING *keydb;
 
@@ -99,6 +104,7 @@ KEYRING *pgpdb_new(char *keyring, int filetype, BUFFER *encryptkey)
   keydb->db = buf_new();
   keydb->modified = 0;
   keydb->lock = NULL;
+  keydb->type = type;
   strncpy(keydb->filename, keyring, sizeof(keydb->filename));
   keydb->filetype = filetype;
   if (encryptkey == NULL)
@@ -116,12 +122,15 @@ int pgpdb_close(KEYRING *keydb)
 
   if (keydb->modified) {
     FILE *f;
-
+#ifndef ndebug
+    assert(keydb->writer);
+#endif
     if (keydb->encryptkey && keydb->encryptkey->length)
       pgp_encrypt(PGP_NCONVENTIONAL | PGP_NOARMOR, keydb->db,
 		  keydb->encryptkey, NULL, NULL, NULL, NULL);
+    assert(keydb->type == PGP_TYPE_PRIVATE || keydb->type == PGP_TYPE_PUBLIC);
     if (keydb->filetype == ARMORED)
-      pgp_armor(keydb->db, PGP_ARMOR_KEY);
+      pgp_armor(keydb->db, keydb->type == PGP_TYPE_PUBLIC ? PGP_ARMOR_KEY : PGP_ARMOR_SECKEY);
     if (keydb->filetype == -1 || (f = mix_openfile(keydb->filename,
 						   keydb->filetype ==
 						   ARMORED ? "w" : "wb"))
@@ -217,6 +226,9 @@ int pgpdb_getnext(KEYRING *keydb, BUFFER *key, BUFFER *keyid, BUFFER *userid)
 int pgpdb_append(KEYRING *keydb, BUFFER *p)
 {
   assert(keydb->lock);
+#ifndef ndebug
+  assert(keydb->writer);
+#endif
   buf_cat(keydb->db, p);
   keydb->modified = 1;
   return (0);
@@ -224,8 +236,10 @@ int pgpdb_append(KEYRING *keydb, BUFFER *p)
 
 #define pgp_preferredalgo PGP_ES_RSA
 
-int pgpdb_getkey(int mode, int algo, int *sym, int *mdc, BUFFER *key, BUFFER *userid,
+int pgpdb_getkey(int mode, int algo, int *sym, int *mdc, long *expires, BUFFER *key, BUFFER *userid,
 		 BUFFER *founduid, BUFFER *keyid, char *keyring, BUFFER *pass)
+/* FIXME: This could be changed to return the key with the latest expiration date if
+ *        a key is not unique */
 {
   KEYRING *r;
   BUFFER *id, *thisid, *thiskey;
@@ -236,19 +250,19 @@ int pgpdb_getkey(int mode, int algo, int *sym, int *mdc, BUFFER *key, BUFFER *us
   thisid = buf_new();
   thiskey = buf_new();
   if (keyring)
-    r = pgpdb_open(keyring, pass, 0);
+    r = pgpdb_open(keyring, pass, 0, PGP_TYPE_UNDEFINED);
   else
     switch (mode) {
     case PK_DECRYPT:
     case PK_SIGN:
-      r = pgpdb_open(PGPREMSECRING, NULL, 0);
+      r = pgpdb_open(PGPREMSECRING, NULL, 0, PGP_TYPE_PRIVATE);
       break;
     case PK_ENCRYPT:
     case PK_VERIFY:
-      r = pgpdb_open(PGPREMPUBRING, NULL, 0);
+      r = pgpdb_open(PGPREMPUBASC, NULL, 0, PGP_TYPE_PUBLIC);
       if (r != NULL && r->filetype == -1) {
 	pgpdb_close(r);
-	r = pgpdb_open(PGPREMPUBASC, NULL, 0);
+	r = pgpdb_open(PGPREMPUBRING, NULL, 0, PGP_TYPE_PUBLIC);
       }
       break;
     default:
@@ -263,7 +277,7 @@ int pgpdb_getkey(int mode, int algo, int *sym, int *mdc, BUFFER *key, BUFFER *us
       break;
     if (keyid) /* pgp_getkey has to chose subkey with given keyid */
       buf_set(thisid, keyid);
-    thisalgo = pgp_getkey(mode, algo, sym, mdc, thiskey, thiskey, thisid, founduid,
+    thisalgo = pgp_getkey(mode, algo, sym, mdc, expires, thiskey, thiskey, thisid, founduid,
 			  pass);
     if (thisalgo == PGP_PASS)
       needpass = 1;
@@ -316,12 +330,19 @@ end:
 int pgp_keymgt(int force)
 {
   FILE *f = NULL;
-  BUFFER *key, *userid, *out, *outkey, *outtxt, *pass;
+  BUFFER *key, *keybak, *userid, *out, *outkey, *outtxt, *pass, *secout;
   KEYRING *keys;
-  int err = 0, type = 0;
+  int err = 0, res, recreate_pubring = 0, dsa_ok = 0;
+#ifdef USE_IDEA
+  int rsa_ok = 0;
+#endif /* USE_IDEA */
+  long expires;
+  LOCK *seclock;
 
   key = buf_new();
   out = buf_new();
+  keybak = buf_new();
+  secout = buf_new();
 
   userid = buf_new();
   buf_sets(userid, REMAILERNAME);
@@ -330,53 +351,113 @@ int pgp_keymgt(int force)
   outtxt = buf_new();
   outkey = buf_new();
 
-#ifdef USE_RSA
   /* We only want to build RSA keys if we also can do IDEA
    * This is to not lose any mail should users try our RSA key
    * with IDEA.
    */
 #ifdef USE_IDEA
-  if (force == 2 || (pgpdb_getkey(PK_DECRYPT, PGP_ES_RSA, NULL, NULL, NULL, NULL,
-				  NULL, NULL, NULL, pass) < 0))
+  /* FIXME: pgpdb_getky returns the expiration date from the last key in the keyring
+   *        which probably works most of the time if the keys are in the correct order
+   *        it doesn't return the latest expiration date (or 0) if the key in question
+   *        is before another matching key in the keyring tho
+   */
+  res = pgpdb_getkey(PK_DECRYPT, PGP_ES_RSA, NULL, NULL, &expires, NULL, NULL,
+				  NULL, NULL, NULL, pass);
+  if (force == 2 || res < 0 || (expires > 0 && expires - KEYOVERLAPPERIOD < time(NULL))) {
+    rsa_ok = -1;
     pgp_keygen(PGP_ES_RSA, 0, userid, pass, PGPKEY, PGPREMSECRING, 0);
+  };
 
-  if (force == 0 && (pgpdb_getkey(PK_ENCRYPT, PGP_ES_RSA, NULL, NULL, NULL, NULL,
-				  NULL, NULL, PGPKEY, NULL) < 0))
-    goto end;
+  if (force == 0 && (pgpdb_getkey(PK_ENCRYPT, PGP_ES_RSA, NULL, NULL, NULL, NULL, NULL,
+				  NULL, NULL, PGPKEY, NULL) < 0) && rsa_ok == 0)
+    rsa_ok = 1;
 #endif /* USE_IDEA */
-#endif /* USE_RSA */
-  if (force == 2 || (pgpdb_getkey(PK_DECRYPT, PGP_E_ELG, NULL, NULL, NULL, NULL,
-				  NULL, NULL, NULL, pass) < 0))
+  /* FIXME: pgpdb_getky returns the expiration date from the last key in the keyring
+   *        which probably works most of the time if the keys are in the correct order
+   *        it doesn't return the latest expiration date (or 0) if the key in question
+   *        is before another matching key in the keyring tho
+   */
+  res = pgpdb_getkey(PK_DECRYPT, PGP_E_ELG, NULL, NULL, &expires, NULL, NULL,
+				  NULL, NULL, NULL, pass);
+  if (force == 2 || res < 0 || (expires > 0 && expires - KEYOVERLAPPERIOD < time(NULL))) {
+    dsa_ok = -1;
     pgp_keygen(PGP_E_ELG, 0, userid, pass, PGPKEY, PGPREMSECRING, 0);
+  }
 
-  if (force == 0 && (pgpdb_getkey(PK_ENCRYPT, PGP_E_ELG, NULL, NULL, NULL, NULL,
-				  NULL, NULL, PGPKEY, NULL) > 0))
+  if (force == 0 && (pgpdb_getkey(PK_ENCRYPT, PGP_E_ELG, NULL, NULL, NULL, NULL, NULL,
+				  NULL, NULL, PGPKEY, NULL) > 0) && dsa_ok == 0)
+    dsa_ok = 1;
+
+  /* No need to rewrite the files - we didn't change a thing */
+  if (
+#ifdef USE_IDEA
+      rsa_ok == 1 &&
+#endif /* USE_IDEA */
+      dsa_ok == 1)
     goto end;
 
-  /* write RSA and DSA/ElGamal keys separately to make old PGP
-     versions happy */
+  /* write keys one key per armor to make hand editing easy and old PGP
+   * versions happy */
   err = -1;
-  for (type = 0; type < 2; type++) {
-    keys = pgpdb_open(PGPREMSECRING, NULL, 0);
-    if (keys == NULL)
-      goto end;
+  keys = pgpdb_open(PGPKEY, NULL, 0, PGP_TYPE_PUBLIC);
+  if (keys == NULL)
+    recreate_pubring = 1;
+  else {
     while (pgpdb_getnext(keys, key, NULL, userid) != -1) {
-      buf_clear(outkey);
       buf_clear(outtxt);
-      if (pgp_makepubkey(key, outtxt, outkey, pass,
-			 type == 0 ? PGP_ES_RSA : PGP_S_DSA) == 0) {
+      if (pgp_makekeyheader(PGP_PUBKEY, key, outtxt, NULL, PGP_ANY) == 0) {
 	err = 0;
-	buf_appends(out, "Type Bits/KeyID    Date       User ID\n");
+	buf_appends(out, "Type Bits/KeyID     Date       User ID\n");
 	buf_cat(out, outtxt);
-	pgp_armor(outkey, PGP_ARMOR_KEY);
 	buf_nl(out);
-	buf_cat(out, outkey);
+	pgp_armor(key, PGP_ARMOR_KEY);
+	buf_cat(out, key);
 	buf_nl(out);
       }
     }
     pgpdb_close(keys);
   }
+  if (err != 0)
+    recreate_pubring = 1;
+  err = -1;
 
+  keys = pgpdb_open(PGPREMSECRING, NULL, 0, PGP_TYPE_PRIVATE);
+  if (keys == NULL)
+    goto end;
+  while (pgpdb_getnext(keys, key, NULL, userid) != -1) {
+    buf_clear(outtxt);
+    buf_clear(outkey);
+    buf_clear(keybak);
+    buf_cat(keybak, key);
+    if (pgp_makekeyheader(PGP_SECKEY, key, outtxt, pass, PGP_ANY) == 0) {
+      err = 0;
+      buf_appends(secout, "Type Bits/KeyID     Date       User ID\n");
+      buf_cat(secout, outtxt);
+      buf_nl(secout);
+      pgp_armor(key, PGP_ARMOR_SECKEY);
+      buf_cat(secout, key);
+      buf_nl(secout);
+    }
+    buf_clear(outtxt);
+    if (recreate_pubring &&
+	pgp_makepubkey(keybak, outtxt, outkey, pass, PGP_ANY) == 0) {
+      buf_appends(out, "Type Bits/KeyID     Date       User ID\n");
+      buf_cat(out, outtxt);
+      buf_nl(out);
+      pgp_armor(outkey, PGP_ARMOR_KEY);
+      buf_cat(out, outkey);
+      buf_nl(out);
+    }
+  }
+  pgpdb_close(keys);
+
+  seclock = lockfile(PGPREMSECRING);
+  if (err == 0 && (f = mix_openfile(PGPREMSECRING, "w")) != NULL) {
+    buf_write(secout, f);
+    fclose(f);
+  } else
+    err = -1;
+  unlockfile(seclock);
   if (err == 0 && (f = mix_openfile(PGPKEY, "w")) != NULL) {
     buf_write(out, f);
     fclose(f);
@@ -384,11 +465,64 @@ int pgp_keymgt(int force)
     err = -1;
 end:
   buf_free(key);
+  buf_free(keybak);
   buf_free(out);
   buf_free(userid);
   buf_free(pass);
   buf_free(outtxt);
   buf_free(outkey);
+  buf_free(secout);
+  return (err);
+}
+
+int pgp_latestkeys(BUFFER* outtxt, int algo)
+/* returns our latest key from pgpkey.txt in the buffer outtxt
+ * with pgp key header, ascii armored
+ *
+ * Can probably be extended to do this for all keys if we pass
+ * the keyring file and the userid
+ *
+ * IN:  algo: PGP_ANY, PGP_ES_RSA, PGP_E_ELG, PGP_S_DSA
+ * OUT: outtxt
+ */
+{
+  int err = -1;
+  long expires_found = 0, expires;
+  BUFFER *key, *userid, *tmptxt;
+  KEYRING *keys;
+
+  key = buf_new();
+  userid = buf_new();
+  buf_sets(userid, REMAILERNAME);
+  tmptxt = buf_new();
+
+  keys = pgpdb_open(PGPKEY, NULL, 0, PGP_TYPE_PUBLIC);
+  if (keys != NULL) {
+    while (pgpdb_getnext(keys, key, NULL, userid) != -1) {
+      buf_clear(tmptxt);
+      if (pgp_makekeyheader(PGP_PUBKEY, key, tmptxt, NULL, algo) == 0) {
+	buf_rewind(key);
+	pgp_getkey(PK_VERIFY, algo, NULL, NULL, &expires, key, NULL, NULL, NULL, NULL);
+	if (expires == 0 || (expires_found <= expires)) {
+	  err = 0;
+	  buf_clear(outtxt);
+	  buf_appends(outtxt, "Type Bits/KeyID     Date       User ID\n");
+	  buf_cat(outtxt, tmptxt);
+	  buf_nl(outtxt);
+	  pgp_armor(key, PGP_ARMOR_KEY);
+	  buf_cat(outtxt, key);
+	  buf_nl(outtxt);
+	  expires_found = expires;
+	}
+      }
+    }
+    pgpdb_close(keys);
+  }
+
+  buf_free(key);
+  buf_free(userid);
+  buf_free(tmptxt);
+
   return (err);
 }
 
@@ -402,8 +536,8 @@ int pgp_rlist(REMAILER remailer[], int n)
   p = buf_new();
   for (i = 1; i < n; i++)
     pgpkey[i] = 0;
-  if (pgp_readkeyring(keyring, PGPREMPUBRING) == -1)
-    pgp_readkeyring(keyring, PGPREMPUBASC);
+  if (pgp_readkeyring(keyring, PGPREMPUBASC) == -1)
+    pgp_readkeyring(keyring, PGPREMPUBRING);
   while ((type = pgp_getpacket(keyring, p)) != -1)
     if (type == PGP_USERID)
       for (i = 1; i < n; i++)
@@ -422,7 +556,6 @@ int pgp_rkeylist(REMAILER remailer[], int keyid[], int n)
   BUFFER *userid;
   BUFFER *id;
   int i, err;
-  int mdc, sym;
 
   userid = buf_new();
   id = buf_new();
@@ -433,12 +566,11 @@ int pgp_rkeylist(REMAILER remailer[], int keyid[], int n)
 
     keyid[i]=0;
     if (remailer[i].flags.pgp) {
-      mdc = sym = 0;
       buf_clear(id);
-      err = pgpdb_getkey(PK_VERIFY, PGP_ANY, &sym, &mdc, NULL, userid, NULL, id, NULL, NULL);
+      err = pgpdb_getkey(PK_VERIFY, PGP_ANY, NULL, NULL, NULL, NULL, userid, NULL, id, NULL, NULL);
       if (id->length == 8) {
 	/* printf("%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x %s\n",
-	 id->data[0], id->data[1], id->data[2], id->data[3], id->data[4], id->data[5], id->data[6], id->data[7], id->data[8], remailer[i].addr); */
+	   id->data[0], id->data[1], id->data[2], id->data[3], id->data[4], id->data[5], id->data[6], id->data[7], id->data[8], remailer[i].addr); */
 	keyid[i] = (((((id->data[4] << 8) + id->data[5]) << 8) + id->data[6]) << 8) + id->data[7];
       }
     }
