@@ -1,4 +1,4 @@
-/* Mixmaster version 2.9  --  (C) 1999 - 2003 Anonymizer Inc. and others.
+/* Mixmaster version 3.0  --  (C) 1999 - 2006 Anonymizer Inc. and others.
 
    Mixmaster may be redistributed and modified under certain conditions.
    This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF
@@ -6,9 +6,9 @@
    details.
 
    Read OpenPGP packets
-   $Id: pgpget.c 665 2003-11-09 01:47:32Z rabbi $ */
+   $Id: pgpget.c 934 2006-06-24 13:40:39Z rabbi $ */
+
 
-#include "mix.h"
 #include "mix3.h"
 #ifdef USE_PGP
 #include "pgp.h"
@@ -264,7 +264,7 @@ int pgp_getsig(BUFFER *p, pgpsig *sig, char *pubring)
   if (hashalgo != PGP_H_MD5)
     goto end;
   hash = buf_geti(p);
-  if (pgpdb_getkey(PK_VERIFY, algo, NULL, NULL, sigkey, NULL, sig->userid, id,
+  if (pgpdb_getkey(PK_VERIFY, algo, NULL, NULL, NULL, sigkey, NULL, sig->userid, id,
 		   pubring, NULL) < 0) {
     sig->ok = PGP_SIGNKEY;
     if (sig->userid)
@@ -272,7 +272,6 @@ int pgp_getsig(BUFFER *p, pgpsig *sig, char *pubring)
     goto end;
   }
   switch (algo) {
-#ifdef USE_RSA
   case PGP_ES_RSA:
     mpi_get(p, i);
     if (pgp_rsa(i, sigkey, PK_VERIFY) == -1 ||
@@ -283,7 +282,6 @@ int pgp_getsig(BUFFER *p, pgpsig *sig, char *pubring)
       goto end;
     sig->ok = PGP_SIGVRFY;
     break;
-#endif /* USE_RSA */
   default:
     break;
   }
@@ -743,6 +741,7 @@ int pgp_getsessionkey(BUFFER *in, BUFFER *pass, char *secring)
   int i, csum = 0;
   int algo = 0;
   int err = -1;
+  long expires;
 
   out = buf_new();
   key = buf_new();
@@ -752,17 +751,20 @@ int pgp_getsessionkey(BUFFER *in, BUFFER *pass, char *secring)
     goto end;
   buf_get(in, keyid, 8);
   algo = buf_getc(in);
-  err = pgpdb_getkey(PK_DECRYPT, algo, NULL, NULL, key, NULL, NULL, keyid,
+  err = pgpdb_getkey(PK_DECRYPT, algo, NULL, NULL, &expires, key, NULL, NULL, keyid,
 		     secring, pass);
   if (err < 0)
     goto end;
+  if (expires > 0 && (expires + KEYGRACEPERIOD < time(NULL))) {
+    errlog(DEBUGINFO, "Key expired.\n"); /* DEBUGINFO ? */
+    err = -1;
+    goto end;
+  }
   switch (algo) {
-#ifdef USE_RSA
   case PGP_ES_RSA:
     mpi_get(in, out);
     err = pgp_rsa(out, key, PK_DECRYPT);
     break;
-#endif /* USE_RSA */
    case PGP_E_ELG:
     buf_rest(out, in);
     err = pgp_elgdecrypt(out, key);
