@@ -1,4 +1,4 @@
-/* Mixmaster version 2.9  --  (C) 1999 - 2003 Anonymizer Inc. and others.
+/* Mixmaster version 3.0  --  (C) 1999 - 2006 Anonymizer Inc. and others.
 
    Mixmaster may be redistributed and modified under certain conditions.
    This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF
@@ -6,7 +6,7 @@
    details.
 
    Send messages from pool
-   $Id: pool.c 665 2003-11-09 01:47:32Z rabbi $ */
+   $Id: pool.c 934 2006-06-24 13:40:39Z rabbi $ */
 
 #include "mix3.h"
 #include <stdlib.h>
@@ -19,10 +19,12 @@
 #else /* end of POSIX */
 #include <io.h>
 #endif /* else if not POSIX */
-#ifndef _MSC_VER
+#ifndef _MSC
 #include <dirent.h>
-#endif /* not _MSC_VER */
+#endif /* not _MSC */
 #include <assert.h>
+#include <sys/stat.h>
+#include <sys/types.h>
 
 #ifdef USE_PCRE
 #include "pcre.h"
@@ -332,6 +334,8 @@ int pool_add(BUFFER *msg, char *type)
 FILE *pool_new(char *type, char *tmpname, char *path)
 {
   FILE *f;
+  struct stat buf;
+  int err;
 
   assert(strlen(type) == 3);
 #ifdef SHORTNAMES
@@ -346,6 +350,9 @@ FILE *pool_new(char *type, char *tmpname, char *path)
   strcpy(path, tmpname);
   strrchr(path, DIRSEP)[1] = type[0];
 #endif /* else if not SHORTNAMES */
+  err = stat(tmpname, &buf);
+  if (err == 0)
+    errlog(WARNING, "Overwriting file %s\n", tmpname);
   f = fopen(tmpname, "wb");
   if (f == NULL)
     errlog(ERRORMSG, "Error creating temporary file %s\n", tmpname);
@@ -411,12 +418,24 @@ int process_mailin()
   return(0);
 }
 
+int create_dummy_mailout()
+{
+  while (rnd_number(100) < OUTDUMMYP) {
+    errlog(DEBUGINFO, "Generating dummy message with outgoing mail.\n");
+    if (mix_encrypt(MSG_NULL, NULL, NULL, 1, NULL) == -1)
+      return -1;
+  }
+  return 0;
+}
+
 int pool_send(void)
 {
   int size, max, i, r;
   BUFFER *pool;
   long int *ptr;
 
+  create_dummy_mailout();
+
   latent_read();
   pool = buf_new();
   size = pool_read(pool);
@@ -478,13 +497,15 @@ int msg_send(char *name)
     if (err == -1)
       goto end;
     err = sendmail(m, REMAILERADDR, addr);
+    stats_log(3);
   } else if (type == MSG_MAIL || type == MSG_POST) {
     err = buf_read(m, f);
     if (err == -1)
       goto end;
-    if (MIDDLEMAN && ! allowmessage(m))
-      mix2_encrypt(type, m, FORWARDTO, 1, NULL);
-    else {
+    if (MIDDLEMAN && ! allowmessage(m)) {
+      mix2_encrypt(type, m, FORWARDTO, 1, 1, NULL);
+      stats_log(6);
+    } else {
       err = filtermsg(m);
       if (err == 1)
 	userfrom = 1, err = 0;
@@ -492,9 +513,10 @@ int msg_send(char *name)
 	/* message has recipients */
 	errlog(DEBUGINFO, "Sending message (%ld bytes)\n", m->length);
 
-	if (type == MSG_MAIL)
+	if (type == MSG_MAIL) {
 	  err = sendmail(m, userfrom ? NULL : ANONNAME, NULL);
-	else if (type == MSG_POST) {
+	  stats_log(4);
+	} else if (type == MSG_POST) {
 	  if (strchr(NEWS, '@') && !strchr(NEWS, ' ')) {
 	    errlog(LOG, "Mailing article to %s.\n", NEWS);
 	    buf_sets(addr, NEWS);
@@ -514,6 +536,7 @@ int msg_send(char *name)
 	    closepipe(f);
 	  } else
 	    errlog(NOTICE, "Rejecting news article.\n");
+	  stats_log(5);
 	}
       } else
 	errlog(ERRORMSG, "Bad message file.\n");
@@ -534,11 +557,12 @@ int allowmessage(BUFFER *in)
  * addresses are in dest.allow. If yes return 1; 0 otherwhise
  */
 {
-  BUFFER *out, *allow, *line, *line2;
+  BUFFER *out, *allow, *allow2, *line, *line2;
   int err=1;
   FILE *f;
 
   allow = buf_new();
+  allow2 = buf_new();
   out = buf_new();
   line = buf_new();
   line2 = buf_new();
@@ -548,6 +572,12 @@ int allowmessage(BUFFER *in)
     buf_read(allow, f);
     fclose(f);
   }
+  f = mix_openfile(DESTALLOW2, "r");
+  if (f != NULL) {
+    buf_read(allow2, f);
+    fclose(f);
+    buf_cat(allow, allow2);
+  }
 
   /* Do header lines */
   while (buf_getline(in, line) == 0) {
@@ -579,6 +609,7 @@ int allowmessage(BUFFER *in)
   buf_move(in, out);
   buf_free(out);
   buf_free(allow);
+  buf_free(allow2);
   buf_free(line);
   buf_free(line2);
   return (err);
