@@ -1,4 +1,4 @@
-/* Mixmaster version 2.9  --  (C) 1999 - 2003 Anonymizer Inc. and others.
+/* Mixmaster version 3.0  --  (C) 1999 - 2006 Anonymizer Inc. and others.
 
    Mixmaster may be redistributed and modified under certain conditions.
    This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF
@@ -6,7 +6,8 @@
    details.
 
    Randomness
-   $Id: random.c 665 2003-11-09 01:47:32Z rabbi $ */
+   $Id: random.c 934 2006-06-24 13:40:39Z rabbi $ */
+
 
 #include "mix3.h"
 #include "crypto.h"
@@ -19,7 +20,7 @@
 #include <process.h>
 #endif /* else if not POSIX */
 #ifdef WIN32
-/* #include <windows.h> */
+#include <windows.h>
 #endif /* WIN32 */
 #include <assert.h>
 #include <string.h>
@@ -102,9 +103,7 @@ void rnd_time(void)
 void rnd_update(byte *seed, int l)
 {
   int fd = -1;
-  #ifdef DEV_URANDOM
   byte b[512];
-  #endif /* DEV_URANDOM */
 
   rnd_time();
   if (seed)
