@@ -1,4 +1,4 @@
-/* Mixmaster version 2.9  --  (C) 1999 - 2003 Anonymizer Inc. and others.
+/* Mixmaster version 3.0  --  (C) 1999 - 2006 Anonymizer Inc. and others.
 
    Mixmaster may be redistributed and modified under certain conditions.
    This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF
@@ -6,7 +6,7 @@
    details.
 
    Process remailer messages
-   $Id: rem.c 665 2003-11-09 01:47:32Z rabbi $ */
+   $Id: rem.c 934 2006-06-24 13:40:39Z rabbi $ */
 
 
 #include "mix3.h"
@@ -21,29 +21,108 @@
 #else /* end of POSIX */
 #include <io.h>
 #endif /* else if not POSIX */
-#ifndef _MSC_VER
+#ifndef _MSC
 #include <dirent.h>
-#endif /* not _MSC_VER */
+#endif /* not _MSC */
 #include <assert.h>
 
 int blockrequest(BUFFER *message);
+int create_dummy_mailin();
 
 #define REQUESTHELP  100
 #define REQUESTSTATS 101
 #define REQUESTKEY   200
 #define REQUESTCONF  201
 #define REQUESTOPKEY 202
+#define REQUESTOTHER 203
 #define BLOCKREQUEST 666
 #define DISABLED      99
 
 #define CPUNKMSG 1
 #define MIXMSG 2
 
+
+/** \brief get replies for additional information requests
+ *
+ * \param reply The buffer to store the reply in
+ * \param file	The file or name of information a user requested
+ * \returns 0 on success; -1 if a ressource has a valid name
+ *     but doesn't exist; 1 if the ressource name isn't valid.
+ * 
+ * This function returns additional information that a
+ * user may have requested.  One obvious example is help files
+ * in different languages.  We don't want to hack the source every
+ * time we or somebody else adds a new language.
+ *
+ * Therefore we add a new directory where the operator may
+ * just create new files (say "remailer-help-de"). If a user
+ * requests that using this (file|ressource) name in the
+ * subject line we respond by sending it.
+ *
+ * Perhaps we should build something that returns an index of
+ * available files (FIXME if done).
+ *
+ * A ressource name needs to start with the string "remailer-"
+ * and must only consist of alphanumerical characters and dashes.
+ * Checking is done by this function and an error returned
+ * if this is violated.
+ */
+int get_otherrequests_reply(BUFFER *reply, BUFFER *filename)
+{
+  FILE *f = NULL;
+  int c;
+  int err;
+  BUFFER *path;
+
+  path = buf_new();
+
+  assert(filename);
+  assert(reply);
+
+  buf_rewind(filename);
+  err = bufileft(filename, "remailer-");
+  if (! err) {
+    err = 1;
+    goto end;
+  };
+
+  while ((c = buf_getc(filename)) != -1) {
+    int ok = (c >= 'A' && c <= 'Z') ||
+	     (c >= 'a' && c <= 'z') ||
+	     (c >= '0' && c <= '9') ||
+	     c == '-';
+    if (!ok) {
+      err = 1;
+      goto end;
+    };
+  };
+  buf_rewind(filename);
+
+  buf_appends(path, REQUESTDIR);
+  buf_appends(path, "/");
+  buf_cat(path, filename);
+
+  f = mix_openfile(path->data, "r");
+  if (f == NULL) {
+    err = -1;
+    goto end;
+  };
+
+  buf_read(reply, f);
+  err = 0;
+end:
+  if (f)
+    fclose(f);
+  buf_free(path);
+  return (err);
+}
+
 int mix_decrypt(BUFFER *message)
 {
   int type = 0;
   BUFFER *field, *content;
   BUFFER *to, *subject, *replyto, *reply;
+  BUFFER *otherrequest;
   FILE *f;
   BUFFER *block;
   int err = 0;
@@ -57,6 +136,7 @@ int mix_decrypt(BUFFER *message)
   reply = buf_new();
   block = buf_new();
   subject = buf_new();
+  otherrequest = buf_new();
   buf_sets(subject, "Subject: Re: your mail");
 
   buf_rewind(message);
@@ -105,7 +185,10 @@ int mix_decrypt(BUFFER *message)
 	type = REQUESTOPKEY;
       else if (bufieq(content, "remailer-conf"))
 	type = REQUESTCONF;
-      else if (bufileft(content, "destination-block"))
+      else if (bufileft(content, "remailer-")) {
+	type = REQUESTOTHER;
+	buf_set(otherrequest, content);
+      } else if (bufileft(content, "destination-block"))
 	type = BLOCKREQUEST;
       else {
 	buf_sets(subject, "Subject: ");
@@ -158,13 +241,19 @@ hdrend:
     if (err == 0)
       err = sendmail(reply, REMAILERNAME, replyto);
     break;
+  case REQUESTOTHER:
+    err = get_otherrequests_reply(reply, otherrequest);
+    if (err == 0)
+      err = sendmail(reply, REMAILERNAME, replyto);
+    break;
   case CPUNKMSG:
     err = t1_decrypt(message);
     if (err != 0) {
       errlog(LOG, "Invalid type 1 message from %b\n", replyto);
       sendinfofile(USAGEFILE, USAGELOG, replyto, NULL);
       logmail(err == -2 ? MAILUSAGE : MAILERROR, message);
-    }
+    } else
+      create_dummy_mailin();
     break;
   case MIXMSG:
     err = t2_decrypt(message);
@@ -172,10 +261,12 @@ hdrend:
       errlog(LOG, "Invalid type 2 message from %b\n", replyto);
       sendinfofile(USAGEFILE, USAGELOG, replyto, NULL);
       logmail(MAILERROR, message);
-    }
+    } else
+      create_dummy_mailin();
     break;
   case BLOCKREQUEST:
     blockrequest(message);
+    /* Already wrote a log entry in blockrequest() */
     logmail(MAILBLOCK, message);
     break;
   case DISABLED:
@@ -189,21 +280,23 @@ hdrend:
 	(replyto->data, "mailer-daemon")) {
       errlog(LOG, "Bounce mail from %b\n", replyto);
       logmail(MAILBOUNCE, message);
-    } else if (bufifind(to, REMAILERADDR) && blockrequest(message))
+    } else if (bufifind(to, REMAILERADDR) && blockrequest(message)) {
+      /* Already wrote a log entry in blockrequest() */
       logmail(MAILBLOCK, message);
-    else if (!AUTOREPLY)
-      logmail(MAILBOX, message);
-    else if (bufifind(to, REMAILERADDR)) {
+    } else if (bufifind(to, REMAILERADDR)) {
       errlog(LOG, "Non-remailer message from %b\n", replyto);
-      sendinfofile(USAGEFILE, USAGELOG, replyto, NULL);
+      if (AUTOREPLY)
+	sendinfofile(USAGEFILE, USAGELOG, replyto, NULL);
       logmail(MAILUSAGE, message);
     } else if (bufifind(to, COMPLAINTS)) {
       errlog(WARNING, "Abuse complaint from %b\n", replyto);
-      sendinfofile(ABUSEFILE, NULL, replyto, subject);
+      if (AUTOREPLY)
+	sendinfofile(ABUSEFILE, NULL, replyto, subject);
       logmail(MAILABUSE, message);
     } else if (ANONADDR[0] && bufifind(to, ANONADDR)) {
       errlog(LOG, "Reply to anonymous message from %b\n", replyto);
-      sendinfofile(REPLYFILE, NULL, replyto, subject);
+      if (AUTOREPLY)
+	sendinfofile(REPLYFILE, NULL, replyto, subject);
       logmail(MAILANON, message);
     } else {
       errlog(DEBUGINFO, "Mail from %b\n", replyto);
@@ -219,9 +312,20 @@ end:
   buf_free(reply);
   buf_free(block);
   buf_free(subject);
+  buf_free(otherrequest);
   return (err);
 }
 
+int create_dummy_mailin()
+{
+  while (rnd_number(100) < INDUMMYP) {
+    errlog(DEBUGINFO, "Generating dummy message with incoming mail.\n");
+    if (mix_encrypt(MSG_NULL, NULL, NULL, 1, NULL) == -1)
+      return -1;
+  }
+  return 0;
+}
+
 int t2_decrypt(BUFFER *in)
 {
   int err = 0;
@@ -231,10 +335,7 @@ int t2_decrypt(BUFFER *in)
   do {
     err = mix_dearmor(in, msg);
     if (err != -1) {
-      if (v3_magic(msg->data))
-	err = mix3_decrypt(msg);
-      else
-	err = mix2_decrypt(msg);
+      err = mix2_decrypt(msg);
     }
   }
   while (in->ptr + 1000 < in->length);	/* accept several packets in one message */
@@ -327,7 +428,7 @@ void logmail(char *mailbox, BUFFER *message)
   struct tm *tc;
   char line[LINELEN];
 
-  /* mailbox is "|program", "user@host", "stdout" or "filename" */
+  /* mailbox is "|program", "user@host", "stdout", "Maildir/" or "filename" */
   buf_rewind(message);
   if (mailbox[0] == '\0')	/* default action */
     mailbox = MAILBOX;
@@ -357,6 +458,12 @@ void logmail(char *mailbox, BUFFER *message)
   isloop:
     buf_free(field);
     buf_free(content);
+  } else if (mailbox[strlen(mailbox)-1] == DIRSEP) {
+    /* the user is requesting Maildir delivery */
+    if(maildirWrite(mailbox, message, 1) != 0) {
+      errlog(ERRORMSG, "Can't write to maildir %s\n", mailbox);
+      return;
+    }
   } else {
     FILE *mbox;
 
@@ -441,7 +548,7 @@ int blockrequest(BUFFER *message)
 	goto end;
       }
       /* Check if some evil person tries to block a known type II remailer */
-      num = mix2_rlist(remailer);
+      num = mix2_rlist(remailer, NULL);
       for (i = 0; i < num; i++) {
 	buf_sets(remailer_addr, remailer[i].addr);
 	if (doblock(remailer_addr, copy_addr, 1)) {
@@ -451,7 +558,7 @@ int blockrequest(BUFFER *message)
 	}
       }
       /* Check if some evil person tries to block a known type I remailer */
-      num = t1_rlist(remailer);
+      num = t1_rlist(remailer, NULL);
       for (i = 0; i < num; i++) {
 	buf_sets(remailer_addr, remailer[i].addr);
 	if (doblock(remailer_addr, copy_addr, 1)) {
@@ -515,7 +622,6 @@ end:
 int idexp(void)
 {
   FILE *f;
-  BUFFER *b;
   long now, then;
   LOCK *i;
   idlog_t idbuf;
@@ -524,7 +630,6 @@ int idexp(void)
   if (IDEXP == 0)
     return (0);
 
-  b = buf_new();
   f = mix_openfile(IDLOG, "rb+");
   if (f == NULL)
     return (-1);
@@ -556,11 +661,47 @@ int idexp(void)
       fseek(f,fpi,SEEK_SET);
     }
   }
-#ifdef _MSC_VER
+#ifdef _MSC
     chsize(fileno(f),fpo);
-#else /* end of _MSC_VER */
+#else /* end of _MSC */
     ftruncate(fileno(f),fpo);
-#endif /* else if not _MSC_VER */
+#endif /* else if not _MSC */
+  fclose(f);
+  unlockfile(i);
+  return (0);
+}
+
+
+int pgpmaxexp(void)
+{
+  FILE *f;
+  BUFFER *b;
+  long now, then;
+  LOCK *i;
+  char temp[LINELEN];
+
+  f = mix_openfile(PGPMAXCOUNT, "rb+");
+  if (f == NULL)
+    return (-1);
+  i = lockfile(PGPMAXCOUNT);
+  b = buf_new();
+  now = time(NULL);
+
+  while (fgets(temp, sizeof(temp), f) != NULL)
+    if (sscanf(temp, "%ld", &then) &&
+	then >= now - SECONDSPERDAY)
+      buf_appends(b, temp);
+
+  fseek(f,0,SEEK_SET);
+
+  buf_write(b, f);
+
+#ifdef _MSC
+    chsize(fileno(f),b->length);
+#else /* end of _MSC */
+    ftruncate(fileno(f),b->length);
+#endif /* else if not _MSC */
+
   fclose(f);
   unlockfile(i);
   buf_free(b);
