@@ -1,4 +1,4 @@
-/* Mixmaster version 2.9  --  (C) 1999 - 2003 Anonymizer Inc. and others.
+/* Mixmaster version 3.0  --  (C) 1999 - 2006 Anonymizer Inc. and others.
 
    Mixmaster may be redistributed and modified under certain conditions.
    This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF
@@ -6,17 +6,15 @@
    details.
 
    Process Cypherpunk remailer messages
-   $Id: rem1.c 665 2003-11-09 01:47:32Z rabbi $ */
+   $Id: rem1.c 934 2006-06-24 13:40:39Z rabbi $ */
 
 
-#include "mix.h"
 #include "mix3.h"
 #include <ctype.h>
 #include <time.h>
 #include <string.h>
-#ifdef USE_PGP
-#	include "pgp.h"
-#endif
+#include <assert.h>
+#include <stdlib.h>
 
 static int t1msg(BUFFER *in, int hdr);
 
@@ -92,6 +90,51 @@ static int readnum(BUFFER *b, int f)
   return (num);
 }
 
+static int readdate(BUFFER *b)
+{
+  int num = -1;
+
+  if (b->length > 0)
+    num = parsedate(b->data);
+  return (num);
+}
+
+static int reached_maxcount(BUFFER *md, int maxcount)
+{
+  FILE *f;
+  char temp[LINELEN];
+  int count = 0;
+  int err = 0;
+  long then;
+  time_t now = time(NULL);
+
+  assert(md->length > 0);
+
+  encode(md, 0);
+
+  f = mix_openfile(PGPMAXCOUNT, "a+"); /* create file if it does not exist */
+  fseek(f,0,SEEK_SET);
+  if (f == NULL) {
+    errlog(ERRORMSG, "Can't open %s!\n", PGPMAXCOUNT);
+    return (-1);
+  }
+  lock(f);
+  while (fgets(temp, sizeof(temp), f) != NULL)
+    if (sscanf(temp, "%ld", &then) &&
+	(then >= now - SECONDSPERDAY) &&
+	strstr (temp, md->data))
+      count++;
+
+  if (count > maxcount)
+    err = 1;
+  else
+    fprintf(f, "%ld %s\n", (long) time(NULL), md->data);
+
+  unlock(f);
+  fclose(f);
+  return (err);
+}
+
 static int t1msg(BUFFER *in, int hdr)
      /* hdr = 1: mail header, hdr = 2: pasted header, hdr = 0: ignore */
 {
@@ -99,12 +142,16 @@ static int t1msg(BUFFER *in, int hdr)
   BUFFER *cutmarks, *to, *newsgroups, *ek, *ekdes, *ekcast, *esub, *subject;
   BUFFER *temp, *header, *out;
   BUFFER *test, *testto, *remixto;
+  BUFFER *digest;
   int err = 0;
   int encrypted = 0;
   int type = -1;
   int latent = 0;
   int remix = 0, repgp = 0;
   int inflate = 0;
+  int maxsize = -1;
+  int maxcount = -1;
+  int maxdate = -2; /* -2 not used, -1 parse error */
 
   field = buf_new();
   content = buf_new();
@@ -123,6 +170,7 @@ static int t1msg(BUFFER *in, int hdr)
   out = buf_new();
   test = buf_new();
   testto = buf_new();
+  digest = buf_new();
 
   if (REMIX == 1)
     remix = 2;
@@ -193,8 +241,13 @@ header:
 	latent = rnd_number(latent);
     } else if (bufieq(field, "null"))
       type = MSG_NULL;
+#ifdef USE_IDEA
     else if (bufieq(field, "encrypt-key") || bufieq(field, "encrypt-idea"))
       buf_set(ek, content);
+#else
+    else if (bufieq(field, "encrypt-key") || bufieq(field, "encrypt-idea"))
+      buf_set(ekdes, content);
+#endif
     else if (bufieq(field, "encrypt-des") || bufieq(field, "encrypt-3des"))
       buf_set(ekdes, content);
     else if (bufieq(field, "encrypt-cast") || bufieq(field, "encrypt-cast5"))
@@ -222,7 +275,12 @@ header:
 	buf_appendc(remixto, ',');
 	buf_cat(remixto, temp);
       }
-    }
+    } else if (bufieq(field, "max-size") || bufieq(field, "maxsize"))
+      maxsize = readnum(content, 1024);
+    else if (bufieq(field, "max-count") || bufieq(field, "maxcount"))
+      maxcount = readnum(content, 1);
+    else if (bufieq(field, "max-date") || bufieq(field, "maxdate"))
+      maxdate = readdate(content);
 #if USE_NSUB
     else if (bufieq(field, "subject"))
       buf_set(subject, content);
@@ -257,6 +315,7 @@ header:
     err = pgp_dearmor(in, temp);
     if (err == 0) {
       BUFFER *pass;
+      digest_sha1(temp, digest);
 
       pass = buf_new();
       buf_sets(pass, PASSPHRASE);
@@ -311,6 +370,43 @@ header:
     err = -2;
     goto end;
   }
+  if (maxdate == -1) {
+    if (testto->length == 0)
+      errlog(LOG, "Could not parse Max-Date: header.\n");
+    buf_appends(test, "Could not parse Max-Date: header.\n");
+    err = -2;
+    goto end;
+  } else if (maxdate >= 0 && maxdate <= time(NULL)) {
+    if (testto->length == 0)
+      errlog(LOG, "Message is expired.\n");
+    buf_appends(test, "Message is expired.\n");
+    err = -2;
+    goto end;
+  }
+  if (maxsize >= 0 && in->length >= maxsize) {
+    if (testto->length == 0)
+      errlog(LOG, "Message Size exceeds Max-Size.\n");
+    buf_appends(test, "Message Size exceeds Max-Size.\n");
+    err = -2;
+    goto end;
+  }
+  if (maxcount >= 0) {
+    if (digest->length == 0) {
+      if (testto->length == 0)
+	errlog(LOG, "Max-Count yet not encrypted.\n");
+      buf_appends(test, "Max-Count yet not encrypted.\n");
+      err = -2;
+      goto end;
+    }
+    if (reached_maxcount(digest, maxcount)) {
+      if (testto->length == 0)
+	errlog(LOG, "Max-Count reached - discarding message.\n");
+      buf_appends(test, "Max-Count reached - discarding message.\n");
+      err = -2;
+      goto end;
+    }
+  }
+
   if (type == MSG_POST && subject->length == 0)
     buf_sets(subject, "(no subject)");
 
@@ -453,7 +549,19 @@ end:
       remix = 0;
     if (remix && remixto->length == 0)
       buf_set(remixto, to);
-    if (remix && !repgp)
+    if (remixto->length > 0) {
+      /* check that the remix-to path isn't too long */
+      int remixcount = 1;
+      char *tmp = remixto->data;
+      while ((tmp = strchr(tmp+1, ','))) {
+	remixcount ++;
+	if (remixcount > MAXRANDHOPS) {
+	  *tmp = '\0';
+	  break;
+	}
+      };
+    }
+    if (remix && !repgp && remixto->length != 0)
       err = mix_encrypt(type, out, remixto->data, 1, line);
     if (err != 0) {
       if (remix == 1 && !repgp)
@@ -486,5 +594,6 @@ end:
   buf_free(header);
   buf_free(test);
   buf_free(testto);
+  buf_free(digest);
   return (err);
 }
