@@ -1,4 +1,4 @@
-/* Mixmaster version 2.9  --  (C) 1999 - 2003 Anonymizer Inc. and others.
+/* Mixmaster version 3.0  --  (C) 1999 - 2006 Anonymizer Inc. and others.
 
    Mixmaster may be redistributed and modified under certain conditions.
    This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF
@@ -6,7 +6,7 @@
    details.
 
    Process Mixmaster remailer messages
-   $Id: rem2.c 665 2003-11-09 01:47:32Z rabbi $ */
+   $Id: rem2.c 934 2006-06-24 13:40:39Z rabbi $ */
 
 
 #include "mix3.h"
@@ -20,9 +20,9 @@
 #else /* end of POSIX */
 #include <io.h>
 #endif /* else if not POSIX */
-#ifndef _MSC_VER
+#ifndef _MSC
 #include <dirent.h>
-#endif /* not _MSC_VER */
+#endif /* not _MSC */
 #include <assert.h>
 
 int mix_dearmor(BUFFER *in, BUFFER *out)
@@ -78,9 +78,12 @@ end:
   return (err);
 }
 
-#ifdef USE_RSA
-
 static int isnewid(BUFFER *id, long timestamp)
+/* return values:
+ *   0: ignore message, no error
+ *   1: ok, process message
+ *  -1: bad message, send reply
+ */
 {
   FILE *f;
   int ret = 1;
@@ -130,11 +133,18 @@ static int isnewid(BUFFER *id, long timestamp)
   i = lockfile(IDLOG);
   while (fread(&idbuf, 1, sizeof(idlog_t), f) == sizeof(idlog_t)) {
     if (!memcmp(idbuf.id, id->data, sizeof(idbuf.id))) {
-      errlog(LOG, "Ignoring redundant message.\n");
+      char idstr[33];
+      id_encode(id->data, idstr);
+      errlog(LOG, "Ignoring redundant message: %s.\n", idstr);
       ret = 0;
       goto end;
     }
   }
+  if (timestamp > now) {
+    errlog(LOG, "Ignoring message with future timestamp.\n");
+    ret = -1;
+    goto end;
+  }
   if (ftell(f)%sizeof(idlog_t)) fseek(f,0-(ftell(f)%sizeof(idlog_t)),SEEK_CUR); /* make sure that we're on sizeof(idlog_t) byte boundary */
   memcpy(idbuf.id,id->data,sizeof(idbuf.id));
   idbuf.time = now;
@@ -147,7 +157,9 @@ end:
 }
 
 int mix2_decrypt(BUFFER *m)
-     /* 0: ok  -1: error  -2: old message */
+     /*  0: ok
+      * -1: error
+      * -2: old message */
 {
   int err = 0;
   int i;
@@ -216,6 +228,10 @@ int mix2_decrypt(BUFFER *m)
       dec->data[dec->ptr + 4] == '\0') {
     dec->ptr += 5;
     timestamp = buf_geti_lo(dec);
+  } else {
+    errlog(LOG, "Ignoring message without timestamp.\n");
+    err = -1;
+    goto end;
   }
   buf_get(dec, digest, 16);
 
@@ -227,9 +243,11 @@ int mix2_decrypt(BUFFER *m)
     err = -1;
     goto end;
   }
-  if (!isnewid(packetid, timestamp * SECONDSPERDAY)) {
-    err = -2;
-    goto end;
+  switch (isnewid(packetid, timestamp * SECONDSPERDAY)) {
+    case  0: err = -2; /* redundant message */
+	     goto end;
+    case -1: err = -1; /* future timestamp */
+	     goto end; 
   }
   buf_append(body, m->data + 20 * 512, 10240);
 
@@ -466,11 +484,3 @@ int v2partial(BUFFER *m, BUFFER *mid, int packet, int numpackets)
 end:
   return (err);
 }
-
-#else /* end of USE_RSA */
-
-int mix2_decrypt(BUFFER *m)
-{
-  return (-1);
-}
-#endif /* else if not USE_RSA */
