@@ -1,16 +1,14 @@
-/* Mixmaster version 2.9  --  (C) 1999 - 2003 Anonymizer Inc. and others.
+/* Mixmaster version 3.0  --  (C) 1999 - 2006 Anonymizer Inc. and others.
 
    Mixmaster may be redistributed and modified under certain conditions.
    This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF
    ANY KIND, either express or implied. See the file COPYRIGHT for
    details.
 
-   WIN32 Service support.
-   $Id: service.c 665 2003-11-09 01:47:32Z rabbi $ */
+   Win32 Service support
+   $Id: service.c 934 2006-06-24 13:40:39Z rabbi $ */
 
 
-#ifdef WIN32SERVICE
-
 #include <windows.h>
 #include <stdio.h>
 #include <direct.h>
@@ -19,6 +17,8 @@
 
 #include "mix3.h"
 
+#ifdef WIN32SERVICE
+
 #define SVCNAME        "Mixmaster"
 #define SVCDISPLAYNAME "Mixmaster Service"
 
@@ -280,7 +280,7 @@ static int remove_service()
     if (ControlService(schService, SERVICE_CONTROL_STOP, &ssStatus)) {
 	printf("Stopping Mixmaster Service");
 	do {
-	    Sleep(1000);
+	    sleep(1);
 	    printf(".");
 	    QueryServiceStatus(schService, &ssStatus);
 	} while (ssStatus.dwCurrentState != SERVICE_STOP_PENDING);
