@@ -1,4 +1,4 @@
-/* Mixmaster version 2.9  --  (C) 1999 - 2003 Anonymizer Inc. and others.
+/* Mixmaster version 3.0  --  (C) 1999 - 2006 Anonymizer Inc. and others.
 
    Mixmaster may be redistributed and modified under certain conditions.
    This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF
@@ -6,7 +6,7 @@
    details.
 
    Remailer statistics
-   $Id: stats.c 665 2003-11-09 01:47:32Z rabbi $ */
+   $Id: stats.c 934 2006-06-24 13:40:39Z rabbi $ */
 
 
 #include "mix3.h"
@@ -59,7 +59,16 @@ int stats(BUFFER *b)
   long now, today, then;
   time_t t;
   long updated = 0, havestats = 0;
-  int msgd[3][24], msg[3][80];
+  int msgd[7][24], msg[7][80];
+  /* 0 .. Unencrypted
+   * 1 .. Type I PGP
+   * 2 .. Mix
+   *
+   * 3 .. intermediate
+   * 4 .. final hop mail
+   * 5 .. final hop news
+   * 6 .. randhopped (will get counted in intermediate again)
+   */
   int poold[2][24], pool[2][80];
   int i, num, type, assigned, daysum;
   char c;
@@ -69,9 +78,9 @@ int stats(BUFFER *b)
   today = (now / SECONDSPERDAY) * SECONDSPERDAY;
 
   for (i = 0; i < 24; i++)
-    msgd[0][i] = msgd[1][i] = msgd[2][i] = poold[0][i] = poold[1][i] = 0;
+    msgd[0][i] = msgd[1][i] = msgd[2][i] = msgd[3][i] = msgd[4][i] = msgd[5][i] = msgd[6][i]= poold[0][i] = poold[1][i] = 0;
   for (i = 0; i < 80; i++)
-    msg[0][i] = msg[1][i] = msg[2][i] = pool[0][i] = pool[1][i] = 0;
+    msg[0][i] = msg[1][i] = msg[2][i] = msg[3][i] = msg[4][i] = msg[5][i] = msg[6][i] = pool[0][i] = pool[1][i] = 0;
 
   s = mix_openfile(STATS, "r");
   if (s != NULL) {
@@ -82,6 +91,10 @@ int stats(BUFFER *b)
       case '0':
       case '1':
       case '2':
+      case '3':
+      case '4':
+      case '5':
+      case '6':
 	c = '\0';
 	assigned = sscanf(line, "%d %d %ld %c", &type, &num, &then, &c);
 	daysum = (assigned == 4 && c == 'd');
@@ -146,14 +159,14 @@ int stats(BUFFER *b)
   lock(s);
   fprintf(s, "%ld\n", (long) time(NULL));	/* time of stats.log update */
   for (i = 0; i < 24; i++) {
-    for (type = 0; type < 3; type++)
+    for (type = 0; type < 7; type++)
       if (msgd[type][i] > 0)
 	fprintf(s, "%d %d %ld\n", type, msgd[type][i], now - i * 60 * 60);
     if (poold[0][i] > 0)
       fprintf(s, "p %d %d %ld\n", poold[0][i], poold[1][i], now - i * 60 * 60);
   }
   for (i = 0; i < 80; i++) {
-    for (type = 0; type < 3; type++)
+    for (type = 0; type < 7; type++)
       if (msg[type][i] > 0)
 	fprintf(s, "%d %d %ld d\n", type, msg[type][i],
 		today - i * 24 * 60 * 60);
@@ -210,6 +223,13 @@ int stats(BUFFER *b)
 	buf_appendf(b, "     PGP: %4d", msg[1][i]);
       if (UNENCRYPTED)
 	buf_appendf(b, "     Unencrypted:%4d", msg[0][i]);
+      if (STATSDETAILS) {
+	buf_appendf(b, "  Intermediate:%4d", msg[3][i]);
+	buf_appendf(b, "  Mail:%4d", msg[4][i]);
+	buf_appendf(b, "  Postings:%4d", msg[5][i]);
+	if (MIDDLEMAN)
+	  buf_appendf(b, "  Randhopped:%4d", msg[6][i]);
+      }
       if (pool[0][i] > 0)
 	buf_appendf(b, "  [Pool size:%4d]", pool[1][i] / pool[0][i]);
 #if 0
@@ -324,8 +344,8 @@ int conf(BUFFER *out)
 
   if (LISTSUPPORTED) {
    /* SUPPORTED CPUNK (TYPE I) REMAILERS
-    * 0xDC7532F9      "Heex Remailer <remailer@xmailer.ods.org>"
-    * 0x759ED311      "znar <ka5tkn@cox-internet.com>"
+    * 0xDC7532F9	"Heex Remailer <remailer@xmailer.ods.org>"
+    * 0x759ED311	"znar <ka5tkn@cox-internet.com>"
     *
     * SUPPORTED MIXMASTER (TYPE II) REMAILERS
     * aarg remailer@aarg.net 475f3f9fe8da22896c10082695a92c2d 2.9b33 C
@@ -333,19 +353,21 @@ int conf(BUFFER *out)
     */
 
     buf_nl(out);
+#ifdef USE_PGP
     if (PGP) {
       buf_appends(out, "SUPPORTED CPUNK (TYPE I) REMAILERS\n");
-      num = t1_rlist(remailer);
+      num = t1_rlist(remailer, NULL);
       pgp_rkeylist(remailer, pgpkeyid, num);
       for (i=1; i<=num; i++) {
 	if (remailer[i].flags.pgp) {
-	  snprintf(tmpline, LINELEN, "0x%08X    \"%s <%s>\"\n", pgpkeyid[i], remailer[i].name, remailer[i].addr);
+	  snprintf(tmpline, LINELEN, "0x%08X	\"%s <%s>\"\n", pgpkeyid[i], remailer[i].name, remailer[i].addr);
 	  tmpline[LINELEN-1] = '\0';
 	  buf_appends(out, tmpline);
 	}
       }
       buf_nl(out);
     }
+#endif /* USE_PGP */
     if (MIX) {
       buf_appends(out, "SUPPORTED MIXMASTER (TYPE II) REMAILERS\n");
       prepare_type2list(out);
@@ -367,7 +389,7 @@ void conf_premail(BUFFER *out)
   buf_appends(out, REMAILERADDR);
   buf_appendc(out, '>');
   if (PGP || UNENCRYPTED)
-    buf_appends(out, " cpunk");
+    buf_appends(out, " cpunk max");
   if (MIX)
     buf_appends(out, " mix");
   if (MIDDLEMAN)
@@ -414,5 +436,7 @@ void conf_premail(BUFFER *out)
     buf_appends(out, " post");
   if (SIZELIMIT)
     buf_appendf(out, " klen%d", SIZELIMIT);
+  if (EXTFLAGS[0])
+    buf_appendf(out, " %s", EXTFLAGS);
   buf_appends(out, "\";\n");
 }
