@@ -1,4 +1,4 @@
-/* Mixmaster version 2.9  --  (C) 1999 - 2003 Anonymizer Inc. and others.
+/* Mixmaster version 3.0  --  (C) 1999 - 2006 Anonymizer Inc. and others.
 
    Mixmaster may be redistributed and modified under certain conditions.
    This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF
@@ -6,8 +6,7 @@
    details.
 
    Utility functions
-   $Id: util.c 665 2003-11-09 01:47:32Z rabbi $ */
-
+   $Id: util.c 934 2006-06-24 13:40:39Z rabbi $ */
 
 #include "mix3.h"
 #include <stdlib.h>
@@ -112,12 +111,22 @@ void strcatn(char *dest, const char *src, int n)
 
 int mixfile(char *path, const char *name)
 {
+  char *h;
   assert(path != NULL && name != NULL);
 
-  if (name[0] == DIRSEP || (isalpha(name[0]) && name[1] == ':') || MIXDIR == NULL)
+#ifdef POSIX
+  if (name[0] == '~' && name[1] == DIRSEP && (h = getenv("HOME")) != NULL) {
+    strncpy(path, h, PATHMAX);
+    path[PATHMAX-1] = '\0';
+    strcatn(path, name + 1, PATHMAX);
+  } else
+#endif /* POSIX */
+  if (name[0] == DIRSEP || (isalpha(name[0]) && name[1] == ':') || MIXDIR == NULL) {
     strncpy(path, name, PATHMAX);
-  else {
+    path[PATHMAX-1] = '\0';
+  } else {
     strncpy(path, MIXDIR, PATHMAX);
+    path[PATHMAX-1] = '\0';
     strcatn(path, name, PATHMAX);
   }
   return (0);
@@ -138,16 +147,17 @@ FILE *openpipe(const char *prog)
 #ifdef POSIX
   p = popen(prog, "w");
 #endif /* POSIX */
-#ifdef _MSC_VER
+#ifdef _MSC
   p = _popen(prog, "w");
-#endif /* _MSC_VER */
+#endif /* _MSC */
 
   if (p == NULL)
     errlog(ERRORMSG, "Unable to open pipe to %s\n", prog);
   return p;
 }
 
-int file_to_out(const char *filename)
+int
+file_to_out(const char *filename)
 {
     int len;
     FILE *fp;
@@ -167,11 +177,11 @@ int closepipe(FILE *p)
 {
 #ifdef POSIX
   return (pclose(p));
-#elif defined(_MSC_VER) /* end of POSIX */
+#elif defined(_MSC) /* end of POSIX */
   return (_pclose(p));
-#else /* end of defined(_MSC_VER) */
+#else /* end of defined(_MSC) */
   return -1;
-#endif /* else if not defined(_MSC_VER), POSIX */
+#endif /* else if not defined(_MSC), POSIX */
 }
 
 /** Base 64 encoding ****************************************************/
@@ -440,10 +450,8 @@ int unlock(FILE *f)
 static int getuserpass(BUFFER *b, int mode)
 {
   char p[LINELEN];
-#ifdef UNIX
   int fd;
   int n;
-#endif
 
 #ifdef HAVE_TERMIOS
   struct termios attr;
@@ -598,6 +606,46 @@ int clear_pidfile(char *pidfile)
 #endif /* else if not POSIX */
 }
 
+time_t parse_yearmonthday(char* str)
+{
+  time_t date;
+  int day, month, year;
+
+  if (sscanf( str, "%d-%d-%d", &year, &month, &day) == 3 ) {
+    struct tm timestruct;
+    char *tz;
+
+    tz = getenv("TZ");
+#ifdef HAVE_SETENV
+    setenv("TZ", "GMT", 1);
+#else /* end of HAVE_SETENV */
+    putenv("TZ=GMT");
+#endif /* else if not HAVE_SETENV */
+    tzset();
+    memset(&timestruct, 0, sizeof(timestruct));
+    timestruct.tm_mday = day;
+    timestruct.tm_mon = month - 1;
+    timestruct.tm_year = year - 1900;
+    date = mktime(&timestruct);
+#ifdef HAVE_SETENV
+    if (tz)
+      setenv("TZ", tz, 1);
+    else
+      unsetenv("TZ");
+#else  /* end of HAVE_SETENV */
+    if (tz) {
+      char envstr[LINELEN];
+      snprintf(envstr, LINELEN, "TZ=%s", tz);
+      putenv(envstr);
+    } else
+      putenv("TZ=");
+#endif /* else if not HAVE_SETENV */
+    tzset();
+    return date;
+  } else
+    return -1;
+}
+
 /* functions missing on some systems *************************************/
 
 #ifdef __RSXNT__
@@ -608,7 +656,7 @@ int fileno(FILE *f)
 
 #endif /* __RSXNT__ */
 
-#ifdef _MSC_VER	/* Visual C lacks dirent */
+#ifdef _MSC	/* Visual C lacks dirent */
 
 DIR *opendir(const char *name)
 {
@@ -653,4 +701,4 @@ int closedir(DIR *dir)
   return (-1);
 }
 
-#endif /* _MSC_VER */
+#endif /* _MSC */
